#!/usr/bin/perl

use Date::Manip;

#
# This shell script is run by cron every hour to collect pages and visitor
# summary stats for all access_logs on this local server.
#
#
# Notes:
#	- Traffic summary logs are stored in same directory as access_log for each web site
#	- You can regnerate yesterday's stats by running this script with 'yesterday' as an argument
#
#
#
# Modified:  Kurt Gray -- Mon Feb 28 11:08:45 EST 2000     Added all sites to log generate list
#            Kurt Gray -- Tue Sep  5 18:09:31 EDT 2000     Added Linux.com, SourceForge, Themes
#            Kurt Gray -- Thu Oct 19 18:15:04 EDT 2000     Removed sitelist so that stats are keyed by directory names
#            Kurt Gray -- Thu Mar 15 11:54:26 EST 2001     Added rotateaccesslog option


# NOTE: No longer need to edit this script to add new sites: 
#       It automatically crawls through the /usr/apache/sites/ directory
#       Kurt -- Thu Oct 19 19:33:36 EDT 2000


# Read options from command line

    foreach $arg (@ARGV) {

	# Option to specify a site

	if ( $arg =~ /site=([\w\-\.]+)/ ) {
	    $sitename = $1;

	# Debug option shows what's happening without writing new logs
	# and a Ctrl+C keystroke will stop script and dumps stats gathered so far

	} elsif ( $arg =~ /debug/ ) {
	    $debugging = 1;
	    $dontsavelogs = 1;
	    $SIG{INT} = dumpStats;

	# Option to get stats for a specific date (date=yesterday is useful for regenerating yesterday's stats)

	} elsif ( $arg =~ /date=(\w+)/ ) {
	    $date_input = $1;

        # Number of lines to read in access_log before stopping (useful for debugging)

	} elsif ( $arg =~ /lines=(\d+)/ ) {
	    $lines_max = $1;

	# Specify a base directory web sites are listed under

	} elsif ( $arg =~ /sitesdir=([\w\/]+)/ ) {
	    $sitesdir = $1;

	# Option to start reading access_log at this line (debugging option, not implemented)

	} elsif ( $arg =~ /linestart=(\d+)/ ) {
	    $line_start = $1;

        # Option to run script wihtout saving logs (useful when debugging)

	} elsif ( $arg =~ /dontsavelogs/ ) {
	    $dontsavelogs = 1;

	# Option to write number_of_hits per IP column in visitors_(date).log

	} elsif ( $arg =~ /hitspervisitor/ ) {
	    $loghitspervisitor = 1;

	# Option to ignore pagecount gif and do regular hit parsing

	} elsif ( $arg =~ /nopagecountgif/ ) {
	    $nopagecountgif = 1;

	# Option to rotate access_log and restart web server just at around midnight

	} elsif ( $arg =~ /rotateaccesslogs?/ ) {
	    $rotateaccesslog = 1;

	# Warn about unparsed inputs
	} else {
	    print "ERROR: $0 does not understand argument: $arg \n";
	    exit;
	} # if arg

    }


    $sitesdir       ||= "/usr/local/apache/sites";

    if ( $date_input ) {
	$logdate = $date_input;
    } else {
	( $logdate, $dayofweek ) = split ',', &Date::Manip::UnixDate("today", '%Y%m%d,%a');
    }

    $accesslog_file   = 'access_log';
    $agentlog_file    = 'agent_log';

    $hour = `/bin/date +'%H'`;	# If it's after midnight, then redo yesterday's stats

    if (($date_input eq 'yesterday') || ($hour =~ '00')) {
	( $logdate, $dayofweek ) = split ',', &Date::Manip::UnixDate("yesterday", '%Y%m%d,%a');
	$accesslog_file = 'access_log.'. $dayofweek;
	$agentlog_file  = 'agent_log.' . $dayofweek;
    }

    $pages_logfile    = 'pages_'    . $logdate . '.log';
    $visitors_logfile = 'visitors_' . $logdate . '.log';
    $summary_logfile  = 'summary_'  . $logdate . '.log';


# Make a list of sites being monitored on this server according to subdirectories under logs

    if ( $sitename ) {
	$logpath = "$sitesdir/$sitename/logs";
	if ( $rotateaccesslog == 1 ) {
	    &rotateAccessLog;
	} else {
	    &gatherStats;
	    &saveStats;
	}
    } else {
	if (opendir (DIR, "$sitesdir")) {
	    @sitelist = readdir DIR;
	    close DIR;
	    foreach $sitename (sort @sitelist) {
		print "\nDEBUG: Looking at $sitename...\n" if $debugging == 1;
		next if $sitename !~ /\w+/;
		$logpath = "$sitesdir/$sitename/logs";
		if ( -d "$logpath" ) {
		    print "\nDEBUG: Looking for $accesslog_file under $logpath... " if $debugging == 1;
		    $numberofsites++;
		    if ( $rotateaccesslog == 1 ) {
			&rotateAccessLog;
		    } else {
			&gatherStats;
			&saveStats;
		    }
		} else {
		    print "\nDEBUG: Skipping $logpath : not a directory " if $debugging == 1;
		    next;
		}
	    }
	    print "ERROR: $0 did not find any subdirectories under $sitesdir\n" if $numberofsites < 1;
	} else {
	    die "ERROR: $0 can not opendir $sitesdir : $!";
	}
    }



# Parse access_log and gather stats lists for the given sitename

sub gatherStats {
  
    if ( ! ( chdir "$logpath" ) ) {
	print "$0 couldn't chdir to $logpath: $! \n";
	return;
    }
  
    # Clear out stats gathering lists
  
    %PagesPerSubsite = ();
    %VisitorsPerSubsite = ();
    %PagesList = ();
    %VisitorsList = ();
  
    # Gather page and visitor stats from access_log

    if ( $nopagecountgif == 1 ) {
	$lineparser = '^(\S+) \- \- \[(.+)\] \"([A-Z]{3,4})( )([\w\/\.]+).*\" [23]';
    } else {
	$lineparser = '^(\S+) \- \- \[(.+)\] \"([A-Z]{3,4}) \/([\w\/\.]+)\/pc\.gif\?([\w\.\/\-]+).*\" [23]';
    }


    if ( open(INFILE, "< $accesslog_file") ) {
	$line_number = 0;
	while (<INFILE>) {

	    last if ($line_number++ > $lines_max) && ($lines_max > 1);
	    # last if $line_number > $lines_max;

	    if ( $_ !~ /$lineparser/o ) {
		print "DEBUG: $line_number did not parse:\n$_" if $debugging == 1;
		next;
	    }

	    $remote_host = substr($1, 0, 40);
	    next if $3 eq 'HEAD';
	    $subsitename = $4;
	    $subsitename = $sitename if $nopagecountgif == 1;
	    $filename = substr($5, 0, 80);

	    # Gather summary stats 

	    $PagesPerSubsite{$subsitename}++;
	    $VisitorsPerSubsite{$subsitename}++ if !$VisitorsList{$remote_host . "\t" . $subsitename};

	    # Gather unique pagenames and visitor IP lists
	    
	    $PagesList{$subsitename . "\t" . $filename}++;
	    $VisitorsList{$remote_host . "\t" . $subsitename}++;
	    
	}
    
	close INFILE;

    } else {
	print "$0 could not read $logpath/$accesslog_file: $!\n";
	return;
    }

} # end gatherStats



# This little function catches Ctrl+C and shows stats we've gathered so far

sub dumpStats {
    $dontsavelogs = 1;
    &saveStats;    
    exit;
}



sub saveStats {

    $visitorslist = '';
    $pageslist = '';
    $summarydata = '';


# Save list of IP addresses and subsite visited

    while (($remoteip_sitename, $requests) = each(%VisitorsList)) {
	if ( $loghitspervisitor == 1 ) {
	    $visitorslist .= "$remoteip_sitename\t$requests\n";
	} else {
	    $visitorslist .= "$remoteip_sitename\n";
	}
    }

    if ( $debugging == 1 ) {
	print "\n\n$logpath/$visitors_logfile:\n" . substr($visitorslist, 0, 200);
    }

    if ( $dontsavelogs == 1 ) {
    	print "\nNOTICE: Not saving $logpath/$visitors_logfile : dontsavelogs = 1 \n";
    } else {
	if ( $visitorslist ) {
	    if (open(OUTFILE, "> $visitors_logfile")) {
		print OUTFILE "$visitorslist";
		print OUTFILE "# $sitename\n";
		close OUTFILE;
	    } else {
		print "\nERROR: $0 couldn't create $logpath/$visitors_logfile : $! \n";
	    }
	} else {
	    print "\nWARNING: $0 did not save $logpath/$visitors_logfile : list was empty \n";
	}
    }



# Save list of pages and hits per page

    while (($pagename, $requests) = each(%PagesList)) {
	$pageslist .= "$pagename\t$requests\n";
    }

    if ( $debugging == 1 ) {
	print "\n\n$logpath/$pages_logfile:\n" . substr($pageslist, 0, 200);
    }

    if ( $dontsavelogs == 1 ) {
    	print "\nNOTICE: Not saving $logpath/$pages_logfile : dontsavelogs = 1 \n";
    } else {
	if ( $pageslist ) {
	    if (open(OUTFILE, "> $pages_logfile")) {
		print OUTFILE "$pageslist";
		print OUTFILE "# $sitename\n";
		close OUTFILE;
	    } else {
		print "\nERROR: $0 couldn't create $logpath/$pages_logfile : $! \n";
	    }
	} else {
	    print "\nWARNING: $0 did not save $logpath/$pages_logfile : list was empty \n";
	}
    }



# Update today's traffic summary log

    while (($subsitename, $pagehits) = each(%PagesPerSubsite)) {
	$summarydata .=  "$subsitename\t$pagehits\t" . $VisitorsPerSubsite{$subsitename} . "\n";
    }

    if ( $debugging == 1 ) {
	print "\n\n$logpath/$summary_logfile:\n" . $summarydata;
    }

    if ( $dontsavelogs == 1 ) {
    	print "\nNOTICE: Not saving $logpath/$summary_logfile : dontsavelogs = 1 \n";
    } else {
	if ( $summarydata ) {
	    if (open(OUTFILE, "> $summary_logfile")) {
		print OUTFILE "$summarydata";
		close OUTFILE;
	    } else {
		print "\nERROR: $0 couldn't create $logpath/$summary_logfile : $! \n";
	    }
	} else {
	    print "\nWARNING: $0 did not save $logpath/$summary_logfile : list was empty \n";
	}
    }


} # end saveStats





# Find all access_log(s) in weblogpath, make day-of-week backup copy, then zero-out access_log(s)

sub rotateAccessLog {

    # Only root user can do this
    if ( $> != 0 ) {
	print "ERROR: Only the root user can effectively use the rotateaccesslog option. ( Effective uid = $> ) \n";
	return;
    }

    print "DEBUG: Looking in $logpath for access_logs to rotate...\n" if $debugging == 1;

    $accesslogfiledate = $dayofweek;
    $accesslogfilename = '';
    @accesslogfiles    = ();
    @logfiles          = ();


    if (opendir (DIR, "$logpath")) {
	@logfiles = grep /log$/, (readdir DIR);
	close DIR;

	     @accesslogfiles = grep /access/,  @logfiles;   # Tried doing this all in one grep but it matched files I didn't want
	push @accesslogfiles, (grep /error/,   @logfiles);
	push @accesslogfiles, (grep /referer/, @logfiles);
	push @accesslogfiles, (grep /agent/,   @logfiles);


	foreach $accesslogfilename ( sort @accesslogfiles ) {

	    $accesslogfile = "$logpath/$accesslogfilename";
		    
	    print "\nDEBUG: $accesslogfilename is ", -s "$accesslogfile", " bytes before rotating.\n" if $debugging == 1; 

	    if ( $dontsavelogs == 1 ) {
		print "NOTICE: Not actually rotating $accesslogfilename ==> $accesslogfilename\.$accesslogfiledate : dontsavelogs == 1 \n";
		$num_files++;
		next;
	    }

	    # By copying the current access_log to new filename then zero out access_log we don't have to restart Apache

	    if ( open INFILE, "< $accesslogfile" ) {
		if ( open OUTFILE, "> $accesslogfile\.$accesslogfiledate" ) {
		    print "DEBUG: Copying $accesslogfile ==> $accesslogfile\.$accesslogfiledate\n" if $debugging == 1;
		    while (<INFILE>) {
			print OUTFILE;
		    }
		    close OUTFILE;

		    qx|/bin/cat /dev/null > $accesslogfile|;

		    print "DEBUG: $accesslogfilename is now ", -s "$accesslogfile", " bytes after rotating.\n" if $debugging == 1; 
		    $num_files++;

		} else {
		    print "ERROR: Could not write to $accesslogfile\.$accesslogfiledate : $! \n";
		}
		close INFILE;
	    } else {
		print "ERROR: Could not read to $accesslogfile : $! \n";
	    }

	} # foreach rotate access_log
 
	if ( $num_files == 1 ) {
	    print "DEBUG: $accesslogfile has been rotated.\n" if $debugging == 1;
	} elsif ( $num_files > 1 ) {
	    print "DEBUG: $num_files access_log files rotated.\n" if $debugging == 1;
	} else {
	    print "ERROR: No access_log files found in $logpath.\n";
	}

    } else {
	print "ERROR: RotateAccessLogs: Could not read directory $logpath : $! \n";
    }

    return;

} # sub rotateAccessLog


__END__

