#!/usr/bin/perl

use Date::Manip;
use Socket;

#
# Shell script to count visitors per domain for a given web site.
# Does a reverse DNS lookup on each IP listed in visitors_(date).log
# then tallies by domain.
#
#
# Modified:  Kurt Gray -- Thu Oct 19 17:55:42 EDT 2000     First rev.


# Read inputs from command line

# Read options from command line

    foreach $arg (@ARGV) {

	# Option to specify a site

	if ( $arg =~ /site=(\w+)/ ) {
	    $sitename = $1;

	# Debug option shows what's happening without writing new logs
	# and a Ctrl+C keystroke will stop script and dumps stats gathered so far

	} elsif ( $arg =~ /debug/ ) {
	    $debugging = 1;
	    $SIG{INT} = showSummary;

	# Option to get stats for a specific date (date=yesterday is useful for regenerating yesterday's stats)

	} elsif ( $arg =~ /date=(\w+)/ ) {
	    $date_input = $1;

	# Abort DNS lookup after this many seconds

	} elsif ( $arg =~ /timeout=(\d+)/ ) {
	    $timeout = $1;

        # Number of lines to read in access_log before stopping (useful for debugging)

	} elsif ( $arg =~ /lines=(\d+)/ ) {
	    $lines_max = $1;

	# Specify a base directory web sites are listed under

	} elsif ( $arg =~ /sitesdir=([\w\/]+)/ ) {
	    $sitesdir = $1;

	# Option to start reading access_log at this line (debugging option, not implemented)

	} elsif ( $arg =~ /linestart=(\d+)/ ) {
	    $line_start = $1;

        # Option to run script wihtout saving logs (useful when debugging)

	} elsif ( $arg =~ /dontsavelogs/ ) {
	    $dontsavelogs = 1;

        # Option to save analyzed results for every x number of lines analyzed:

	} elsif ( $arg =~ /saveoninterval=(\d+)/ ) {
	    $save_on_interval = $1;

	# Option to specify which IP list file to read

	} elsif ( $arg =~ /infile=([\w\.\/\-]+)/ ) {
	    $visitors_logfile = $1;

	} else {
	    print "ERROR: $0 does accept argument '$arg'\n";
	    exit;
	}

    } # foreach arg



    die "Usage: $0 site=SiteName" if !$sitename;


    $sitesdir      ||= '/usr/apache/sites';
    $logpath         = "$sitesdir/$sitename/logs";
    $timeout       ||= 2;
    $logdate         = $date_input;
    $logdate       ||= &Date::Manip::UnixDate("today", '%Y%m%d');
    $hour            = `/bin/date +'%H'`;	# If it's after midnight, then redo yesterday's stats

    $visitors_logfile       ||= "$logpath/visitors_" . $logdate . '.log';
    $domain_summary_logfile ||= "$logpath/domain_summary_" . $logdate . '_'  . $sitename . '.log';
  
  

# Read summary list of IP addresses

  if ( open INFILE, "< $visitors_logfile" ) {
	
	if ( $debugging == 1 ) {
		print "VISITOR IP DOMAIN ANALYSIS for $sitename\n";
		print "READING visitors IP list file: $visitors_logfile\n";
		print "OUTPUT summary file: $domain_summary_logfile \n";
	}

      while (<INFILE>) {
	  next if ($line_number++ < $line_start) && $line_start;
	  ($tmp_ip, $tmp_sitename) = split(/\t/, $_);
	  $hostname    = '';
	  $domain_code = '??';

	  # if ( $tmp_ip =~ /(\d{2,3}\.\d{1,3}\.\d{1,3}\.)/ ) {
	      # $remote_ip = $1 . '1'; # Try reverse lookup on the .1 address only (faster? doesn't seem to be)

	  if ( $tmp_ip =~ /^\d{1,3}\./ ) {

	      $remote_ip = $tmp_ip;

	      eval {
		  local $SIG{ALRM} = sub { die "timeout\n" }; # NB: \n required
		  alarm $timeout;
		  $hostname = gethostbyaddr( inet_aton( $remote_ip ), AF_INET );
		  alarm 0;
	      };

	      if ($@) {
		  $domain_code = 'xx' if $@ eq "timeout\n";   # propagate unexpected errors
		  $hostname = '';
		  # timed out
	      } else {
		  if ( $hostname =~ /\.([A-Za-z]{2,3})$/ ) {
		      $domain_code = lc($1);
		  } 
	      }

	  } elsif ( $tmp_ip =~ /\.([A-Za-z]{2,3})$/ ) {
	      $hostname = $tmp_ip;
	      $domain_code = lc($1);

	  } else {
	      
	      $hostname = $tmp_ip;
	      print "DEBUG: Line $line_number did not parse: $hostname" if $debugging == 1;

	  }

	  $VisitorsPerDomain{ $domain_code }++;
	  $total_visitors++;
	  last if ($line_number > $lines_max) && ($lines_max > 1);
	  print "$line_number\t$remote_ip\t=>\t$hostname\t=>\t$domain_code\n" if $debugging == 1;

	  # We can save progress so far for every x number of lines analyzed:

          if ( $save_on_interval > 10 ) {
	     if ( ( $line_number % $save_on_interval ) == 0 ) {
	  	$dont_exit_flag = 1;
	  	&showSummary;
	     }
          } # save_on_interval


      } # while INFILE

      close INFILE;
  } else {
      print "$0 couldn't read $visitors_logfile : $! ";
  }



  &showSummary;


# End



# Write/dislpay Visitors per Domain summary log:

sub showSummary {

    if ( $total_visitors > 1 ) {

	# $display_date = &Date::Manip::UnixDate("$logdate", '%F');
 
	$domain_summary_data  = "\n# Domain Analysis of $visitors_logfile\n";
	$domain_summary_data .= "\n# Visitors, Ratio to Total, Domain Code, Country\n\n";

	foreach $domain_code (sort { $VisitorsPerDomain{$b} <=> $VisitorsPerDomain{$a} } keys %VisitorsPerDomain) {
	    $visitors = $VisitorsPerDomain{$domain_code};
	    $ratio = "$visitors/$total_visitors=" . substr(($visitors/$total_visitors), 0 , 5);
	    $domain_summary_data .= "$visitors\t$ratio\t$domain_code\t" . &getCountryByDomain($domain_code) . "\n";
	}	

	print $domain_summary_data if $debugging == 1;
	
	if ( $dontsavelogs == 1 ) {
	    print "NOTICE: Not saving $domain_summary_logfile : dontsavelogs == 1\n";
	} else {
	    if ( $domain_summary_data ) {
		if (open(OUTFILE, "> $domain_summary_logfile")) {
		    print OUTFILE $domain_summary_data;
		    close OUTFILE;
		    print "DEBUG: Saved $domain_summary_logfile \n" if $debugging == 1;			
		} else {
		    print "$0 couldn't create $domain_summary_logfile : $! ";
		}
	    } else {
		print "$0 did not create $domain_summary_logfile : VisitorsPerDomain array was empty. ";
	    }
	}

    } # if total_visitors


    # Exit here if end of script or Ctrl+C was pressed:

    if (( $dont_exit_flag ) && ( $save_on_interval )) {
	$dont_exit_flag = 0; # Allow Ctrl+C to exit
	return 1;
    } else {
    	exit;  # exit needed here to stop after Crtl+C 
    }

} # end showSummary



sub getCountryByDomain {

    my $domain_code = shift;

    $domain_codes ||=<<DOMAIN_CODES;
ad   Andorra
ae   United Arab Emirates
af   Afghanistan
ag   Antigua and Barbuda
ai   Anguilla
al   Albania
am   Armenia
an   Netherlands Antilles
ao   Angola
aq   Antarctica
ar   Argentina
arpa Old style Arpanet
as   American Samoa
at   Austria
au   Australia
aw   Aruba
az   Azerbaidjan
ba   Bosnia-Herzegovina
bb   Barbados
bd   Bangladesh
be   Belgium
bf   Burkina Faso
bg   Bulgaria
bh   Bahrain
bi   Burundi
bj   Benin
bm   Bermuda
bn   Brunei Darussalam
bo   Bolivia
br   Brazil
bs   Bahamas
bt   Bhutan
bv   Bouvet Island
bw   Botswana
by   Belarus
bz   Belize
ca   Canada
cc   Cocos (Keeling) Islands
cf   Central African Republic
cg   Congo
ch   Switzerland
ci   Ivory Coast (Cote D\'Ivoire)
ck   Cook Islands
cl   Chile
cm   Cameroon
cn   China
co   Colombia
com  USA Commercial
cr   Costa Rica
cs   Former Czechoslovakia
cu   Cuba
cv   Cape Verde
cx   Christmas Island
cy   Cyprus
cz   Czech Republic
de   Germany
dj   Djibouti
dk   Denmark
dm   Dominica
do   Dominican Republic
dz   Algeria
ec   Ecuador
edu   USA Educational
ee   Estonia
eg   Egypt
eh   Western Sahara
es   Spain
et   Ethiopia
fi   Finland
fj   Fiji
fk   Falkland Islands
fm   Micronesia
fo   Faroe Islands
fr   France
fx   France (European Territory)
ga   Gabon
gb   Great Britain
gd   Grenada
ge   Georgia
gf   French Guyana
gh   Ghana
gi   Gibraltar
gl   Greenland
gm   Gambia
gn   Guinea
gov  USA Government
gp   Guadeloupe (French)
gq   Equatorial Guinea
gr   Greece
gs   S. Georgia & S. Sandwich Isls.
gt   Guatemala
gu   Guam (USA)
gw   Guinea Bissau
gy   Guyana
hk   Hong Kong
hm   Heard and McDonald Islands
hn   Honduras
hr   Croatia
ht   Haiti
hu   Hungary
id   Indonesia
ie   Ireland
il   Israel
in   India
int  International
io   British Indian Ocean Territory
iq   Iraq
ir   Iran
is   Iceland
it   Italy
jm   Jamaica
jo   Jordan
jp   Japan
ke   Kenya
kg   Kyrgyzstan
kh   Cambodia
ki   Kiribati
km   Comoros
kn   Saint Kitts & Nevis Anguilla
kp   North Korea
kr   South Korea
kw   Kuwait
ky   Cayman Islands
kz   Kazakhstan
la   Laos
lb   Lebanon
lc   Saint Lucia
li   Liechtenstein
lk   Sri Lanka
lr   Liberia
ls   Lesotho
lt   Lithuania
lu   Luxembourg
lv   Latvia
ly   Libya
ma   Morocco
mc   Monaco
md   Moldavia
mg   Madagascar
mh   Marshall Islands
mil  USA Military
mk   Macedonia
ml   Mali
mm   Myanmar
mn   Mongolia
mo   Macau
mp   Northern Mariana Islands
mq   Martinique (French)
mr   Mauritania
ms   Montserrat
mt   Malta
mu   Mauritius
mv   Maldives
mw   Malawi
mx   Mexico
my   Malaysia
mz   Mozambique
na   Namibia
nato   NATO
nc   New Caledonia (French)
ne   Niger
net  USA Network
nf   Norfolk Island
ng   Nigeria
ni   Nicaragua
nl   Netherlands
no   Norway
np   Nepal
nr   Nauru
nt   Neutral Zone
nu   Niue
nz   New Zealand
om   Oman
org  USA Non-Profit
pa   Panama
pe   Peru
pf   Polynesia (French)
pg   Papua New Guinea
ph   Philippines
pk   Pakistan
pl   Poland
pm   Saint Pierre and Miquelon
pn   Pitcairn Island
pr   Puerto Rico
pt   Portugal
pw   Palau
py   Paraguay
qa   Qatar
re   Reunion (French)
ro   Romania
ru   Russian Federation
rw   Rwanda
sa   Saudi Arabia
sb   Solomon Islands
sc   Seychelles
sd   Sudan
se   Sweden
sg   Singapore
sh   Saint Helena
si   Slovenia
sj   Svalbard and Jan Mayen Islands
sk   Slovak Republic
sl   Sierra Leone
sm   San Marino
sn   Senegal
so   Somalia
sr   Suriname
st   Saint Tome and Principe
su   Former USSR
sv   El Salvador
sy   Syria
sz   Swaziland
tc   Turks and Caicos Islands
td   Chad
tf   French Southern Territories
tg   Togo
th   Thailand
tj   Tadjikistan
tk   Tokelau
tm   Turkmenistan
tn   Tunisia
to   Tonga
tp   East Timor
tr   Turkey
tt   Trinidad and Tobago
tv   Tuvalu
tw   Taiwan
tz   Tanzania
ua   Ukraine
ug   Uganda
uk   United Kingdom
um   USA Minor Outlying Islands
us   United States
uy   Uruguay
uz   Uzbekistan
va   Vatican City State
vc   Saint Vincent & Grenadines
ve   Venezuela
vg   Virgin Islands (British)
vi   Virgin Islands (USA)
vn   Vietnam
vu   Vanuatu
wf   Wallis and Futuna Islands
ws   Samoa
ye   Yemen
yt   Mayotte
yu   Yugoslavia
za   South Africa
zm   Zambia
zr   Zaire
zw   Zimbabwe
??   DNS Unknown
xx   DNS Timeout
DOMAIN_CODES


    if (! %domain_names) {
	foreach $dline (split(/[\n\r]/, $domain_codes)) {
	    chomp $dline;
	    ($dkey, $dname) = split(/\s\s+/, $dline);
	    $domain_names{$dkey} = $dname;
	}
    }

    $country_name = $domain_names{$domain_code};
    $country_name ||= $domain_code;
    return $country_name;

} # end sub getCountryByDomain



__END__







