#!/usr/bin/perl

use Date::Manip;

#
# Shell script which tabulates unique visitors per month 
# from a list of IP address files (visitors_(date_.log files)
#
# Modified:  Kurt Gray -- Wed Dec  6 10:40:56 EST 2000       First rev.
#

# Read options from command line

    foreach $arg (@ARGV) {

	# Option to specify a site

	if ( $arg =~ /site=(\w+)/ ) {
	    $sitename = $1;

	# Debug option shows what's happening without writing new logs
	# and a Ctrl+C keystroke will stop script and dumps stats gathered so far

	} elsif ( $arg =~ /debug/ ) {
	    $debugging = 1;
	    # $dontsavelogs = 1;
	    $SIG{INT} = dumpStats;

	# Option to get stats for a specific date (date=yesterday is useful for regenerating yesterday's stats)

	} elsif ( $arg =~ /date=(\w+)/ ) {
	    $date_input = $1;

        # Number of lines to read in access_log before stopping (useful for debugging)

	} elsif ( $arg =~ /lines=(\d+)/ ) {
	    $lines_max = $1;

	# Specify a base directory web sites are listed under

	} elsif ( $arg =~ /sitesdir=([\w\/]+)/ ) {
	    $sitesdir = $1;

	# Option to start reading access_log at this line (debugging option, not implemented)

	} elsif ( $arg =~ /linestart=(\d+)/ ) {
	    $line_start = $1;

        # Option to run script wihtout saving logs (useful when debugging)

	} elsif ( $arg =~ /dontsavelogs/ ) {
	    $dontsavelogs = 1;

	# Option to write number_of_hits per IP column in visitors_(date).log

	} elsif ( $arg =~ /hitspervisitor/ ) {
	    $loghitspervisitor = 1;

	# Option to ignore pagecount gif and do regular hit parsing

	} elsif ( $arg =~ /nopagecountgif/ ) {
	    $nopagecountgif = 1;

	# Parse all monthly logs into a network total report and exit.

	} elsif ( $arg =~ /network/ ) {
	    $network_unique = 1;

	} else {
	    
	    print "ERROR: $0 does not understand argument: $arg \n";
	    exit;

	}

    } # foreach arg


    $sitesdir      ||= '/usr/apache/sites';
    $serverlogpath ||= "$sitesdir/../logs";


# Do network total analysis only:

	if ( $network_unique == 1 ) {
		&countNetworkUnique;
		exit;
	}


# Make a list of sites being monitored on this server according to subdirectories under logs

    if ( $sitename ) {
	print "\nDEBUG: Looking at $sitename...\n" if $debugging == 1;
	$logpath = "$sitesdir/$sitename/logs";
	&gatherStats;
    } else {
	if (opendir (DIR, "$sitesdir")) {
	    @sitelist = readdir DIR;
	    close DIR;
	    foreach $sitename (sort @sitelist) {
		print "\nDEBUG: Looking at $sitename...\n" if $debugging == 1;
		next if $sitename !~ /\w+/;
		$logpath = "$sitesdir/$sitename/logs";
		if ( -d "$logpath" ) {
		    $numberofsites++;
		    &gatherStats;
	    	    %UniqueVisitors = ();	
	            %UniqueVisitorsSubSites = ();
		} else {
		    print "\nDEBUG: Skipping $logpath : not a directory " if $debugging == 1;
		    next;
		}
	    }
	    if ( $numberofsites < 1 ) {
		print "ERROR: $0 did not find any subdirectories under $sitesdir\n";
	    }
	} else {
	    die "ERROR: $0 can not opendir $sitesdir : $!";
	}
    }





# Read all month visitors list a report on total unique visitors:

sub countNetworkUnique {

    if ( opendir DIR, "$serverlogpath" ) {
	@visitors_logs = grep /^unique_visitors_\d{6}_[\w\-\.]+.log$/, (readdir DIR);
	closedir DIR;
	$number_of_logs = @visitors_logs;
	
	if ( $number_of_logs < 1 ) {
		print "ERROR: Did not find any unique_visitors_date_logs in $serverlogpath \n";
		return 0;
    	}

	foreach $tmp_visitor_log ( sort @visitors_logs ) {
		$visitors_log = "$serverlogpath/$tmp_visitor_log";
		print "DEBUG: Reading $visitors_log .... " if $debugging == 1;
		if ( open INFILE, "< $visitors_log" ) {
			while ($line = <INFILE>) {
				next if $line !~ /^([\d\.]+)\t/;
				$NetworkUniqueVisitors{ $1 }++; 
			}
			close INFILE;
			$subtotal = (keys %NetworkUniqueVisitors);
			print "NETWORK SUBTOTAL: $subtotal\n";
		} else {
			print "ERROR: Could not read $visitors_log : $! \n";
		}
	} # foreach visitors_log

    } else {
	print "ERROR: Could not readdir $serverlogpath: $! \n";
	return 0;
    } 


} # sub networkUnique






# Parse access_log and gather stats lists for the given sitename:

sub gatherStats {

    my ( $month, $day, $year ) = ();
    $previous_month            = '';
    $previous_year             = '';
    %UniqueVisitors            = ();
    %UniqueVisitorsSubSites    = ();
    @visitors_logs             = ();


    # Make list of visitors_logs in this directory

    if ( opendir DIR, "$logpath" ) {
	@visitors_logs = grep /^visitors_\d{8}\.log$/, (readdir DIR);
	closedir DIR;
	$number_of_logs = @visitors_logs;
    } else {
	print "ERROR: Could not readdir $logpath: $! \n";
	return 0;
    } 

    if ( $number_of_logs < 1 ) {
	print "NOTICE: Skipping $sitename: gatherStats did not find any visitors_logs in $logpath \n";
	return 0;
    }

    if ( $number_of_logs < 5 ) {
	print "NOTICE: Skipping $sitename: gatherStats found only $number_of_logs visitors_logs in $logpath, not enough to do monthly stats. \n";
	if ( $debugging == 1 ) {
	    print "DEBUG: Continue counting unique visitors for $sitename anyway? [y/n] ";
	    $input = <STDIN>;
	    return 0 if $input =~ /n/i;
	} else {
	    return 0;
	}
    } # logs < 5
    

    # Presize large hashes to supposedly improves memory performance

    $average_logsize  = ( -s "$logpath/$visitors_logs[1]" );
    $approx_arraysize = int( $average_logsize / 13 );
    
    print "DEBUG: Typical visitor log size = $average_logsize bytes, guessing we'll have $approx_arraysize hash keys... " if $debugging == 1;

    if ( $approx_arraysize > 10000 ) {
	print "allocating memory for two pre-sized hashes. \n" if $debugging == 1;
	keys ( %UniqueVisitors )         = $approx_arraysize;  # Presizing big hashes supposedly saves time
	keys ( %UniqueVisitorsSubSites ) = $approx_arraysize;  # Presizing big hashes supposedly saves time
    } else {
	print "not bothering to pre-size hashes. \n" if $debugging == 1;
    }

    # Foreach month create a unique visitors log

    foreach $visitors_log ( sort { $a <=> $b } @visitors_logs ) {

	$previous_year  = $year;
	$previous_month = $month;

	if ( $date_input ) {
	    next if $visitors_log !~ /$date_input/;
	}

	next if $visitors_log !~ /^visitors_(\d{4})(\d{2})(\d{2})\.log$/;
	$year  = $1;
	$month = $2;
	$day   = $3;

	# If new month save previous month's stats

	if (( $previous_month ) && ( $previous_month ne $month )) { 

	    &saveStats( "$serverlogpath/unique_visitors\_$previous_year$previous_month\_$sitename.log" );

	    %UniqueVisitors = ();
	    %UniqueVisitorsSubSites = ();
	    
	} # if previous_month


	# Tally in IPs for this visitors log

	if ( open INFILE, "< $logpath/$visitors_log" ) {
	    print "DEBUG: Reading $logpath/$visitors_log...\n" if $debugging == 1;
	    while (<INFILE>) {
		next if $_ !~ /^(\S+)\t(\S+)/;
		if ( $network_total == 1 ) {
			$UniqueVisitorsNetwork{ $1 }++;          # Tally of unique IPs for entire network
		} else {
			$UniqueVisitors{ $1 }++;                 # Tally of unique IPs to this group of sites
			$UniqueVisitorsSubSites{ "$1\t$2" }++;   # Tally of repeat visits during this month per IP to each subsite
		}
	    }
	    close INFILE;
	} else {
	    print "ERROR: Could not read log file $logpath/$visitors_log : $! \n";
	    next;
	}

    } # foreach visitors_logs

    # save stats for last month read

    &saveStats( "$serverlogpath/unique_visitors\_$previous_year$previous_month\_$sitename.log" );


} # end gatherStats




# This little function catches Ctrl+C and shows stats we've gathered so far

sub dumpStats {
    $dontsavelogs = 1;
    &saveStats;    
    exit;
}



sub saveStats {

    my $logfile          = shift;
    my $total_visits     = 0;
    my %visits_histogram = ();
    my %UniquePerSiteid  = ();
    my $visitshistogram  = '';
    my $average_days     = 0;
    my $number_of_days   = 0;
    my $percentage       = 0;
    my $total_percentage = 0;
    my $displayed_IPs    = 0;
    my $visits           = 0;
    my $tmp_siteid       = '';
    my $IP               = '';
    my $summary_line     = '';
    my $summary_data     = '';
    my $summary_key      = '';
    my $summary_line_key = '';


    $total_visitors = ( keys %UniqueVisitors );
    if ( $total_visitors < 1 ) {
	print "WARNING: Skipping saveStats $previous_month/$previous_year: UniqueVisitors list for $sitename is empty: total_visitors == 0 \n";
	return 0;
    }


    print "DEBUG: Tallying unique visitors per siteid during $previous_month/$previous_year... \n" if $debugging == 1;

    while ( ($tmp_IP_siteid, $visits) = each %UniqueVisitorsSubSites ) {
	( $IP, $subsitename ) = split /\t/, $tmp_IP_siteid;  # Get the sub siteid
	$UniquePerSiteid{ $subsitename }++;                  # Tally unique visitors by sub siteid
    }


    print "DEBUG: Tallying visitor frequencies for $sitename sites during $previous_month/$previous_year... \n" if $debugging == 1;

    while ( ( $IP, $visits ) = each %UniqueVisitors ) {
	$total_visits += $visits;                           # Total repeat visits to this group of sites
	$visits_histogram{ $visits }++;                     # Common frequencies of visits to this group of si
    }

    print "DEBUG: Sorting visitor frequency list by number of vists for $sitename... \n" if $debugging == 1;

    foreach $visits ( sort { $a <=> $b } keys %visits_histogram ) {
	$number_of_IPs = $visits_histogram{ $visits };
	$displayed_IPs += $number_of_IPs;
	$percentage = ( $number_of_IPs / $total_visitors ) * 100;
	$total_percentage += $percentage;
	last if $percentage < .5;
	$last_visits = $visits;
	$percentage = substr( $percentage, 0, 4 );
	$visitshistogram .= "$number_of_IPs\t$visits\t$percentage\n";
    }

    $visitshistogram .= ( $total_visitors - $displayed_IPs ) . "\t> " . $last_visits  . "\t" . substr(( 100 - $total_percentage ), 0, 4) . "\n";




    $summary_key = qq|
# Summary of Unique Visitors per Month. Data by column:
#
# Month/Year, SiteGroup, Total IPs, Subsite, Subsite IPs  
|;


    foreach $subsitename ( sort keys %UniquePerSiteid ) {
	$site_total_visitors = $UniquePerSiteid{ $subsitename };
	$summary_line .= join "\t", (
					"$previous_month/$previous_year", 
					$sitename,
					$total_visitors,
					$subsitename,
					$site_total_visitors, "\n" );
    }


    # Append summary line to summary log file

    if ( -f "$serverlogpath/unique_visitors_summary.log" ) {
	$summary_line_key = '';
    } else {
	$summary_line_key = $summary_key;
    }

    if ( open OUTFILE, ">> $serverlogpath/unique_visitors_summary.log" ) {
	print OUTFILE "$summary_line_key\n" if $summary_line_key;
	print OUTFILE $summary_line;
	close OUTFILE;
	print "DEBUG: Appended summary_line to $serverlogpath/unique_visitors_summary.log\n" if $debugging == 1;
    } else {
	print "ERROR: Could not append to $serverlogpath/unique_visitors_summary.log : $!\n";
    }



    # Make header information for unique_visitors log:

    $summary_data = qq|# Traffic Stats:  Unique visitors (IP addresses) on $sitename during month of $previous_month/$previous_year
$summary_key
<Summary>
$summary_line</Summary>

#
# Histogram of Frequency of Visits per Month:
#
# Number of Visitors
# \|\tNumber of Days
# \|\t\|\t\% of Total Visitors

<Histogram>
$visitshistogram</Histogram>

# IP Address list for $sitename during $previous_month/$previous_year
#
# IP Address
# \|\t\tSiteid
# \|\t\t\|\tNumber of days visited this site during month of $previous_month/$previous_year

|;


    if ( $debugging == 1 ) {
	print "$summary_data";
    }

    if ( $dontsavelogs == 1 ) {
    	print "\nNOTICE: Not saving $logfile : dontsavelogs = 1 \n";
    } else {
	if (open(OUTFILE, "> $logfile")) {
	    # NOTE: I was considering unbuffering while saving this big OUTFILE  
	    # but would probably cause the hard disk to spin like mad.
	    print OUTFILE $summary_data;
	    print OUTFILE "<Visitors>\n";
	    while ( ($tmp_IP_siteid, $visits) = each %UniqueVisitorsSubSites ) {
		print OUTFILE "$tmp_IP_siteid\t$visits\n";
	    }
	    print OUTFILE "</Visitors>\n";
	    close OUTFILE;
	    print "DEBUG: Saved $logfile\n" if $debugging == 1;
	} else {
	    print "\nERROR: Could not create $logfile : $! \n";
	}
    }

    return 1;

} # end saveStats


__END__



