#!/usr/bin/perl

#
# This CGI script is the interface used to 
# set the daily ad allocation on each web site.
#
# Notes:
#	- ad_info.dat = properties of each ad
#	- ad_x_alloc.dat = unbalanced large ad allocations
#	- ad_x_rotate.dat = final rotation list for large banner 
#			 (generated each hour by ad_rotateset.pl)
#	- surviving add list is sorted by AdID
#	- removed TABLEs to make page display faster
#	- Added 'Stop on Target' and 'Stop on Date' columns
#	- Reads and writes ad_x_alloc.dat
#	- Shows projected inventory (page impressions)
#	- Ad position set by changing adposition variable
#	- adinfo.dat changed to ad_info.dat
#	- Fixed American date input to ISO
#	- When totalling impressions allocated, only count ads within current date range
#
#
# Modified:  Kurt Gray -- Tue Sep 26 13:29:57 EDT 2000     Added 125x125 height restrictions for VA sites
#            Kurt Gray -- Fri Oct 13 15:32:52 EDT 2000     Added NULL option to top of adid droplist to avoid rewriting expired adids
#            Kurt Gray -- Sat Oct 14 10:05:44 EDT 2000     Modified to be centralized on report.andover.net
#            Kurt Gray -- Tue Oct 24 15:51:05 EDT 2000     Using new %sitesconfig array from ad_config_lib.pl
#            Kurt Gray -- Wed Nov  8 11:04:48 EST 2000     Moved to ad system CGI interfaces /usr/local/adsystem/cgi-bin
#            Kurt Gray -- Mon Nov 13 14:49:07 EST 2000     Script settings code cleanup
#            Kurt Gray -- Sun Jan  7 20:12:25 EST 2001     Reformatted to view, sort, and edit allocation tables 

use Date::Manip;

push @INC, '/usr/local/adsystem/lib/';
require 'ad_config_lib.pl';	        # Site-specific settings for adsystem control scripts
require 'ad_util_lib.pl';	        # Common functions for adsystem scripts

{

# Script settings and base inputs:

    $ad_position               = $1 if $ENV{'QUERY_STRING'} =~ /adpos=(\d)/;
    $ad_position             ||= 1;
    $siteid                    = $1 if $ENV{'QUERY_STRING'} =~ /siteid=(\w{7})/;
    $site_initials             = substr( $siteid, 0, 2 );
    $sitename                  = $sitesconfig{ $siteid . 'sitename' };
    @ad_height_restrictions    = split( ',', $sitesconfig{ $siteid . 'ad_layout' });
    $can_run_megabanners       = 0 if $sitesconfig{ $siteid . 'ad_restrictions' } =~ /MEGABANNERS=0/;
    $thisscript                = "$ad_allocedit_url\?adpos=$ad_position&siteid=$siteid";	
    $allocfile                 = "ad\_$ad_position\_alloc.dat\.$siteid";
    $largead_position          = 1 if ( grep ( /$ad_position/, ( 1, 5 ) ) );	
    $adtype                    = $adtypes{$ad_position};

    $yesterday                 = &Date::Manip::UnixDate("yesterday", '%y%m%d');
    $today                     = &Date::Manip::UnixDate("today", '%y%m%d');
    $todayfullsysdate          = &Date::Manip::UnixDate("today", '%Y%m%d');

    $yesterday_adlog           = $adlogpath  . '/ads_' . $yesterday . '.log';  
    $pagetrafficlog            = $weblogpath . '/visitors/visitors' . $today . '.log';



    &loginUser( $editorial_staff );


# Read form inputs:

    $op                  = $1 if $form->{'in'}->{'op'} =~ /(\w+)/;
    $op                ||= 'view';
    $function            = 'save' if $form->{'in'}->{'GoButton'} =~ /Save/;
    $sortby              = $1 if $form->{'in'}->{'sortby'} =~ /(\w+)/;




# Determine how much inventory will be available today across all sites that share allocations with this site

    if ( -e "$pagetrafficlog" ) {
	
	$hour = `/bin/date +%H`;
	$hour =~ s/^0//;	
	if ($hour > 1) {
	    if ( open(INFILE, "< $pagetrafficlog" )) {
		while (<INFILE>) {
		    chomp;
		    ( $dt, $vs, $pages, $tmp_siteid ) = split ',', $_;
		    # Add page hits from allocation subsites:
		    if ( $sitesconfig{ $tmp_siteid . 'ad_allocation_group' } =~ /([a-z][a-z])/ ) {
			$tmp_siteid = $1 . '000en';
		    }
		    $pages_per_siteid{ $tmp_siteid } += int( $pages * ( 24 / $hour) );		    
		}
		close INFILE;
	    } else {
		$error_messages .= "WARNING: Couldn't read $pagetrafficlog: $! <BR> \n";
	    }
	}
	
    }  # if local pagelog



# Show allocation menu if no siteid specified:

    if ( $siteid !~ /^\w{7}$/ ) {

	# Make table of ad allocation links:

	delete $nav_links->{ $editorial_staff }->{'Ad Allocations'};
	
	$table_data      =  '';
	$table_contents  =  '';
	$table_title     =  'Ad Allocations per Web Site';
	if ( $op ne 'showvolume' ) {
	    $table_title    .= qq| &nbsp [<A HREF="$thisscript&op=showvolume">Show Allocations Graph</A>] |;
	} else {
	    $table_title    .= qq| Percentages of Page Traffic Allocated &nbsp [<A HREF="$thisscript">Show Normal View</A>] |;
	}
	@table_headers   =  ( 'Web Site', ( values %adtypes ), 'Reserved', 'Newsletter' );
	

	&getActiveSitenames;  # sets up %sitenames hash

	foreach $sitename ( sort keys %sitenames ) {

	    $siteid                =  $sitenames{ $sitename };
	    $ad_positions          =  $sitesconfig{ $siteid . 'ad_positions' };
	    $ad_newsletter         =  $sitesconfig{ $siteid . 'ad_newsletter' };
	    $ad_restrictions       =  $sitesconfig{ $siteid . 'ad_restrictions' };
	    
	    $table_data .= "$sitename\t";

	    $page_impressions = $pages_per_siteid{ $siteid };
	    
	    # Links for each ad position on this site:
	    
	    foreach $ad_position ( keys %adtypes ) {

		$adtype = $adtypes{$ad_position};
	    
		if ( $ad_positions =~ /\b$ad_position\b/ ) {

		    $allocation_url = "$ad_allocedit_url\?siteid\=$siteid\&adpos\=$ad_position";

		    # Calculate % percent allocated for this site and ad position

		    if (( $op eq 'showvolume' ) && ( $page_impressions > 100 )) {

			$adallocfile = "$datfilepath/ad_" . $ad_position . "_alloc.dat." . $siteid;
			if ( open( INFILE, "< $adallocfile" ) ) { 
			    while (<INFILE>) {
				my ( $ad_id,
				  $ad_impressions,
				  $ad_StopOnTarget,
				  $ad_StartDate,
				  $ad_StopDate ) = split /\t/, $_;
				$total_allocated += $ad_impressions if &checkAllocationDateRange;
			    }
			    close INFILE;
			} else {
			    $error_messages .= "ERROR: Could not read ad_allocation file $adallocfile : $! <BR>\n";
			}
			$percent_allocated = int(( $total_allocated / $page_impressions ) * 100);
			$percent_width     = $percent_allocated;
			if ( $percent_width < 100 ) {
			    $barcolor = 'WHITE';
			} else {
			    $percent_width = 110;
			    $barcolor = 'FF6666';
			}

			$table_data .= qq|<TABLE CELLSPACING=0 CELLPADDING=0 BORDER=0><TR BGCOLOR="$barcolor"><TD BGCOLOR="BLUE" WIDTH=$percent_width>.</TD><TD BGCOLOR="$barcolor" NOWRAP> &nbsp; <A HREF="$allocation_url">$percent_allocated % </A></TD></TR></TABLE>|;

			$total_allocated = 0;

		    } else {

			$table_data .= qq|<A HREF="$allocation_url">$adtype</A>|;

		    } # if page_impressions

		} # if $ad_position
		
		$table_data .= "\t";
		
	    }  # for ad_position (1..5)    
	

	    # Reserved pages link for this site:
	    
	    if ( $ad_restrictions =~ /\bReservedPages\=0\b/i ) {
		$table_data .= '&nbsp;';
	    } else {
		$reservations_url = "$ad_reservations_url\?siteid\=$siteid";
		$table_data .= qq|<A HREF="$reservations_url">Pages</A> &nbsp; |;
	    }

	    # Reserved domains link for this site:
	    
	    if ( $ad_restrictions =~ /\bReservedDomains\=0\b/i ) {
		$table_data .= '&nbsp;';
	    } else {
		$reservations_url = "$ad_reserved_domains_url\?siteid\=$siteid";
		$table_data .= qq| <A HREF="$reservations_url">GeoTargets</A> |;
	    }

	    $table_data .= "\t";

	    # Newsletter allocation link for this site:

	    if ( $ad_newsletter ) {
		$newsletter_url = "$ad_mailedit_url\?siteid\=$siteid";
		$table_data .= qq|<A HREF="$newsletter_url">Newsletter</A>|;   
	    } else {
		$table_data .= '&nbsp;';
	    }
	    $table_data .= "\n";
    
	} # foreach sitename
    

	&makeTable( 'HTML' );
	$page_contents = $table_contents;
	$table_contents = '';

        # Print main menu page:	

	&printWebpage;

	exit;
    
    } # if siteid
    

# Stop here if ad_position is wrong

    if ( $sitesconfig{ $siteid . 'ad_positions' } !~ /\b$ad_position\b/ ) {
	print "<BR><B>ERROR: Bad input.</B> $sitename is not configured for ad position $ad_position.</BODY></HTML>";
	exit;    
    }



# Show user which web sites share allocations with this web site:

    $allocation_shared_with = '';
    foreach $tmp_logname ( sort keys %logname2siteid ) {
	$tmp_siteid = $logname2siteid{$tmp_logname};
	next if $sitesconfig{ $tmp_siteid . 'ad_allocation_group' } ne $site_initials;
	$allocation_shared_with .= " " . $sitesconfig{ $tmp_siteid . 'sitename' } . " ";
    }


# Get number of page impressions delivered so far today:

    $page_impressions = $pages_per_siteid{ $siteid };


# Read ad impressions delivered to each ad yesterday on all sites that share ad allocations with this web site

    if ( open ( INFILE, "< $yesterday_adlog" ) ) {
	while (<INFILE>) {
	    chomp;
	    ($ad_id, $impr, $cliks, $tmp_siteid) = split(/\t/, $_);
	    if ( $tmp_siteid ne $siteid ) {
		if ( $sitesconfig{ $tmp_siteid . 'ad_allocation_group'} ne $site_initials ) {
		    next;
		}
	    }
	    $ad_banners{ $ad_id }->{'delivered'} += $impr;
	}
	close INFILE;
    } else {
	$error_messages .= "WARNING: Could not read ad stats $yesterday_adlog: $! <BR> \n";
    }


# Notify user of any ad type restrictions for this web site

    if ( $largead_position ) {
	if ( $can_run_megabanners != 1 ) {
	    $gif_errors .= "<BR>NOTICE: This site does not accomodate megabanners.<BR>\n";
	}
    } else {
	$small_ad_height_restriction = $ad_height_restrictions[$ad_position];
	if ( $small_ad_height_restriction > 0 ) {
	    $gif_errors .= "<BR>NOTICE: This site only allows 125x$small_ad_height_restriction size ads in this ad position.<BR>\n";
	}
    }



# Build a hash of ads available in ad_info.dat for this ad position:

    &getCampaignsInfo;

    if ( open( ADINFILE, "< $adinfofile" ) ) {
	while (<ADINFILE>) {
	    chomp;
	    ($ad_id, $ht, $wh, $tt, $at, $ad_name) = split(/\t/,$_);

	    # Only list ads that have proper dimensions for this ad position
	    if ( $largead_position ) {
		next if $wh < 380;
	    } else {
		next if $wh > 140;
		next if ( ($small_ad_height_restriction > 0) && ( $ht != $small_ad_height_restriction ));
	    }
	    
	    if ( $can_run_megabanners != 1 ) {
		if ( (substr($ad_id, 4, 1)) eq '6' ) {
		    $error_messages .= "DEBUG: Skipping $ad_id since this site can not run megabanners.\n" if ($debugging == 1);
		    next;
		}
	    }
		
	    if ( $verify_gifs_exists == 1 ) {
		$gif_filename = "$gif_filepath/$ad_id.gif";
		if ( !( -f $gif_filename )) {
		    $error_messages .= "NOTICE: <A HREF=\"$ad_infoedit_url\?adid=$ad_id\" TARGET=TomThumb>$ad_id</A> does not have a GIF image in $gif_filepath <BR>\n";
		    next;
		}
	    }
	    $tmp_campaign      = substr( $ad_id, 0, 4 );
	    $tmp_campaign_name = $ad_campaign_info{ $tmp_campaign }->{ 'name' } || $tmp_campaign;
	    $ad_banners{ $ad_id }->{'name'} = $ad_name || $ad_id;
	    $ad_banners{ $ad_id }->{'campaign_id'} = $tmp_campaign;
	    $ad_banners{ $ad_id }->{'campaign_name'} = $tmp_campaign_name;
	}
	close ADINFILE;
	
    } else {
	$error_messages .= "ERROR: Couldn't read $adinfofile : $! <BR> \n";
    }  # if ad_info.dat
	



    if ( $function eq 'save' ) {
	
	foreach $tmp_key ( keys %{$form->{'in'}} ) {

	    next if $tmp_key !~ /^ad_id_(\d+)$/;   # Get the input base ID

	    $idx              = $1;
	    $ad_kill          = $form->{'in'}->{ 'ad_kill_'           . $idx };
	    $ad_id            = $form->{'in'}->{ 'ad_id_'             . $idx };
	    $ad_impressions   = $form->{'in'}->{ 'ad_impressions_'    . $idx };
	    $ad_StopOnTarget  = $form->{'in'}->{ 'ad_StopOnTarget_'   . $idx };
	    $ad_StartDate     = $form->{'in'}->{ 'ad_StartDate_'      . $idx };
	    $ad_StopDate      = $form->{'in'}->{ 'ad_StopDate_'       . $idx };

	    push @replaced_entries, $ad_id;

	    # Skip this entry if it was deleted:

	    if ( $ad_kill eq 'T' ) {
		$messages .= "NOTICE: Entry removed :  $ad_id, $ad_impressions <BR>";
		next;
	    }


	    if ( $ad_impressions < 10 ) {
		next;
	    }

	    $ad_StartDate = &NiceDate2ISODate( $ad_StartDate );
	    $ad_StopDate = &NiceDate2ISODate( $ad_StopDate );

	    # Assemble new info settings string for this campaign:

	    $new_file_data .= join "\t", ( 
				       $ad_id,
				       $ad_impressions,
				       $ad_StopOnTarget,
				       $ad_StartDate,
				       $ad_StopDate,
				       "\n" );

	} # foreach form input

    } # if save



# Read all data on file, append new input data to it, and remove replaced entries:

    $file_data = $new_file_data;

    $adallocfile = "$datfilepath/ad_" . $ad_position . "_alloc.dat." . $siteid;
    if ( open( INFILE, "< $adallocfile" ) ) { 
	while ( $line = <INFILE>) {
	    next if $line !~ /(\w{10})\t/;
	    next if grep /$1/, @replaced_entries;  # Skip over deleted or edited records
	    $file_data .= $line;
	}
	close INFILE;
    } else {
	$error_messages .= "ERROR: Could not read ad_allocation file $adallocfile : $! <BR>\n";
    }


# Save new ad file data

    if ( $function eq 'save' ) {

	if ( $user_access_level < $admin_access ) {
	    $error_messages .= "ERROR: Can not save file: User $username does not have ad system control privileges. <BR>\n"; 
	} else {
	    &BackupAndSaveFile( "$adallocfile" );
	}

    } # if save changes



# Read all data into a sortable hash:

    $idx = '';
    foreach $line ( ( split /\n/, $file_data ) ) {
	
	( $ad_id,
	  $ad_impressions,
	  $ad_StopOnTarget,
	  $ad_StartDate,
	  $ad_StopDate ) = split /\t/, $line;

	$idx = $ad_id . $i++;

	if ( $sortby eq 'name' ) {
	    $tmp_key = $ad_banners{ $ad_id }->{'name'} . "z$idx";
	} elsif ( $sortby eq 'campaign' ) {
	    $tmp_key = $ad_banners{ $ad_id }->{'campaign_name'} . "z$idx";
	} elsif ( $sortby eq 'impressions' ) {
	    $tmp_key = $ad_impressions . "z$idx";
	} elsif ( $sortby eq 'delivered' ) {
	    $tmp_key = $ad_banners{ $ad_id }->{'delivered'} . "z$idx";
	} elsif ( $sortby eq 'StopOnTarget' ) {
	    $tmp_key = $ad_StopOnTarget . "z$idx";
	} elsif ( $sortby eq 'StartDate' ) {
	    $tmp_key = &ISODate2logdate( $ad_StartDate ) . "z$idx";
	} elsif ( $sortby eq 'StopDate' ) {
	    $tmp_key = &ISODate2logdate( $ad_StopDate ) . "z$idx";
	} else {
	    $tmp_key = $ad_id;
	} #if sortby

	$ads_list{ $tmp_key } = $line;

	$numberofads++;
	$adspercampaign{  $ad_banners{ $ad_id }->{'campaign_name'} }++;
	$total_delivered += $ad_banners{ $ad_id }->{'delivered'};

	$total_allocated += $ad_impressions if &checkAllocationDateRange;

    } # foreach data_file line

    $numberofcampaigns = ( keys %adspercampaign );
    $available_inventory = $page_impressions > 1 ? ( $page_impressions - $total_allocated ) : '';

    $percent_allocated = $page_impressions > 1 ? substr( (( $total_allocated / $page_impressions ) * 100 ), 0, 4) . ' %' : '';

    $table_title = "Allocation Summary for $sitename $adtypes{$ad_position} Ad Position $ad_position";
    $table_data = join "\n", ( qq|Number of $adtypes{$ad_position} Ads Allocated:\t$numberofads|,
			       qq|Number of Campaigns Allocated:\t$numberofcampaigns|,
			       qq|Number of Impressions Allocated:\t$total_allocated|,
			       qq|Projected Page Impressions Today:\t$page_impressions|,
			       qq|Projected Excess Page Impressions:\t$available_inventory|,
			       qq|Projected Percentage of Pages Allocated Today:\t$percent_allocated|,
			       qq|Ad Impressions Delivered Yesterday:\t$total_delivered|,
			       ( $allocation_shared_with ? qq|$sitename Allocations Copied To:\t$allocation_shared_with| : '' ) 
			       );
    &makeTable('HTML');
    $page_contents = $table_contents;
    $table_data = '';

# Edit a specific entry:

    if ( $op eq 'edit' ) {

	if ( $user_access_level < $admin_access ) {
	    print "ERROR: User $username does not have editing priviliges on this page.";
	    exit;
	}

	foreach $tmp_key ( sort { &sortAlphaNumeric($a, $b) } keys %ads_list ) {

	    $idx++;

	    ( $ad_id,
	      $ad_impressions,
	      $ad_StopOnTarget,
	      $ad_StartDate,
	      $ad_StopDate ) = split /\t/, $ads_list{ $tmp_key };

	    $tmp_killchecked = $ad_banners{ $ad_id }->{'name'} ? '' : 'CHECKED';
	    $tmp_ad_id       = $tmp_killchecked ? "<STRIKE>$ad_id</STRIKE>" : "<A HREF=\"$ad_infoedit_url?adid=$ad_id\">$ad_id</A>";
	    $tmp_stopchecked = $ad_StopOnTarget ? 'CHECKED' : '';

	    $table_data .= join '   ', ( qq|<INPUT NAME="ad_kill\_$idx" TYPE=CHECKBOX VALUE="T" $tmp_killchecked>|,
					qq|$tmp_ad_id <INPUT NAME="ad_id\_$idx" TYPE=HIDDEN VALUE="$ad_id">|,
					qq|<INPUT NAME="ad_impressions\_$idx" TYPE=TEXT SIZE=8 MAXLENGTH=12 VALUE="$ad_impressions">|,
					qq|<INPUT NAME="ad_StopOnTarget\_$idx" TYPE=CHECKBOX VALUE="T" $tmp_stopchecked>|,
					qq|<INPUT NAME="ad_StartDate\_$idx" TYPE=TEXT SIZE=12 MAXLENGTH=12 VALUE="$ad_StartDate">|,
					qq|<INPUT NAME="ad_StopDate\_$idx" TYPE=TEXT SIZE=12 MAXLENGTH=12 VALUE="$ad_StopDate">\n| );


	} # foreach ad_id

	foreach $ad_id ( sort keys %ad_banners ) {
	    $ads_droplist .= "<OPTION>$ad_id</OPTION>\n";
	}

	for ( 1..5 ) {
	    $idx++;
	    $table_data .= join '   ', ( qq|<SELECT NAME="ad_id\_$idx">$ads_droplist</SELECT>|,
					qq|<INPUT NAME="ad_impressions\_$idx" TYPE=TEXT SIZE=8 MAXLENGTH=12>|,
					qq|<INPUT NAME="ad_StopOnTarget\_$idx" TYPE=CHECKBOX VALUE="T">|,
					qq|<INPUT NAME="ad_StartDate\_$idx" TYPE=TEXT SIZE=12 MAXLENGTH=12>|,
					qq|<INPUT NAME="ad_StopDate\_$idx" TYPE=TEXT SIZE=12 MAXLENGTH=12>\n| );
	} # for 1..2


	$table_header = join '   ', ( qq|Kill?|,
				      qq|<A HREF="$thisscript&op=edit">Ad ID</A>|,
				      qq|<A HREF="$thisscript&sortby=impressions&op=edit">Impressions</A>|,
				      qq|<A HREF="$thisscript&sortby=StopOnTarget&op=edit">StopOnTarget</A>|,
				      qq|<A HREF="$thisscript&sortby=StartDate&op=edit">StartDate</A>|,
				      qq|<A HREF="$thisscript&sortby=StopDate&op=edit">StopDate</A>| );

	$page_contents .= qq|<FORM METHOD=POST ACTION="$thisscript&op=edit">
	                   <PRE> Editing $sitename Position $ad_position Ad Allocation \n\n$table_header \n$table_data\t$table_footer</PRE>
			   <INPUT TYPE="HIDDEN" NAME="op" VALUE="edit">
			   <INPUT TYPE="SUBMIT" NAME="GoButton" VALUE="Save Changes to $sitename Position $ad_position"></FORM>|;



# Show ad allocation table for this web site and ad position:

    } else {

	$edit_link = $user_access_level >= $admin_access ? qq|[<A HREF="$thisscript&op=edit">Edit</A>]| : '';

	$table_title = "$sitename Position $ad_position Ad Allocation $edit_link";

	@table_headers = split /\n\s+/, qq|<A HREF="$thisscript">Ad ID</A>
			   <A HREF="$thisscript&sortby=name">Banner Name</A>
			   <A HREF="$thisscript&sortby=campaign">Campaign</A>
			   <A HREF="$thisscript&sortby=impressions">Impressions</A>
			   <A HREF="$thisscript&sortby=StartDate">StartDate</A>
			   <A HREF="$thisscript&sortby=StopDate">StopDate</A>
			   <A HREF="$thisscript&sortby=delivered">Delivered Yesterday</A>|;

	foreach $tmp_key ( sort { &sortAlphaNumeric($a, $b) } keys %ads_list ) {

	    ( $ad_id,
	      $ad_impressions,
	      $ad_StopOnTarget,
	      $ad_StartDate,
	      $ad_StopDate ) = split /\t/, $ads_list{ $tmp_key };

	    $tmp_campaign_name = $ad_banners{ $ad_id }->{'campaign_name'};
	    $tmp_campaign_id =  $ad_banners{ $ad_id }->{'campaign_id'};

	    $table_data .= join "\t", ( qq|<A HREF="$ad_infoedit_url?adid=$ad_id">$ad_id</A>|,
					$ad_banners{ $ad_id }->{'name'},
					qq|<A HREF="$ad_infoedit_url?adcamp=$tmp_campaign_id">$tmp_campaign_name</A>|,
					$ad_impressions,
					$ad_StartDate,
					$ad_StopDate,
					$ad_banners{ $ad_id }->{'delivered'} . "\n");

	} # foreach tmp_key

	$output_format = $numberofads > 0 ? 'PRE' : 'HTML';

	&makeTable( $output_format );
	$page_contents .= $table_contents;

	$page_contents .= "<P>$edit_link";

    } # show allocation table




# Print output web page

    &printWebpage; 


}



sub checkAllocationDateRange {

    # Only count active impressions within current date range
    
    if ( ( ! $ad_StartDate) || $todayfullsysdate >= &ISODate2SysDate( $ad_StartDate ) ) {
	$error_messages .= "DEBUG: $ad_id $todayfullsysdate >= $ad_StartDate\n" if ($debugging == 1);
	if ( ( ! $ad_StopDate ) ||  ( $todayfullsysdate < &ISODate2SysDate( $ad_StopDate ) ) ) {
	    $error_messages .= "DEBUG: $ad_id is within active date range.\n\n" if ($debugging == 1);
	    return 1;
	} else {
	    $error_messages .= "DEBUG: FALLTHRU B: $ad_id $todayfullsysdate ! < $ad_StopDate\n\n" if ($debugging == 1);
	}
    } else {
	$error_messages .= "DEBUG: FALLTHRU A: $ad_id $todayfullsysdate ! >= $ad_StartDate\n" if ($debugging == 1);
    }

    return 0;

} # sub checkAllocationDateRange


__END__













