#!/usr/bin/perl

#
# CGI interface for listing all ad banners, creating/modifying ad banners
# 
# Notes:
#	- Doesn't allow quotes in any field, breaks HTML is ads
#	- Corrects target urls missing http://
#
#
# Modified:   Kurt Gray  -- Fri Nov 19 12:33:07 EST 1999     Changed adcomp.dat to tab delimited
#             Tomas      -- 11/05/99                         Y2K passed
#             Marc Pilon -- Fri Apr 21 2000                  Rsync for Slashdot upload
#             Kurt Gray  -- Mon Apr 24 15:22:15 EDT 2000     Added www.questionexchange.com
#             Marc Pilon -- Mon Jul 17 13:00:00 EDT 2000     Added animfactory to rsync
#             Kurt Gray  -- Fri Aug  4 16:44:57 EDT 2000     Creates separate set of HTML ads just for friggin' Slashdot
#             Kurt Gray  -- Thu Aug 10 14:48:47 EDT 2000     FTPs admail_adtext*.txt from www.andover.net --> local htmlad/
#             Marc Pilon -- Mon Aug 10 13:00:00 EDT 2000     Added rsync for animfactory
#             Marc Pilon -- Mon Jul 17 13:00:00 EDT 2000     Added rsync for www.osdn.com
#             Marc Pilon -- Mon Jul 17 13:00:00 EDT 2000     Added rsync for www.newsforge.com
#             Kurt Gray  -- Thu Sep 14 17:03:51 EDT 2000     Not allowing % characters in ad_info because 
#                                                            mod_adbanner's ad_info parser chokes on it
#             Kurt Gray  -- Tue Oct 10 12:03:08 EDT 2000     Added linux.com VA ad server 
#             Kurt Gray  -- Thu Oct 12 17:48:07 EDT 2000     GIFWorks.com moved to Exodus
#             Kurt Gray  -- Wed Oct 18 13:40:41 EDT 2000     Added option to geotarget ads by country or region
#             Kurt Gray  -- Mon Nov  6 17:18:49 EST 2000     Changed admin cookie login verification
#             Kurt Gray  -- Fri Nov 10 18:45:19 EST 2000     Changed adcomp.dat ==> ad_campaign_info.dat
#             Kurt Gray  -- Mon Nov 13 18:15:00 EST 2000     Added basic ability to scan all current impressions and allocations for this ad_id
#             Kurt Gray  -- Wed Nov 22 14:53:42 EST 2000     Moved ad_info.dat to /usr/local/adsystem/dat; Removed FTP/Rsync code from here
#             Kurt Gray  -- Tue Nov 28 10:43:23 EST 2000     Fixed current allocation links for each ad
#             Kurt Gray  -- Wed Jan 31 19:15:37 EST 2001     Yowza! Total rewrite to new ad system schema



use Date::Manip;

push @INC, '/usr/local/adsystem/lib/';
require 'ad_config_lib.pl';	        # Site-specific settings for adsystem control scripts
require 'ad_util_lib.pl';	        # Common functions for adsystem scripts


# Read inputs:


    &loginUser( $editorial_staff );
    
    $op             = $1 if $form->{'in'}->{'op'}         =~ /(\w{2,12})/;
    $ad_campaign_in = $1 if $form->{'in'}->{'adcamp'}     =~ /(\w{4})/;
    $adpreview      = $1 if $form->{'in'}->{'adpreview'}  =~ /(\w{2,12})/;
    $adid_in        = $1 if $form->{'in'}->{'adid'}       =~ /(\w{2,12})/;
    $alphaindex     = $1 if $form->{'in'}->{'alphaindex'} =~ /(\w{1})/;
    $sortby         = $1 if $form->{'in'}->{'sortby'}     =~ /(\w{2,14})/;
    $sortorder      = $1 if $form->{'in'}->{'sortorder'}  =~ /(\w{2,14})/;
    $function       = 'save' if $form->{'in'}->{'GoButton'} =~ /Save/;
    $function       = 'flush' if $form->{'in'}->{'GoButton'} =~ /Delete/;

# Script settings and base inputs:


    if ( $sortorder eq 'forward' ) {
	$sortorder  = 'reverse';
    } else {
        $sortorder  = 'forward';
    }


    if ( $adid_in ) {
	if ( $user_access_level < $admin_access ) {
	    $op = 'view';
	} else {
	    $op ||= 'edit';
	}
    }

    $thisscript                = $ENV{'SCRIPT_NAME'};
    $yesterday                 = &Date::Manip::UnixDate("yesterday", '%y%m%d');
    $today                     = &Date::Manip::UnixDate("today", '%y%m%d');
    $todayfullsysdate          = &Date::Manip::UnixDate("today", '%Y%m%d');
    $adlogfile                 = $adlogpath . '/ads_' . $today . '.log';
 
    $nav_links->{ $admin_access }->{'Create New Ad'}  = "$thisscript?op=create";



# Save ad banner info:

    if ( $function eq 'save' ) {

	if ( $user_access_level < $admin_access ) {
	    print "ERROR: User $username does not have ad system admin priviliges to edit ads. <BR>\n";
	    exit;
	}

	# Check each input for this ad

	$ad_kill       = $form->{'in'}->{'ad_kill'};
	$ad_client_id  = $form->{'in'}->{'ad_client_id'};
	$ad_target_url = $form->{'in'}->{'ad_target_url'};
	$ad_alt_text   = $form->{'in'}->{'ad_alt_text'};
	$ad_width      = $form->{'in'}->{'ad_width'};
	$ad_height     = $form->{'in'}->{'ad_height'};	
	$ad_geotarget  = $1 if $form->{'in'}->{'Geotarget'} =~ /^(\w\w)/;
	$ad_html_layer = $form->{'in'}->{'ad_html_layer'};
	$ad_image_type = substr( $form->{'in'}->{'upfile'}->{'value'}, 0, 4 );
	$ad_type       = substr( $adid_in, 4, 1 );


	# Make sure adid input is valid:

	if ( $adid_in =~ /^(\w{4}\d{4})\w{2}/ ) {
	    $adid = $1 . 'en'; # Deactivated geotargeting Jan 8, 2001: mod_adbanners was overloading web servers
	    # substr($adid, 8, 2) = $ad_geotarget if $ad_geotarget;
	} else {
	    $error_messages .= "<FONT COLOR=RED>ERROR:</FONT> Ad ID is not valid: Must be four letters, four digits, then two letters.<BR>\n";
	    $ignoring_input++;
	    $dontsaveinput++;
	    $op = 'edit';
	}


	# Is this ad being deleted:

	if ( $ad_kill eq 'T' ) {
	    $messages .= "NOTICE: Ad banner DELETED : $adid $ad_target_url <BR>";	    
	    push @replaced_records, $adid;
	    &deleteAdImages( $adid );
	    $ignoring_input++;
	    $ad_campaign_in = substr( $adid, 0, 4 );  # Return to campaign ads list
	    $adid = '';
	    $op = '';
	}
    

	if ( ! $ignoring_input ) {

	    push @replaced_records, $adid;

	    # Size inputs: Custom ad size inputs override size radio buttons
	    
	    if ( $form->{'in'}->{'SizeRadio'} eq 'Size468' ) {
		$ad_width  ||= 468;
		$ad_height ||= 60;
	    } elsif ( $form->{'in'}->{'SizeRadio'} eq 'Size125' ) {
		$ad_width  ||= 125;
		$ad_height ||= 125;
	    }
	    
	    # Clean up target url, client ID, and alt text (some characters can crash mod_adbanners' ad_info parser)
	    
	    $ad_html_layer =~ s/[\n\r]+/\n/g;   # Soft-carriage returns replaced with hard carriage returns
	    $ad_target_url =~ s/[\"\s]//g;      # Double-quotes and spaces in target URL would be bad
	    $ad_target_url = 'http://' . $ad_target_url if ($ad_target_url !~ 'tp://');  # Target URL needs http:// prefix
	    $error_messages .= "<FONT COLOR=RED>WARNING:</FONT> Bad characters (either \" or \%) in target url were removed.<BR>\n" if ( $ad_target_url =~ s/[\"\%]//g );
	    $error_messages .= "<FONT COLOR=RED>WARNING:</FONT> Bad characters (either \" or \%) in client id were removed.<BR>\n" if ( $ad_client_id =~ s/[\"\%]//g );
	    $error_messages .= "<FONT COLOR=RED>WARNING:</FONT> Bad characters (either \" or \%) in alt text were removed.<BR>\n" if ( $ad_alt_text =~ s/[\"\%]//g );
	    
        
	    # If GIF file is being uploaded then save it
    
	    if ( $ad_image_type ) {
		if ( $ad_image_type eq 'GIF8' ) {
		    
		    $giffilename      = $adid . '.gif';
		    $gifimagedestpath = "$ad_banner_path/$giffilename";
		    
		    if ( open ( UPFILE, "> $gifimagedestpath") ) {
			binmode UPFILE;
			print UPFILE $form->{'in'}->{'upfile'}->{'value'};
			close UPFILE;
			
			if ( -f 'gifquery' ) {
			    $fileinfo = `./gifquery $gifimagedestpath`;
			    foreach $infoline ( split(/\n/, $fileinfo)) {
				($label, $value) = split(':' ,$infoline);
				$numcolors = $value if $label eq 'Numberofcolors';
				$filesize = $value if $label eq 'Filesize';
				$width = $value if $label eq 'Gifwidth';
				$height = $value if $label eq 'Gifheight';				
			    }
			    $status_messages .= "NOTICE: New GIF image saved as $gifimagedestpath. Images stats: $width x $height, $numcolors colors, $filesize bytes<BR>\n";
			    if ( $filesize > 30000 ) {
				$error_messages .= "<B>WARNING! GIF file is too big.</B> The GIF image you just uploaded is $filesize bytes \
					    which exceeds reasonable size limits for ads ( 12k-15k ) Prepare to be yelled at.<BR>\n";
			    }
			} else {
			    $status_messages .= "NOTICE: Image data stats not available : gifquery tool not found. (Not a big deal)<BR>\n";
			}
		    } else {
			$error_messages .= "<FONT COLOR=RED>ERROR:</FONT> Error saving GIF image $gifimagedestpath: $! <BR>\n";
		    } # if open UPFILE
		} else {
		    $error_messages .= "<FONT COLOR=RED>ERROR:</FONT> Error saving GIF image $gifimagedestpath: Selected file is not a GIF image. <BR>\n";
		} # if GIF8
	    } # if upload ad_image_type


    
	    # Save HTML layer for HTML ads:
    
	    if ( $ad_type gt '4' ) {
		
		( $ad_html_layer, $Megabanner_b ) = split ( /\nMEGABANNER_SEPARATOR\n/, $ad_html_layer );
		
		# Save the HTML ad layer file:
		
		if ( open( OUTFILE, "> $ad_htmlad_path/ad$adid.dat" ) ) {
		    print OUTFILE $ad_html_layer;
		    close OUTFILE;
		    $status_messages .= "NOTICE: SAVED HTML ad layer file $ad_htmlad_path/ad$adid.dat <BR>\n";
		} else {
		    $error_messages .= "<FONT COLOR=RED>ERROR:</FONT> Could not create HTML ad layer file $ad_htmlad_path/ad$adid.dat : $! <BR>\n";
		}
		
	
		# If this is a megabanner, then save the MEGABANNER part B ad layer file
		
		if ( $Megabanner_b ) {
		    if ( open( OUTFILE, "> $ad_htmlad_path/ad$adid-b.dat" ) ) {
			print OUTFILE $Megabanner_b;
			close OUTFILE;
			$status_messages .= "NOTICE: SAVED megabanner ad layer file $ad_htmlad_path/ad$adid-b.dat <BR>\n";
		    } else {
			$error_messages .= "<FONT COLOR=RED>ERROR:</FONT> Could not create MEGABANNER-B ad layer file $ad_htmlad_path/ad$adid-b.dat : $! <BR>\n";
		    }
		}	    
	

# TODO: Move adcounter.gif to adsystem banner path

		# Create an impression counter GIF for ad layer if it doesn't exist
	
		if ( ! ( -f "$ad_banner_path/$adid.gif" ) ) {
		    $command = `/bin/cp $backupfilepath/adcounter.gif $ad_banner_path/$adid.gif`;
		    if ( -f "$ad_banner_path/$adid.gif" ) {
			$status_messages .= "NOTICE: Created $ad_banner_path/$adid.gif impression counter gif.<BR>\n";
		    } else {
			$error_messages .= "<FONT COLOR=RED>ERROR:</FONT> Could not create $ad_banner_path/$adid.gif impression counter gif: $! <BR>\n";
		    }
		}
	

		# Rewrite relative URLs in HTML ad layers for sites which don't serve ads locally:

		foreach $tmp_key ( keys %sitesconfig ) {
	
		    next if $tmp_key !~ /^(\w{7})ad_htmlad_set/;
		    $tmp_siteid       = $1;
		    next if $tmp_siteid eq 'sc000en';  # Slashcode shares htmlad layers with Slashdot
		    $tmp_htmlad_set   = $sitesconfig{ $tmp_key };
		    next if $tmp_htmlad_set !~ /\w\w/;
		    $tmp_htmlad_path  = "$ad_htmlad_path/../" . $tmp_htmlad_set;
		    $tmp_image_server = 'http://' .  $sitesconfig{ $tmp_siteid . 'ad_image_server' } . '/';
		    $tmp_html_layer   = $ad_html_layer;
		    $tmp_Megabanner_b = $Megabanner_b;
		    $tmp_html_layer   =~ s# SRC=(\W)/(\w+)# SRC=$1$tmp_image_server$2#ig;
		    $tmp_html_layer   =~ s# HREF=(\W)/(\w+)# HREF=$1$tmp_image_server$2#ig;
		    $tmp_Megabanner_b =~ s# SRC=(\W)/(\w+)# SRC=$1$tmp_image_server$2#ig;
		    $tmp_Megabanner_b =~ s# HREF=(\W)/(\w+)# HREF=$1$tmp_image_server$2#ig;
       

# TODO: Copy *.gif from htmlad to each htmlad_set directory... or rsync them up in ad_sendfiles.pl

	
		    # Save the HTML ad layer file for this web site:
		    
		    if ( open( OUTFILE, "> $tmp_htmlad_path/ad$adid.dat" ) ) {
			print OUTFILE $tmp_html_layer;
			close OUTFILE;
			$status_messages .= "NOTICE: HTML ad layer file $tmp_htmlad_path/ad$adid.dat saved. <BR>\n";
		    } else {
			$error_messages .= "<FONT COLOR=RED>ERROR:</FONT> Could not create HTML ad layer file $tmp_htmlad_path/ad$adid.dat : $! <BR>\n";
		    }
	
	
		    # If this is a megabanner, then save the MEGABANNER part B ad layer file
		    
		    if ( $Megabanner_b ) {
			if ( open( OUTFILE, "> $tmp_htmlad_path/ad$adid-b.dat" ) ) {
			    print OUTFILE $tmp_Megabanner_b;
			    close OUTFILE;
			    $status_messages .= "NOTICE: HTML MEGABANNER-B ad layer file $tmp_htmlad_path/ad$adid-b.dat saved. <BR>\n";
			} else {
			    $error_messages .= "<FONT COLOR=RED>ERROR:</FONT> Could not create MEGABANNER-B ad layer file $tmp_htmlad_path/ad$adid-b.dat : $! <BR>\n";
			}
		    }	
		    
		} # foreach ad image server
		
	    } # if ad_html_layer
    

	    $new_file_data = join "\t", ( $adid,
					  $ad_height,
					  $ad_width,
					  $ad_target_url,
					  $ad_alt_text,
					  $ad_client_id . "\n" );


	} # if not ignoring_input

    } # if save




# Flush a series of ads down the drain:

    if ( $function eq 'flush' ) {
	if ( $user_access_level < $admin_access ) {
	    print "ERROR: User $username does not have ad system admin priviliges to delete ads. <BR>\n";
	    exit;
	}
	foreach $tmp_key ( keys %{$form->{'in'}} ) {
	    next if $tmp_key !~ /^ad_kill_(\w+)$/;
	    $tmp_adid = $1;
	    if ( $form->{'in'}->{ $tmp_key } eq 'T' ) {
		$messages .= "NOTICE: Ad banner DELETED : $tmp_adid <BR>\n";
		push @replaced_records, $tmp_adid;
		&deleteAdImages( $tmp_adid );
	    }
	}
	$function = 'save' if @replaced_records;
	$ad_campaign_in = substr( $tmp_adid, 0, 4 ); # Return to campaign ads list
    } # if flush



# Read all ad data on file, replace overwritten data, append new input data to it:

    $file_data = $new_file_data;

    if ( open( INFILE, "< $adinfofile" ) ) { 
	while ( $line = <INFILE>) {
	    next if $line !~ /(\w+)\t/;
	    next if grep /$1/, @replaced_records;  # Skip over deleted or edited records
	    $file_data .= $line;
	}
	close INFILE;
    } else {
	$error_messages .= "ERROR: Could not read ad_info file $adinfofile : $! <BR>\n";
    }



# Save new ad_info file:

    if (( $function eq 'save' ) && ( ! $dontsaveinput )) {

	if ( $user_access_level < $admin_access ) {
	    $error_messages .= "ERROR: Can not save file: User $username does not have write privileges to this interface. <BR>\n"; 
	} else {
	    &BackupAndSaveFile( "$adinfofile" );
	}

    } # if save changes



# Create a new ad:

    if ( $op eq 'create' ) {

	$ad_html_layer = "<TR><TD VALIGN=TOP><FONT FACE=\"ARIAL,HELVETICA\" SIZE=3><B>HTML Layer:</B></FONT></TD><TD> <TEXTAREA WRAP=OFF ROWS=12 COLS=49 NAME=\"ad_html_layer\"></TEXTAREA> </TD></TR>";

	$op = 'edit';

    } # if create





# Get info for given ad campaign and its banners:

    $ad_campaign       = $ad_campaign_in || substr( $adid_in, 0, 4 );
    &getCampaignsInfo( $ad_campaign );       # Make global hash of all ad campaign ids => name, numberofads
    $ad_campaign_name  = $ad_campaign_info{ $ad_campaign }->{'name'} || $ad_campaign;
    &setCampaignNavlinks if $ad_campaign;    # Set sales links for given campaign
    $adid              = $1 if $adid_in =~ /(\w{2,12})/;
    &listSitesAllocatedOn if $adid; 




# Edit a given ad:

    if ( $op eq 'edit' ) {

	$ad_width        = $ad_banner_info{ $adid }->{ 'width'      };
	$ad_height       = $ad_banner_info{ $adid }->{ 'height'     };
	$ad_target_url   = $ad_banner_info{ $adid }->{ 'targeturl'  };
	$ad_alt_text     = $ad_banner_info{ $adid }->{ 'alttext'    };
	$ad_client_id    = $ad_banner_info{ $adid }->{ 'name'       };
	$ad_preview_html = $ad_banner_info{ $adid }->{ 'HTML'       };
	$ad_type         = $ad_banner_info{ $adid }->{ 'adtype'     };
	$ad_html_layer   = $ad_banner_info{ $adid }->{ 'html_layer' };
	$ad_html_layer   .= "\n\nMEGABANNER_SEPARATOR\n\n" . $ad_banner_info{ $adid }->{ 'mega_layer' } if $ad_type eq '6';


	# Show ad preview?

	if ( $ad_preview_html ) {
	    if ( $adpreview eq 'off' ) {
		$ad_preview_html = qq|Preview mode if off. <A HREF="$thisscript\?adid=$adid">Click here to see preview of $adid.</A>|;
	    } else {
		$ad_preview_html = qq|
		Preview mode is on. <A HREF="$thisscript\?adid=$adid&adpreview=off">Click here to turn off the preview</A> 
		if this page is messed up.<BR> (Some poorly written HTML ads break the ad preview feature.)
		<BR><BR><CENTER> $ad_preview_html </CENTER>|;
	    }
	} # if ad_preview_html

	
	# Adjust adsize inputs:

	$adsize_inputs  = qq|<INPUT TYPE=RADIO NAME="SizeRadio" VALUE=Size468> 468x60 &nbsp; &nbsp; 
	                     <INPUT TYPE=RADIO NAME="SizeRadio" VALUE=Size125> 125x125 &nbsp;  &nbsp;
	             Custom: <INPUT TYPE=TEXT SIZE=3 NAME=ad_width>x<INPUT TYPE=TEXT SIZE=3 NAME=ad_height> </FONT>|;

    
	if ( ( $ad_width == 468 ) && ( $ad_height == 60 ) ) {
	    $adsize_inputs =~ s/Size468/Size468 CHECKED/;
	} elsif ( ( $ad_width == 125 ) && ( $ad_height == 125 ) ) {
	    $adsize_inputs =~ s/Size125/Size125 CHECKED/;
	} else {
	    $adsize_inputs =~ s/ad_width/ad_width VALUE=$ad_width/;
	    $adsize_inputs =~ s/ad_height/ad_height VALUE=$ad_height/;
	}


	# Insert TEXTAREA input on HTML ads:

	if (( $ad_type gt '4' ) || ( ! $adid )) {
	    $ad_html_layer_input = qq|<TR><TD VALIGN=TOP><B>HTML Layer:</B></TD><TD> 
		<TEXTAREA WRAP=OFF ROWS=12 COLS=65 NAME="ad_html_layer">$ad_html_layer</TEXTAREA> </TD></TR>|;
	}
	
	
	$page_contents .= qq|<FORM METHOD="POST" ENCTYPE='multipart/form-data' ACTION="$thisscript?adid=$adid">
	    <TABLE BORDER=0 CELLPADDING=4 CELLSPACING=4 BGCOLOR=WHITE>
	    <TR BGCOLOR=DDDDDD><TD COLSPAN=2> $ad_preview_html </TD></TR>
	    $sites_allocated_on
	    <TR><TD ALIGN=RIGHT><B>Ad ID:</B></FONT></TD><TD> <INPUT SIZE="11" MAXLENGTH="10" NAME="adid" VALUE="$adid"> 
	    &nbsp;  &nbsp; <!-- Geotarget list removed --> </TD></TR>
            <TR><TD ALIGN=RIGHT><B>Client ID:</B></TD><TD> <INPUT SIZE="40" MAXLENGTH=50 NAME="ad_client_id" VALUE="$ad_client_id"> </TD></TR>
            <TR><TD ALIGN=RIGHT><B>GIF Image:</B></TD><TD> <INPUT TYPE=FILE NAME=upfile></TD></TR>
	    <TR><TD ALIGN=RIGHT><B>Size:</B></TD><TD> $adsize_inputs </TD></TR>
            <TR><TD ALIGN=RIGHT><B>Target URL:</B></TD><TD> <INPUT SIZE="65" NAME="ad_target_url" VALUE="$ad_target_url"> </TD></TR>
            <TR><TD ALIGN=RIGHT><B>Alt Text:</B></FONT></TD><TD> <INPUT SIZE="65" MAXLENGTH="60" NAME="ad_alt_text" VALUE="$ad_alt_text"> </TD></TR>
            $ad_html_layer_input
            <TR BGCOLOR=DDDDDD><TD>
		<INPUT TYPE="CHECKBOX" NAME="ad_kill" VALUE="T"> Delete this ad </TD><TD>
		<INPUT TYPE="SUBMIT" NAME="GoButton" VALUE="Save Changes"></TD></TR>
            </TABLE>
	    </FORM>|;



# View an ad banner:

    } elsif ( $op eq 'view' ) {

	$page_contents .= qq|<TABLE BGCOLOR=WHITE><TR><TD COLSPAN=2> $ad_banner_info{ $adid }->{'infotable'} </TD></TR>
	                     <TR BGCOLOR=DDDDDD><TD COLSPAN=2 ALIGN=MIDDLE> $ad_banner_info{ $adid }->{'HTML'} </TD></TR>
			     $sites_allocated_on </TABLE> |;


# Script was called with an ad campaign as an argument: list all ad's in this campaign


    } elsif ( $ad_campaign_in ) {

	# Give admin option to flush ads:

	if ( $user_access_level >= $admin_access ) {
	    $ad_kill_header = 'Kill?';
	    $ad_kill_form_open  = qq|<FORM METHOD=POST ACTION="$thisscript">|;
	    $ad_kill_form_close = '<INPUT TYPE=SUBMIT NAME=GoButton Value="Delete Selected Ads"></FORM>';
	}

	# Read ad impressions for all ad campaigns delivered today:

	&listAdImpressions( $ad_campaign_in, 'adid');

	$number_of_ads = $ad_campaign_info{ $ad_campaign_in }->{ 'numberofads' } * 1;

	# Setup key by which list will be sorted:
	
	$idx = 0;
	foreach $tmp_adid (sort keys %ad_banner_info ) {
	    
	    $idx++;

	    $tmp_width       = $ad_banner_info{ $tmp_adid }->{ 'width' };
	    $tmp_height      = $ad_banner_info{ $tmp_adid }->{ 'height' };
	    $tmp_name        = $ad_banner_info{ $tmp_adid }->{ 'name' };
	    $tmp_impressions = $ad_impressions{ $tmp_adid } * 1;

	    if ( $sortby eq 'clientid' ) {
		$tmp_key = $tmp_name . "z$idx";
	    } elsif ( $sortby eq 'impressions' ) {
		$tmp_key = $tmp_impressions . "z$idx";
	    } elsif ( $sortby eq 'size' ) {
		$tmp_key = $tmp_width . "z$idx";
	    } else {
		$tmp_key = $tmp_adid . "z$idx";
	    }
	    
	    $sort_hash{ $tmp_key } = join "\t", ( $tmp_adid, $tmp_width .'x'. $tmp_height, $tmp_name, $tmp_impressions );
	    
	}  # foreach tmp_campaign


	# Sort list keys forward or in reverse, alphabetic or numeric depending on sorting column: 

	if (( $sortby eq 'impressions' ) || ( $sortby eq 'size' )) { 
	    if ( $sortorder eq 'reverse' ) {
		@tmp_keys = ( reverse sort { $a <=> $b } keys %sort_hash );
	    } else {
		@tmp_keys = ( sort { $a <=> $b } keys %sort_hash );
	    }
	} else {
	    if ( $sortorder eq 'reverse' ) {
		@tmp_keys = ( reverse sort keys %sort_hash );
	    } else {
		@tmp_keys = ( sort keys %sort_hash );
	    }
	}


	# Make sorted list of all ad campaigns:

	foreach $tmp_key ( @tmp_keys ) {
	    ( $tmp_adid, $tmp_size, $tmp_name, $tmp_impressions ) = split /\t/, $sort_hash{ $tmp_key };
	    $table_data .= qq|<INPUT TYPE=CHECKBOX NAME=ad_kill_$tmp_adid VALUE=T>\t| if $ad_kill_form_open;
	    $table_data .= qq|<A HREF="$thisscript?adid=$tmp_adid">$tmp_adid</A>\t|;
	    $table_data .= "$tmp_size\t\t";
	    $table_data .= "$tmp_impressions\t";
	    $table_data .= qq|<A HREF="$thisscript?adid=$tmp_adid">$tmp_name</A>\n|;
	} # foreach tmp_campaign

	$table_headers = join "\t", ( $ad_kill_header,
				      qq|  <A HREF="$thisscript?adcamp=$ad_campaign_in&sortorder=$sortorder">Ad ID</A>|,
				      qq|\t  <A HREF="$thisscript?adcamp=$ad_campaign_in&sortby=size&sortorder=$sortorder">Size</A>|,
				      qq|   <A HREF="$thisscript?adcamp=$ad_campaign_in&sortby=impressions&sortorder=$sortorder">Delivered Today</A>|,
				      qq|<A HREF="$thisscript?adcamp=$ad_campaign_in&sortby=clientid&sortorder=$sortorder">Client ID</A>| );

 
	$page_contents .= $ad_kill_form_open if $ad_kill_form_open;
	$page_contents .= qq|<PRE><B>$ad_campaign_name Ad Banners: $number_of_ads Total</B>\n\n<B>$table_headers</B>\n\n$table_data</PRE>|;
	$page_contents .= qq|$table_contents $ad_kill_form_close|;





# List all campaigns:

    } else {


	@table_headers = ( qq|<A HREF="$thisscript?sortby=impressions&sortorder=$sortorder">Delivered Today</A>|,
			   qq|<A HREF="$thisscript?sortby=numberofads&sortorder=$sortorder">\# Ads</A>|,
			   qq|<A HREF="$thisscript?sortby=campid&sortorder=$sortorder">Campaign ID</A>|,
			   qq|<A HREF="$thisscript?sortby=campname&sortorder=$sortorder">Campaign Name</A>| );



	# Read ad impressions for all ad campaigns delivered today:

	&listAdImpressions( '*', 'campaign');
	
	$number_of_ads = 0;
	$number_of_campaigns = ( keys %ad_campaign_info );
	
	# Setup key by which list will be sorted:
	
	$idx = 0;
	foreach $tmp_campaign (sort keys %ad_campaign_info ) {

	    next if ( $alphaindex ) && ( $tmp_campaign !~ /^$alphaindex/i );
	    
	    $idx++;
	    $tmp_impressions = $ad_impressions{ $tmp_campaign } * 1;
	    $tmp_campname    = $ad_campaign_info{ $tmp_campaign }->{ 'name' } || "- unnamed campaign -";
	    $tmp_numberofads = $ad_campaign_info{ $tmp_campaign }->{ 'numberofads' } * 1;
	    $number_of_ads  += $tmp_numberofads;

	    if ( $sortby eq 'campname' ) {
		$tmp_key = $tmp_campname . "z$idx";
	    } elsif ( $sortby eq 'impressions' ) {
		$tmp_key = $tmp_impressions . "z$idx";
	    } elsif ( $sortby eq 'numberofads' ) {
		$tmp_key = $tmp_numberofads . "z$idx";
	    } else {
		$tmp_key = $tmp_campaign . "z$idx";
	    }
	    
	    $sort_hash{ $tmp_key } = join "\t", ( $tmp_campaign, $tmp_campname, $tmp_impressions, $tmp_numberofads );
	    
	}  # foreach tmp_campaign


	# Sort list keys forward or in reverse, alphabetic or numeric depending on sorting column: 

	if (( $sortby eq 'impressions' ) || ( $sortby eq 'numberofads' )) { 
	    if ( $sortorder eq 'reverse' ) {
		@tmp_keys = ( reverse sort { $a <=> $b } keys %sort_hash );
	    } else {
		@tmp_keys = ( sort { $a <=> $b } keys %sort_hash );
	    }
	} else {
	    if ( $sortorder eq 'reverse' ) {
		@tmp_keys = ( reverse sort keys %sort_hash );
	    } else {
		@tmp_keys = ( sort keys %sort_hash );
	    }
	}


	# Make sorted list of all ad campaigns:

	foreach $tmp_key ( @tmp_keys ) {
	    ( $tmp_campaign, $tmp_campname, $tmp_impressions, $tmp_numberofads ) = split /\t/, $sort_hash{ $tmp_key };
	    $table_data .= "$tmp_impressions\t";
	    $table_data .= "$tmp_numberofads\t";
	    $table_data .= qq|<A HREF="$thisscript?adcamp=$tmp_campaign">$tmp_campaign</A>\t|;
	    $table_data .= "$tmp_campname\n";
	} # foreach tmp_campaign

	$letterbar = qq| <B>List Index:</B> <A HREF="$thisscript">All</A> |;
	for $i (A..Z) {	    
	    $letterbar .= $alphaindex eq $i ? " [$i] "  : qq| <A HREF="$thisscript?alphaindex=$i">$i</A> |;
	}
	$page_contents .= $letterbar;

	$table_title = qq|$number_of_ads Ad Banners Listed By Campaign|;
	&makeTable( 'PRE' );

	$page_contents .= "$table_contents$ad_kill_form_close<PRE><B>Totals:</B>\n\n$ad_impressions_total Impressions      $number_of_ads Ads";
	$page_contents .= "      $number_of_campaigns Campaigns </PRE>";
    
	
    } # if QUERY_STRING





# Output web page:

    &printWebpage;




# End of script







# sub deleteAdImages: Removes the GIF image and HTML layers of given (local) adid

sub deleteAdImages {

    my $tmp_adid = shift;

    # Delete the GIF image of the ad
		    
    if ( unlink( "$ad_banner_path/$tmp_adid.gif" ) ) {
	$messages .= "NOTICE: DELETED ad image $tmp_adid.gif <BR>\n";
    } else {
	$error_messages .= "<FONT COLOR=RED>WARNING:</FONT> $ad_banner_path/$tmp_adid.gif was not deleted: $! <BR>\n";
    }
		    
    # If it's an HTML ad, delete its HTML layer file too
		    
    if ( (substr($tmp_adid,4,1)) gt '4') {
	if ( unlink( "$ad_htmlad_path/ad$tmp_adid.dat" ) ) {
	    $messages .= "NOTICE: DELETED html layer ad$tmp_adid.dat <BR>\n";
	} else {
	    $error_messages .= "<FONT COLOR=RED>WARNING:</FONT> $ad_htmlad_path/ad$tmp_adid.dat was not deleted: $! <BR>\n";
	}
    }
    
} # sub deleteAdImages





# listAdImpressions:  Read given ad log file and tally ad impressions by banner or campaign
#
#        Inputs:    $adid      (local)        Either a banner ID or ad campaign ID to read stats for
#                   $group_by   (local)        How to group stats hash: 'campaign', 'siteid', 'adid'
#
#        Output:    %ad_impressions (global)   Hash of ad impressions per banner, campaign, or siteid

sub listAdImpressions {

    my $adid                 = shift;    # First argument is an adid or ad_campaign (optional)
    my $ad_stats_group_by    = shift;    # Second argument is how to group stats: 'campaign', 'siteid', 'adid'
    my $ad_stats_group       = '';
       $ad_impressions       = ();
       $ad_impressions_total = 0;

    $adid  = '' if $adid !~ /\w{4,12}/;

    if ( open ( INFILE, "< $adlogfile" ) ) {
	while (<INFILE>) {
	    chomp;
	    ($tmp_adid, $impr, $cliks, $tmp_siteid) = split(/\t/, $_);
	    next if ( $adid ) && ( $tmp_adid !~ /^$adid/ );
	    if ( $ad_stats_group_by eq 'campaign' ) {
		$ad_stats_group = substr($tmp_adid, 0, 4);
	    } elsif ( $ad_stats_group_by eq 'siteid' ) {
		if ( $sitesconfig{ $tmp_siteid . 'ad_allocation_group' } =~ /([a-z][a-z])/ ) {    # 'XX' used for at000en
		    $tmp_siteid = $1 . '000en';
		}
		$ad_stats_group = $tmp_siteid;
	    } else {
		$ad_stats_group = $tmp_adid;
	    }
	    $ad_impressions{$ad_stats_group} += $impr;
	    $ad_impressions_total += $impr;
	}
	close INFILE;
    } else {
	$error_messages .= "WARNING: Scanning ad stats by $ad_stats_group_by : Could not read ad stats log $adlogfile: $! <BR>\n";
    }

} # sub listImpressions




# listSitesAllocatedOn: List all sites and positions where this ad is currently allocated:
#
# Inputs:         $adid  (global)    ID of the ad to find all current allocations of
#
# Outputs:        $sites_allocated_on  (global)  HTML table of sites this ad is allocated on and current impressions
#


sub listSitesAllocatedOn {

    my $sites_alloctated_rows           = '';
    my @allocfiles                      = ();
    my $number_of_positions_allocated   = 0;
    my $number_of_sites_allocated       = 0;
    my $allocated                       = ();
       $sites_allocated_on              = '';

    &listAdImpressions( $adid, 'siteid' );

    if ( opendir DIR, $datfilepath ) {
	@allocfiles = grep /alloc\.dat/, readdir DIR;
	closedir DIR;
	if ( ! @allocfiles ) {
	    $error_messages .= "WARNING: Can not scan for allocations : No allocation files found in $datfilepath : $! <BR>";
	}
	foreach $allocfile ( sort @allocfiles ) {

	    # Find this ad in general rotation on any site:

	    if ( $allocfile =~ /_(\d)_alloc\.dat\.(\w{4,8})$/ ) {

		$tmp_ad_position = $1;
		$tmp_siteid = $2;
		next if $sitesconfig{ $tmp_siteid . 'ad_allocation_group' } =~ /\w\w/; # Skip subsites
		if ( open INFILE, "< $datfilepath/$allocfile" ) {
		    while (<INFILE>) {
			next if $_ !~ /$adid\t(\d+)\t(\w?)\t([\w\/]*)\t([\w\/]*)/;
			$tmp_stopontarget = ( $2 eq 'T' ? '' : '+');
			$tmp_allocation   = "($1$tmp_stopontarget)";
			$tmp_startdate    = $3;
			$tmp_stopdate     = $4;
			if ( $tmp_stopdate ) {
			    if ( &ISODate2SysDate($tmp_stopdate) <= $todayfullsysdate ) {
				$tmp_allocation = "(<STRIKE>$1</STRIKE>)";
			    }
			}
			$number_of_positions_allocated++;
			$allocated{ $tmp_siteid } .= qq|<A HREF="$ad_allocedit_url?siteid=$tmp_siteid&adpos=$tmp_ad_position" TARGET=TomThumb>$tmp_ad_position $tmp_allocation</A> |;
		    }
		    close INFILE;
		} else {
		    $error_messages .= "WARNING: Can not read allocation file $datfilepath/$allocfile : $! <BR>";
		}

	    # Find this ad in any newsletter allocation:

	    } elsif ( $allocfile eq 'ad_mail_alloc.dat' ) {

		if ( open INFILE, "< $datfilepath/$allocfile" ) {
		    while (<INFILE>) {
			next if $_ !~ /\{ '(\w+)' \} = '$adid'/;
			$tmp_siteid = $1;
		     # TODO: Workout the next few lines of hardcoding by $siteid:
			$tmp_siteid = 'at000en' if $tmp_siteid eq 'newsletter';
			$tmp_siteid = 'af000en' if $tmp_siteid eq 'anim';
			$tmp_siteid = 'os000en' if $tmp_siteid eq 'osdn';
			$allocated{ $tmp_siteid } .= qq|<A HREF="$ad_mailedit_url?siteid=$tmp_siteid" TARGET=TomThumb>Newsletter</A> |;
			$number_of_positions_allocated++;
		    }
		    close INFILE;
		} else {
		    $error_messages .= "WARNING: Can not read allocation file $datfilepath/$allocfile : $! <BR>";
		}

	    # Find this ad on any reserved pages:

	    } elsif ( $allocfile =~ /ad_reservations_alloc\.dat\.(\w{4,8})/ ) {
		$tmp_siteid = $1;
		if ( open INFILE, "< $datfilepath/$allocfile" ) {
		    while (<INFILE>) {
			next if $_ !~ /$adid/;
			$allocated{ $tmp_siteid } .= qq|<A HREF="$ad_reservations_url?siteid=$tmp_siteid" TARGET=TomThumb>ReservedPages</A> |;
			$number_of_positions_allocated++;
		    }
		    close INFILE;
		} else {
		    $error_messages .= "WARNING: Can not read allocation file $datfilepath/$allocfile : $! <BR>";
		}


	    } elsif ( $allocfile =~ /ad_reserved_domains_alloc\.dat\.(\w{4,8})/ ) {
		$tmp_siteid = $1;
		if ( open INFILE, "< $datfilepath/$allocfile" ) {
		    while (<INFILE>) {
			next if $_ !~ /$adid/;
			$allocated{ $tmp_siteid } .= qq|<A HREF="$ad_reserved_domains_url?siteid=$tmp_siteid" TARGET=TomThumb>GeoTargeted</A> |;
			$number_of_positions_allocated++;
		    }
		    close INFILE;
		} else {
		    $error_messages .= "WARNING: Can not read allocation file $datfilepath/$allocfile : $! <BR>";
		}

	    } else {
		$error_messages .= "DEBUG: Skipping $datfilepath/$allocfile <BR>";
	    }
	} # foreach allocfile

	$number_of_sites_allocated = ( keys %allocated );

	# List allocated positions for this ad by number of impressions on the given site:

	foreach $tmp_siteid ( sort keys %ad_impressions ) {
	    next if $ad_impressions{$tmp_siteid} < 2;
	    $sites_allocated_rows .= "\n<TR><TD><FONT FACE=ARIAL,HELVETICA>";
	    $sites_allocated_rows .= $sitesconfig{ $tmp_siteid . 'sitename' } || $tmp_siteid;
	    $sites_allocated_rows .= "</FONT></TD><TD ALIGN=LEFT><FONT FACE=ARIAL,HELVETICA>";
	    $sites_allocated_rows .= $allocated{ $tmp_siteid } || '-none-'; 
	    $sites_allocated_rows .= "</FONT></TD>";
	    $sites_allocated_rows .= "<TD ALIGN=RIGHT><FONT FACE=ARIAL,HELVETICA>" . $ad_impressions{$tmp_siteid};
	    $sites_allocated_rows .= "</FONT></TD></TR>";
	} # foreach site allocated

	# This ad may also be allocated but not have any impressions on that that site, so show those allocations too:

	foreach $tmp_siteid ( sort keys %allocated ) {
	    next if $ad_impressions{$tmp_siteid} > 10;
	    $sites_allocated_rows .= "\n<TR><TD><FONT FACE=ARIAL,HELVETICA>";
	    $sites_allocated_rows .= $sitesconfig{ $tmp_siteid . 'sitename' } || $tmp_siteid;
	    $sites_allocated_rows .= "</FONT></TD><TD ALIGN=LEFT><FONT FACE=ARIAL,HELVETICA>";
	    $sites_allocated_rows .= $allocated{ $tmp_siteid } || '-none-'; 
	    $sites_allocated_rows .= "</FONT></TD>";
	    $sites_allocated_rows .= "<TD ALIGN=RIGHT><FONT FACE=ARIAL,HELVETICA>" . $ad_impressions{$tmp_siteid};
	    $sites_allocated_rows .= "</FONT></TD></TR>";
	} # foreach site allocated

	# Put the entire allocations list table together:

	if ( $sites_allocated_rows =~ /\w+/ ) {
	    $sites_allocated_on  = "<TR><TD ALIGN=RIGHT VALIGN=TOP><FONT FACE=ARIAL,HELVETICA SIZE=3>";
            $sites_allocated_on .= "<B>Allocated On:</B></FONT></TD><TD VALIGN=TOP> ";
	    $sites_allocated_on .= "<TABLE BORDER=0 CELLSPACING=8><TR>";
	    $sites_allocated_on .= "<TD ALIGN=LEFT><FONT FACE=ARIAL,HELVETICA> <B>Site</B> </FONT></TD>";
	    $sites_allocated_on .= "<TD ALIGN=LEFT><FONT FACE=ARIAL,HELVETICA> <B>Positions</B> </FONT></TD>";
	    $sites_allocated_on .= "<TD ALIGN=RIGHT><FONT FACE=ARIAL,HELVETICA> <B>Delivered Today</B> </FONT></TD>";
	    $sites_allocated_on .= "</TR> $sites_allocated_rows </TR>";
	    $sites_allocated_on .= "<TD ALIGN=LEFT><FONT FACE=ARIAL,HELVETICA> <B>Totals:</B> $number_of_sites_allocated Sites</FONT></TD>";
	    $sites_allocated_on .= "<TD ALIGN=LEFT><FONT FACE=ARIAL,HELVETICA> $number_of_positions_allocated Positions</FONT></TD>";
	    $sites_allocated_on .= "<TD ALIGN=RIGHT><FONT FACE=ARIAL,HELVETICA> $ad_impressions_total Impressions </FONT></TD></TR></TABLE>";
	    $sites_allocated_on .= "</FONT></TD></TR>";
	} else {
	    $sites_allocated_on = "<TR><TD ALIGN=RIGHT><FONT FACE=ARIAL,HELVETICA SIZE=3><B>Allocated On:</B></FONT></TD><TD> <FONT FACE=ARIAL,HELVETICA SIZE=3> Not currently allocated anywhere. </FONT></TD></TR>";
        }
    } else {
	$error_messages .= "WARNING: Can not scan ad allocations : Can not readdir dat file path $datfilepath : $! ";
    }


} # sub listSitesAllocatedOn




__END__

