#!/usr/bin/perl

#
# This is a CGI interface for ad campaign info
#
# Notes: 
#
#
# Modified:  Kurt Gray -- Fri Nov 19 12:33:07 EST 1999	   Changed adcomp.dat to tab delimited
#            Kurt Gray -- Wed Nov  8 10:02:28 EST 2000     Moved to /usr/local/adsystem
#            Kurt Gray -- Mon Nov 13 15:13:12 EST 2000     Script settings code cleanup
#            Kurt Gray -- Wed Nov 29 18:10:43 EST 2000     Adding invoice number and other extra fields to each campaign
#            Kurt Gray -- Sun Dec  3 18:15:55 EST 2000     Arranged invoice, cpmrate, and run dates into run lists
#            Kurt Gray -- Mon Dec 18 13:48:46 EST 2000     Rearranged ad run data and added checkbox lists
#            Kurt Gray -- Wed Jan  3 20:25:37 EST 2001     Ad run data moved to a separate file: ad_campaign_runs.dat

push @INC, '/usr/local/adsystem/lib/';
require 'ad_config_lib.pl';	        # Site-specific settings for adsystem control scripts
require 'ad_util_lib.pl';	        # Common functions for adsystem scripts


# Script settings and base inputs:

    $thisscript           ||= $ENV{'SCRIPT_NAME'};

    $nav_links->{ $admin_access }->{'Create New Campaign'} = qq|$ad_compedit_url?adcamp=new1&op=edit|;


# Print top of output web page and check user login

    &loginUser( $accountants_access );


    &getActiveSitenames;    # sets %sitenames hash to render sites checkboxes for each run    
    $sitenames{ 'Andover: Run Of Network' } ||= 'atronen';
    $sitenames{ 'OSDN: Run Of Network' }    ||= 'osronen';


# Make a list of ad agencies, advertiser companies, and ad sales territories
 
    %ad_agencies =          (
			     'adau', 'Ad Auction',
			     'adws', 'Addison-Wellsey',
			     'comn', 'Community Banners',
			     'crtf', 'Carat-Freeman', 
			     'dgtm', 'Douglas-Thomsan', 
			     'doub', 'DoubleClick',
			     'hill', 'Hill-Holiday',
			     'idga', 'IDG',
			     'key3', 'Key3Media',
			     'medi', 'Media Matrix',
			     'osdn', 'OSDN In-House',
			     );


    %ad_companies =         (
			     'cmpq', 'Compaq',
			     'ibma', 'IBM',
			     'intl', 'Intel',
			     'hpkd', 'Hewlett-Packard',
			     'osdn', 'OSDN',
			     'rhat', 'RedHat',
			     'sunm', 'Sun',
			     'lnux', 'VA Linux',
			     'zdnn', 'Ziff-Davis',
			     );


    %ad_categories  =       (

			     'comn', 'Community',
			     'embd', 'Embedded Systems',
			     'fina', 'Finance',
			     'hdwr', 'Hardware',
			     'host', 'Hosting',
			     'jobs', 'Job Market',
			     'medi', 'Media',
			     'nets', 'Network Storage',
			     'netw', 'Networking',
			     'sofw', 'Software',
			     'webp', 'Web Portal',
			     'wire', 'Wireless',
			     );


    %ad_territories =       (
			     'calf', 'California',
			     'euro', 'Europe',
			     'intn', 'International',
			     'newy', 'New York',
			     'usmw', 'US Midwest',
			     'natl', 'US National',
			     'usnw', 'US NorthWest',
			     'usne', 'US NorthEast',
			     'usse', 'US SouthEast',
			     'ussw', 'US SouthWest',
			     );


    %ad_email_frequencies = (
			     'EmailAtNever',     'Never',
			     'EmailEveryday',    'Every day from Start Date to Stop Date',
			     'EmailMonthly',     'On the last day of every month',
			     'EmailOnStopDate',  'On the last day of campaign',
			     'EmailWeekly1Mon',  'Every week on Monday',
			     'EmailWeekly2Tue',  'Every week on Tuesday',
			     'EmailWeekly3Wed',  'Every week on Wednesday',
			     'EmailWeekly4Thu',  'Every week on Thursday',
			     'EmailWeekly5Fri',  'Every week on Friday',
			     );


    $dont_confirm_overwrite = 1;   # Set to 0 to force confirmation of record overwrite (BUG: Not passing previous inputs forward)


# Make a list of ad sales people based on ad_system_users who considered to be part of the sales dept

    %ad_sales_people = ();
    foreach $tmp_key ( keys %ad_system_users ) {
	$tmp_access_level = $ad_system_users{ $tmp_key }->{ 'access_level' };
	next if (( $tmp_access_level < $sales_dept ) || ( $tmp_access_level > $admin_access ));
	$ad_sales_people{ $tmp_key } = $ad_system_users{ $tmp_key } -> { 'name' };
    }



# Read query string inputs:

    $ad_campaign_in = $1 if $ENV{'QUERY_STRING'} =~ /adcamp=(\w{4})/;
    $op             = $1 if $ENV{'QUERY_STRING'} =~ /op=(\w{4,8})/;
    $op             ||= $1 if $ARGV[0]             =~ /op=(\w{4,8})/;      # Added this command line input for debugging
    $sortby         = $1 if $ENV{'QUERY_STRING'} =~ /sortby=(\w{2,14})/;
    $alphaindex     = $1 if $form->{'in'}->{'alphaindex'} =~ /(\w{1})/;


# Set default modes based on user access level

   if ( $user_access_level < $admin_access ) {
       $default_op     = 'view';
       $op             = 'view' if ( $op eq 'edit');
   } else {
       $default_op     = 'edit';
       $edit_link      = qq|<A HREF="$thisscript?adcamp=$ad_campaign_in&op=edit">Edit Record</A>|;
   } # if access_level

   $op = $default_op if ( $ad_campaign_in && ( ! $op ));

   # $alphaindex ||= 'A' if ! ( $op || $sortby );

   &getInvoicesInfo( $ad_campaign_in );


# Read edit form inputs:

    $new_file_data       =  '';
    @replaced_records    =  ();


    foreach $tmp_key ( keys %{$form->{'in'}} ) {

	$replace_confirmed     = $1 if $form->{'in'}->{ 'replace_confirmed' } =~ /(\w)/; 

	next if $tmp_key !~ /^ad_campaign_(\d+)$/;   # Get the input base ID

	$idx                   = $1;
	$ad_kill               = $form->{'in'}->{ 'ad_kill_' .              $idx };
	$ad_campaign           = $form->{'in'}->{ 'ad_campaign_' .          $idx };
	$ad_campaign_name      = $form->{'in'}->{ 'ad_campaign_name_' .     $idx };
	$ad_campaign_password  = $form->{'in'}->{ 'ad_campaign_password_' . $idx };
	$ad_company            = $form->{'in'}->{ 'ad_company_' .           $idx };
	$ad_agency             = $form->{'in'}->{ 'ad_agency_' .            $idx };
	$ad_product_category   = $form->{'in'}->{ 'ad_product_category_' .  $idx };
	$ad_territory          = $form->{'in'}->{ 'ad_territory_' .         $idx };
	$ad_emailto            = $form->{'in'}->{ 'ad_emailto_' .           $idx };
	$ad_emailat            = $form->{'in'}->{ 'ad_emailat_' .           $idx };
	$ad_StartDate          = $form->{'in'}->{ 'ad_StartDate_' .         $idx };
	$ad_contactsidc        = $form->{'in'}->{ 'ad_contactsidc_' .       $idx };
	$ad_contactsida        = $form->{'in'}->{ 'ad_contactsida_' .       $idx };

	# Skip this campaign if it was deleted:

	if ( $ad_kill eq 'T' ) {
	    $messages .= "NOTICE: Entry removed :  $ad_campaign, $ad_campaign_name <BR>";
	    push @replaced_records, $ad_campaign;
	    next;
	} 


	# Read campaign salespersons checkboxes:

	@tmp_salespersons  =  ();
	for $i ( 1..(keys %ad_sales_people) ) {
	    push ( @tmp_salespersons, $1 ) if $form->{'in'}->{ "ad_salespersons\_$idx\_$i" } =~ /(\w+)/;
	}
	$ad_salespersons = join ',', @tmp_salespersons;


	# Blank out emailat if set to 'never'
	
	if ( $ad_emailat eq 'EmailAtNever' ) {
	    $ad_emailat = '';
	}

	$ad_StartDate = &NiceDate2ISODate( $ad_StartDate );

	# Fix any blank campaign password inputs:

	if ( $ad_campaign_password !~ /\w+/ ) {
	    if ( $ad_campaign_name !~ /\w+/ ) {
	        $messages .= "NOTICE: Entry ignored : No password and campaign name enetered for '$ad_campaign' <BR>\n";
		next;
	    } else {
		$ad_campaign_password = &makeRandomPassword;
		$messages .= "NOTICE: Blank password for $ad_campaign changed to '$ad_campaign_password' <BR>";
	    }
	}


	# Assemble new info settings string for this campaign:

	push @replaced_records, $ad_campaign;

	$new_file_data .= join "\t", ( 
				       $ad_campaign,
				       $ad_campaign_name,
				       $ad_campaign_password,
				       $ad_company,
				       $ad_agency,
				       $ad_product_category,
				       $ad_salespersons,
				       $ad_territory,
				       $ad_emailto,
				       $ad_emailat,
				       $ad_StartDate,
				       $ad_contactsidc,
				       $ad_contactsida,
				       "\n" 
				       );

    } # foreach form input



# Read all ad campaign data on file, append new input data to it, and remove replaced campaigns:

    $file_data = $new_file_data;

    if ( open( INFILE, "< $adcompfile" ) ) { 
	while ( $line = <INFILE>) {
	    next if $line !~ /(\w{4})\t/;
	    if ( grep /$1/, @replaced_records ) {
		next if $ad_kill;
		if (( $dont_confirm_overwrites == 1 ) || ( $replace_confirmed )) {
		    next;
		} else {
		    $messages .= qq|WARNING: You are modifying an existing campaign [$ad_campaign] $ad_campaign_name If you 
		    are sure about this then click "Overwrite Confirmed" then click the Save button again. <BR>\n|;
                    $confirm_replace = qq|<INPUT TYPE=CHECKBOX NAME="replace_confirmed" VALUE="T" CHECKED> Overwrite confirmed|;
		    $op = 'edit';
		    $dontsavechanges++;
		}
	    }
	    $file_data .= $line;
	}
	close INFILE;
    } else {
	$error_messages .= "ERROR: Could not read ad_campaign_info file $adcompfile : $! <BR>\n";
    }



# TODO: Save and edit next record


# Save new ad password file data


   if ( $form->{'in'}->{'GoButton'} =~ /Save Changes/ ) {

       if ( $dontsavechanges ) {
       } else {
	   if ( $user_access_level < $admin_access ) {
	       $error_messages .= "ERROR: Can not save file: User $username does not have ad system control privileges. <BR>\n"; 
	   } else {
	       &BackupAndSaveFile( "$adcompfile" );
	   }
       }

    } # if save changes



# Read all campaign data into sorted arrays

    $idx = '';
    foreach $line ( ( split /\n/, $file_data ) ) {
	
	( $ad_campaign,
	  $ad_campaign_name,
	  $ad_campaign_password,
	  $ad_company,
	  $ad_agency,
	  $ad_product_category,
	  $ad_salespersons,
	  $ad_territory,
	  $ad_emailto,
	  $ad_emailat,
	  $ad_StartDate,
	  $ad_contactsidc,
	  $ad_contactsida ) = split /\t/, $line;

	next if ( $alphaindex ) && ( $ad_campaign !~ /^$alphaindex/i );

	$idx++;

	if ( $sortby eq 'name' ) {
	    $tmp_key = $ad_campaign_name . "z$idx";
	} elsif ( $sortby eq 'salesperson' ) {
	    $tmp_key = ( $ad_sales_people{ $ad_salespersons } || $ad_salespersons ) . "z$idx";
	} elsif ( $sortby eq 'password' ) {
	    $tmp_key = $ad_campaign_password . "z$idx";
	} elsif ( $sortby eq 'invoice' ) {
	    $tmp_key = $ad_invoices_info{ $ad_campaign }->{ 'invoice_id' } . "z$idx";
	} elsif ( $sortby eq 'agency' ) {
	    $tmp_key = ( $ad_agencies{ $ad_agency } || $ad_agency ) . "z$idx";
	} elsif ( $sortby eq 'company' ) {
	    $tmp_key = ( $ad_companies{ $ad_company } || $ad_company ) . "z$idx";
	} else {
	    $tmp_key = $ad_campaign;
	} #if sortby

	$campaign_list{ $tmp_key } = $line;

    } # foreach data_file line



# Make an HTML edit form for the given campaign:

    if ( $op eq 'edit' ) {

	&getCampaignInfo( $ad_campaign_in );
	$tmp_campaign      = $ad_campaign      || $ad_campaign_in;
	$tmp_campaign_name = $ad_campaign_name || $ad_campaign_in;	
	&setCampaignNavlinks;
	$page_title = "Editing Campaign $tmp_campaign_name: ";

	$page_contents .= qq|
	    <FORM METHOD=POST ACTION=$thisscript?adcamp=$tmp_campaign&op=edit>
	        <TABLE BGCOLOR=WHITE>
	        <TR BGCOLOR=DDFFCC>   
		<TD ALIGN=RIGHT><B>Campaign ID:</B></TD>
		<TD>
		    <INPUT NAME="ad_campaign_$idx" SIZE="5" MAXLENGTH="4" VALUE="$tmp_campaign">
		    &nbsp; <B>StartDate:</B> <INPUT NAME="ad_StartDate_$idx" SIZE=12 MAXLENGTH=12 VALUE="$ad_StartDate"></TD>
		</TR><TR BGCOLOR=DDFFCC>
		<TD ALIGN=RIGHT><B>Campaign Name:</B></TD>
		<TD><INPUT NAME="ad_campaign_name_$idx" SIZE="40" MAXLENGTH="40" VALUE="$ad_campaign_name"></TD>
		</TR><TR>
                <TD ALIGN=RIGHT><B>Product Category:</B></TD><TD> |;
	    
	%drop_list = %ad_categories;
	&makeDropList ( "ad_product_category_$idx", $ad_product_category );

	$page_contents .= qq| </TD>
		</TR><TR>
		<TD ALIGN=RIGHT><B>Client:</B></TD>
		<TD> |;

	# Debating on whether to show agency and company name as text fields or drop lists:

	if ( $agency_company_droplists == 1 ) {

	    $page_contents .= qq| Company: |;
	    %drop_list = %ad_companies;
	    &makeDropList ( "ad_company_$idx", $ad_company );
	    $page_contents .= qq| &nbsp; Billing Agency: |;
	    %drop_list = %ad_agencies;
	    &makeDropList ( "ad_agency_$idx", $ad_agency );
	    
	} else {

	    $page_contents .= qq| <INPUT NAME="ad_company_$idx" SIZE="30" MAXLENGTH="40" VALUE="$ad_company">
		&nbsp; <B>Contact ID</B>: <INPUT NAME="ad_contactsidc_$idx" SIZE="6" MAXLENGTH="5" VALUE="$ad_contactsidc"></TD>
                </TR><TR BGCOLOR=DDFFCC>
		</TR><TR>
                <TD ALIGN=RIGHT><B>Billing Agent:</B></TD>
                <TD> <INPUT NAME="ad_agency_$idx" SIZE="30" MAXLENGTH="40" VALUE="$ad_agency">
		&nbsp; <B>Contact ID</B>: <INPUT NAME="ad_contactsida_$idx" SIZE="6" MAXLENGTH="5" VALUE="$ad_contactsida"></TD>
                    |;

	} # if agency droplists


	$page_contents .= qq|</TD>
		</TR><TR>
		<TD ALIGN=RIGHT VALIGN=TOP><B>Account:</B></TD>
		<TD> Territory: |;

	%drop_list = %ad_territories;
	&makeDropList ( "ad_territory_$idx", $ad_territory );

	$page_contents .= qq| &nbsp; Current Sales Reps: |;

	%drop_list = %ad_sales_people;
	&makeCheckboxes ( "ad_salespersons_$idx", $ad_salespersons );

	$page_contents .= qq|</TD>
		</TR><TR BGCOLOR=\#DDDDDD>
		<TD ALIGN=RIGHT><B>Ad Stats Password:</B></TD>
		<TD>
		    <INPUT NAME="ad_campaign_password_$idx" SIZE="8" MAXLENGTH="8" VALUE="$ad_campaign_password"></TD>
		</TR><TR>
		<TD ALIGN=RIGHT><B>Email Stats To:</B></TD>
		<TD>
		    <INPUT NAME="ad_emailto_$idx" SIZE="40" MAXLENGTH="100" VALUE="$ad_emailto"> </TD>
		</TR><TR>
		<TD ALIGN=RIGHT><B>Email Stats On:</B></TD>
		<TD>
		    |;

	%drop_list = %ad_email_frequencies;
	&makeDropList ( "ad_emailat_$idx", $ad_emailat || 'EmailAtNever' );
			
	$page_contents .= qq|</TD>
		</TR><TR BGCOLOR=\#AAAAFF>
		<TD>
		    <INPUT TYPE="CHECKBOX" NAME="ad_kill_$idx" VALUE="T"> Delete this campaign </TD>
		<TD>
		$confirm_replace &nbsp; <INPUT TYPE="SUBMIT" NAME=GoButton VALUE=" Save Changes "></TD>
		</TR>
		</TABLE>
	    </FORM>
		|;


# View the given ad_campaign_in
	
    } elsif ( $op eq 'view'  ) {

	&getCampaignInfo( $ad_campaign_in );
	$tmp_campaign      = $ad_campaign      || $ad_campaign_in;
	$tmp_campaign_name = $ad_campaign_name || $ad_campaign_in;	
	&setCampaignNavlinks;
	$page_title = "Viewing Campaign $tmp_campaign_name: ";

	$tmp_company     = $ad_companies{ $ad_company }         || $ad_company;
	$tmp_agency      = $ad_agencies{ $ad_agency }           || $ad_agency;
	$tmp_category    = $ad_categories{ $ad_product_category }       || $ad_product_category;
	$tmp_salesperson = $ad_sales_people{ $ad_salespersons } || $ad_salespersons;
	$tmp_territory   = $ad_territories{ $ad_territory }     || $ad_territory;
	$tmp_emailat     = $ad_email_frequencies{ $ad_emailat } || $ad_emailat;

	$page_contents .= qq|

		<TABLE BGCOLOR=WHITE CELLPADDING=5>
		<TR>
		<TD> </TD>
		<TD ALIGN=MIDDLE BGCOLOR=DDDDDD> $edit_link </TD>
		</TR><TR>
		<TD ALIGN=RIGHT><B> Campaign ID:</B></TD>
		<TD> $ad_campaign </TD>
		</TR><TR>
		<TD ALIGN=RIGHT><B>Campaign Name:</B></TD>
		<TD> $ad_campaign_name </TD>
		</TR><TR>
		<TD ALIGN=RIGHT><B>Company:</B></TD>
		<TD> $tmp_company </TD>
		</TR><TR>
		<TD ALIGN=RIGHT><B>Product Category:</B></TD>
		<TD> $tmp_category </TD>
		</TR><TR>
		<TD ALIGN=RIGHT><B>Ad Agency:</B></TD>
		<TD> $tmp_agency </TD>
		</TR><TR>
		<TD ALIGN=RIGHT><B>Ad Stats Password:</B></TD>
		<TD> $ad_campaign_password </TD>
		</TR><TR>
		<TD ALIGN=RIGHT><B>Email Stats To:</B></TD>
		<TD> $ad_emailto </TD>
		</TR><TR>
		<TD ALIGN=RIGHT><B>Email Stats On:</B></TD>
		<TD> $tmp_emailat  </TD>
		    |;


	$page_contents .= qq|
		</TR>
		</TABLE>

		    |;




# List all ad_campaigns:

    } else {

	delete $nav_links->{ $sales_dept }->{'Ad Campaigns'};

	if ( $user_access_level >= $admin_access ) {
	    $list_passwords_title = qq|<A HREF="$thisscript?sortby=password">Password</A>|;
	}


	$letterbar = qq| <B>List Index:</B> <A HREF="$thisscript">All</A> |;
	for $i (A..Z) {	    
	    $letterbar .= $alphaindex eq $i ? " [$i] "  : qq| <A HREF="$thisscript?alphaindex=$i">$i</A> |;
	}
	$page_contents .= $letterbar;

	$table_title = "OSDN Advertiser Campaigns: $idx listed as $alphaindex ";

	@table_headers = split /\n\s+/, qq|<A HREF="$thisscript">ID</A>
			   <A HREF="$thisscript?sortby=name">Campaign Name</A>
			   <A HREF="$thisscript?sortby=company">Company</A>
			   <A HREF="$thisscript?sortby=agency">Agency</A>
			   <A HREF="$thisscript?sortby=invoice">Invoice</A>
			   $list_passwords_title|;

	$idx = 0;

	foreach $tmp_key ( sort keys %campaign_list ) {

	    $idx++;

	    &getCampaignInfo( $tmp_key );

	    $tmp_campaign_name  = ( $ad_campaign_name ? 
				    qq|<A HREF="$thisscript?adcamp=$ad_campaign">$ad_campaign_name</A>| 
				    : ' ' );

	    $tmp_company        = ( $ad_company ? 
				    $ad_companies{ $ad_company } || $ad_company 
				    : ' ' );

	    $tmp_agency         = ( $ad_agency 
				    ? $ad_agencies{ $ad_agency } || $ad_agency 
				    : ' ' );

	    $tmp_password       = ( $list_passwords_title ? 
				    $ad_campaign_password 
				    : ' ' );

	    $table_data .= join "\t", ( qq|<A HREF="$thisscript?adcamp=$ad_campaign">$ad_campaign</A>|,
					$tmp_campaign_name,
					$tmp_company,
					$tmp_agency,
					$ad_invoice_link,
					$tmp_password . "\n");

	} # foreach ad_campaign


	&makeTable('PRE');
	$page_contents .= $table_contents;


    } # list all ad_campaigns




# Print output web page

    &printWebpage; 





# Given a ad_campaign (four letters) set the (globla) ad_ settings for that campaign

sub getCampaignInfo {

    my $tmp_campaign      = shift;

    ( $ad_campaign,
      $ad_campaign_name,
      $ad_campaign_password,
      $ad_company,
      $ad_agency,
      $ad_product_category,
      $ad_salespersons,
      $ad_territory,
      $ad_emailto,
      $ad_emailat,			       
      $ad_StartDate,
      $ad_contactsidc,
      $ad_contactsida ) = split /\t/, $campaign_list{ $tmp_campaign };

    $ad_invoice      = $ad_invoices_info{ $ad_campaign }->{ 'invoice_id' };
    $ad_invoice_link = $ad_invoice ? qq|<A HREF="$ad_invoices_url?adcamp=$ad_campaign">$ad_invoice</A>| : ' ';

    # $messages .= "DEBUG: Campaign =  $campaign_list{ $tmp_campaign } <BR>\n";

    return 1;

} # sub getCampaignInfo






__END__


