#!/usr/bin/perl

#
# CGI interface for listing and editing ad campaign invoices
#
# Modified:  Kurt Gray -- Sun Jan 14 13:38:40 EST 2001     Code that was chopped out of ad_campaigns.pl


push @INC, '/usr/local/adsystem/lib/';
require 'ad_config_lib.pl';	        # Site-specific settings for adsystem control scripts
require 'ad_util_lib.pl';	        # Common functions for adsystem scripts

{

# Script settings and base inputs:

    $thisscript           ||= $ENV{'SCRIPT_NAME'};

    $nav_links->{ $admin_access }->{'Create New Invoice'} = qq|$thisscript?op=edit|;


# Print top of output web page and check user login

    &loginUser( $accountants_access );

    if (( $user_access_level < $admin_access ) && ( $user_access_level != $accountants_access )) {
	$user_cant_edit++;
    } else {
	$user_can_edit++;
    }

    &getCampaignsInfo;

    &getActiveSitenames;    # sets %sitenames hash to render sites checkboxes for each run    
    $sitenames{ 'Andover: Run Of Network' } ||= 'atronen';
    $sitenames{ 'OSDN: Run Of Network' }    ||= 'osronen';
    %siteids = reverse %sitenames;
    $numofsites = (keys %siteids);


    %ad_media_types =       (
			     'BANNERS',      'Ad Banners',
			     'BUTTON',       'Sponsorship Button/Link',
			     'NEWSLETTER',   'Newsletter Sponsorship',
			     'MAGAZINE',     'Print Magazine Ads',
			     'SPONSORSHIP',  'Site Sponsorship',
			     'TECH_MONTH',   'Tech Month Sponsor',
			     );


    %ad_status_types =     ( 
			     'BARTERED',      'Barter Deal',
			     'BONUSED',       'Bonused',
			     'PARTNERSHIP',   'Partnership',
			     'PAID_IN_FULL',  'Paid in full',
			     'PAID_IN_PART',  'Paid in part',
			     'COMMUNITY',     'Community Freebie',
			     'NOT_PAID',      'Not paid yet',
			     );


# Make a list of ad sales people based on ad_system_users who considered to be part of the sales dept:

    &getSalesPeople;



# Read query string inputs:

    $ad_campaign_in = $1 if $form->{'in'}->{'adcamp'}      =~ /(\w{4})/;
    $ad_invoice_in  = $1 if $form->{'in'}->{'adinvoice'}   =~ /([\w\-]+)/;
    $op             = $1 if $form->{'in'}->{'op'}          =~ /(\w{4,8})/;
    $op             ||= $1 if $ARGV[0]                     =~ /(\w{4,8})/;      # Added this command line input for debugging
    $function            = 'save' if $form->{'in'}->{'GoButton'} =~ /Save/;
    $sortby         = $1 if $form->{'in'}->{'sortby'}      =~ /(\w{2,14})/;
    $alphaindex     = $1 if $form->{'in'}->{'alphaindex'}  =~ /(\w{1})/;


# Set default modes based on user access level

   if ( $user_cant_edit ) {
       $default_op     = 'view';
       $op             = 'view' if ( $op eq 'edit');
   } else {
       $default_op     = 'edit';
       $edit_link      = qq|<A HREF="$thisscript?adinvoice=$ad_invoice_in&op=edit">Edit Record</A>|;
   } # if access_level

   $op = $default_op if ( $ad_invoice_in && ( ! $op ));

   # $alphaindex ||= 'A' if ! ( $op || $sortby );


# Read edit form inputs:

    $new_file_data       =  '';
    @replaced_records    =  ();

# Read form inputs:

    if ( $function eq 'save' ) {

	if ( $user_cant_edit ) {
	    print "ERROR: Only admins are allowed to edit invoice records. <BR>\n";
	    exit;
	}

	$ad_kill           = $form->{'in'}->{ 'ad_kill'        };
	$ad_campaign       = $form->{'in'}->{ 'ad_campaign'    };
	$ad_invoice        = $form->{'in'}->{ 'ad_invoice'     };
	$ad_cpmrate        = $form->{'in'}->{ 'ad_cpmrate'     };
	$ad_impressions    = $form->{'in'}->{ 'ad_impressions' };
	$ad_media          = $form->{'in'}->{ 'ad_media'       };
	$ad_adids          = $form->{'in'}->{ 'ad_adids'       };
	$ad_comment        = $form->{'in'}->{ 'ad_comment'     };
	$ad_status         = $form->{'in'}->{ 'ad_status'      };
	$ad_PaidDate       = &NiceDate2ISODate( $form->{'in'}->{ 'ad_PaidDate' } );
	$ad_recdby         = $form->{'in'}->{ 'ad_recdby'      };
	$ad_salespersons   = &readCheckboxes( 'ad_salespersons', $numsalespeople );
	$ad_siteids        = &readCheckboxes( 'ad_siteids', $numofsites );
	$ad_StartDate      = &NiceDate2ISODate( $form->{'in'}->{ 'ad_StartDate' } );
	$ad_StopDate       = &NiceDate2ISODate( $form->{'in'}->{ 'ad_StopDate' } );

	# Skip this campaign if it was deleted:

	if ( $ad_kill eq 'T' ) {
	    $messages .= "NOTICE: Entry removed :  $ad_campaign, $ad_invoice, $ad_StartDate, $ad_StopDate <BR>\n";
	    $ignoring_input++;
	    $op = 'listall';
	} 

	if ( $ad_invoice !~ /\w+/ ) {
	    $error_messages .= "INPUT ERROR: Not saving invoice: Invoice ID input is blank. <BR>";
	    $dontsaveinput++;
	    $op = 'edit';
	}

	# Remove any non-numeric chars from CPM rate:

	$ad_cpmrate     =~ s/(\d+)\.\d+/$1/;
	$ad_impressions =~ s/(\d+)\.\d+/$1/;

	if ( $ad_cpmrate =~ s/\D//g ) {
	    $messages .= "WARNING: CPM Rate input should only be a number: non-numeric characters removed <BR>";
	}

	if ( $ad_impressions =~ s/\D//g ) {
	    $messages .= "WARNING: Impressions input should only be a number: non-numeric characters removed <BR>";
	}


	push @replaced_records, "$ad_campaign\t$ad_invoice";

	# Save the note:

	if ( ! $ignoring_input ) {

	    $new_file_data .= join "\t", ( $ad_campaign,
					   $ad_invoice,
					   $ad_StartDate,
					   $ad_StopDate,
					   $ad_impressions,
					   $ad_cpmrate,
					   $ad_siteids,
					   $ad_media,
					   $ad_salespersons,
					   $ad_comment,
					   $ad_adids,
					   $ad_status,
					   $ad_PaidDate,
					   $ad_recdby,
					   "\n" );

	    $op = 'view';
	    $ad_invoice_in = $ad_invoice;
	    
	} # if input save

    } # if Save


# Read all notes data on file, replace overwritten data, append new input data to it, 

    $file_data = $new_file_data;

    if ( open( INFILE, "< $adrunsfile" ) ) { 
	while ( $line = <INFILE>) {
	    next if $line !~ /^(\w{4}\t[\w\-]+)\t/;
	    next if grep /$1/, @replaced_records;     # Skip over deleted or edited invoice records
	    $file_data .= $line;
	}
	close INFILE;
    } else {
	$error_messages .= "ERROR: Could not read invoices file $adrunsfile : $! <BR>\n";
    }


# Save new idx file:

    if (( $function eq 'save' ) && ( ! $dontsaveinput )) {

	if ( $user_cant_edit ) {
	    $error_messages .= "ERROR: Can not save file: User $username does not have write privileges to this interface. <BR>\n"; 
	} else {
	    &BackupAndSaveFile( "$adrunsfile" );
	}

    } # if save changes



# Read all data into a sortable hash:

    foreach $line ( ( split /\n/, $file_data ) ) {
	
	( $tmp_campaign,
	  $tmp_invoice,
	  $tmp_StartDate,
	  $tmp_StopDate,
	  $tmp_impressions,
	  $tmp_cpmrate,
	  $tmp_siteids,
	  $tmp_media,
	  $tmp_salespersons,
	  $tmp_comment,
	  $tmp_adids,
	  $tmp_status,
	  $tmp_PaidDate,
	  $tmp_recdby ) = split /\t/, $line;

	next if ( $ad_campaign_in ) && ( $tmp_campaign ne $ad_campaign_in );
	next if ( $alphaindex ) && ( $tmp_campaign !~ /^$alphaindex/i );
	$numrecords++;

	if ( $sortby eq 'campaign' ) {
	    $tmp_key = $tmp_campaign . "z$tmp_invoice";
	} elsif ( $sortby eq 'invoice' ) {
	    $tmp_key = $tmp_invoice;
	} elsif ( $sortby eq 'StartDate' ) {
	    $tmp_key = &ISODate2SysDate( $tmp_StartDate ) . "z$tmp_invoice";
	} elsif ( $sortby eq 'StopDate' ) {
	    $tmp_key = &ISODate2SysDate( $tmp_StopDate ) . "z$tmp_invoice";
	} elsif ( $sortby eq 'status' ) {
	    $tmp_key = $tmp_status . "z$tmp_invoice";
	} elsif ( $sortby eq 'campname' ) {
	    $tmp_key = $ad_campaign_info{ $tmp_campaign }->{'name'} . "z$tmp_invoice";
	} elsif ( $sortby eq 'salespersons' ) {
	    $tmp_key = $tmp_salespersons . "z$tmp_invoice";
	} elsif ( $sortby eq 'siteids' ) {
	    $tmp_key = $tmp_siteids . "z$tmp_invoice";
	} elsif ( $sortby eq 'impressions' ) {
	    $tmp_key = $tmp_impressions . "z$tmp_invoice";
	} else {
	    $tmp_key = $tmp_invoice;
	} #if sortby

	$sort_list{ $tmp_key } = $line;

    } # foreach data_file line



# Make an HTML edit form for the given note:

    if ( $op eq 'edit' ) {

	&getInfo( $ad_invoice_in );

	$page_contents .= qq|<FORM METHOD=POST ACTION="$thisscript?op=view&adinvoice=$ad_invoice">
	    <TABLE BGCOLOR=WHITE>
	    <TR BGCOLOR=DDFFCC>   
	    <TD ALIGN=RIGHT><B>Campaign ID:</B></TD><TD>
		<INPUT NAME="ad_campaign" SIZE="5" MAXLENGTH="5" VALUE="$ad_campaign">  $ad_campaign_name &nbsp; 
	        <B>Invoice ID:</B> <INPUT NAME="ad_invoice" SIZE="16" MAXLENGTH="16" VALUE="$ad_invoice"></TD>
	    </TR><TR>
	    <TD ALIGN=RIGHT><B>Start Date:</B></TD><TD>
		<INPUT NAME="ad_StartDate" SIZE="12" MAXLENGTH="11" VALUE="$ad_StartDate"> &nbsp;
	        <B>Stop Date:</B> <INPUT NAME="ad_StopDate" SIZE="12" MAXLENGTH="11" VALUE="$ad_StopDate"> </TD>
	    </TR><TR BGCOLOR=FFFFCC>
	    <TD ALIGN=RIGHT><B>Ad Media:</B></TD><TD> |;

	%drop_list = %ad_media_types;
	&makeDropList ( 'ad_media', $ad_media || 'BANNERS' );

	$page_contents .= qq|
                <B>Impressions:</B> <INPUT NAME="ad_impressions" SIZE="10" MAXLENGTH="16" VALUE="$ad_impressions"> &nbsp;
	        <B>CPM Rate:</B> <INPUT NAME="ad_cpmrate" SIZE="8" MAXLENGTH="12" VALUE="$ad_cpmrate"> </TD>
	    </TR><TR>
	    <TD ALIGN=RIGHT><B>Banner IDs:</B></TD><TD>
		<INPUT NAME="ad_adids" SIZE="50" MAXLENGTH="200" VALUE="$ad_adids"></TD>
	    </TR><TR BGCOLOR=FFFFCC>
		<TD ALIGN=RIGHT><B>Status:</B></TD><TD> |;

	%drop_list = %ad_status_types;
	&makeDropList ( 'ad_status', $ad_status || 'NEW' );

	$page_contents .= qq|
	       <B>Payment Date:</B> <INPUT NAME="ad_PaidDate" SIZE="12" MAXLENGTH="11" VALUE="$ad_PaidDate"> &nbsp;
	       <B>Received By:</B>  <INPUT NAME="ad_recdby" SIZE="12" MAXLENGTH="30" VALUE="$ad_recdby"> </TD>
	    </TR><TR>
	    <TD ALIGN=RIGHT><B>Comment:</B></TD><TD>
		<INPUT NAME="ad_comment" SIZE="50" MAXLENGTH="100" VALUE="$ad_comment"> </TD>
	    </TR><TR>
	    <TD ALIGN=RIGHT VALIGN=TOP><B>Web Sites:</B></TD><TD> |;

	%drop_list = %siteids;
	&makeCheckboxes ( 'ad_siteids', $ad_siteids );

	$page_contents .= qq|</TD>
	    </TR><TR BGCOLOR=FFFFCC>
	    <TD ALIGN=RIGHT VALIGN=TOP><B>Sales Reps:</B></TD><TD> |;

	%drop_list = %ad_sales_people;                           
	&makeCheckboxes ( 'ad_salespersons', $ad_salespersons ); 

	$page_contents .= qq|</TD>
	    </TR><TR BGCOLOR=\#AAAAFF>
               <TD> <INPUT TYPE="CHECKBOX" NAME="ad_kill" VALUE="T"> Delete this invoice </TD>
		   <TD>
		       <INPUT TYPE="SUBMIT" NAME=GoButton VALUE="Save Changes"></TD>
		</TR>
		</TABLE>
	    </FORM>|;
	    

# View record:

    } elsif ( $op eq 'view' ) {

	&getInfo( $ad_invoice_in );

	$edit_link = qq|<A HREF="$thisscript?op=edit&adinvoice=$ad_invoice_in">Edit Invoice</A>| if $user_can_edit;

	foreach $tmp ( sort (split ',', $ad_siteids) ) {
	    $tmp_sitenames .= ( $siteids{ $tmp } || $tmp ) . ' &nbsp; ';
	}

	foreach $tmp ( sort (split ',', $ad_salespersons) ) {
	    $tmp_salespersons .= ( $ad_sales_people{ $tmp } || $tmp ) . ' &nbsp; ';
	}

	$page_contents .= qq|

		<TABLE BGCOLOR=WHITE CELLPADDING=5>
		<TR>
		<TD> </TD>
		<TD ALIGN=MIDDLE BGCOLOR=DDDDDD> $edit_link </TD>
		</TR><TR>
		<TD ALIGN=RIGHT><B> Campaign ID: </B></TD>
		<TD> [$ad_campaign] $ad_campaign_name </TD>
		</TR><TR>
		<TD ALIGN=RIGHT><B> Invoice ID: </B></TD>
		<TD> $ad_invoice </TD>
		</TR><TR>
		<TD ALIGN=RIGHT><B> Sales Reps: </B></TD>
		<TD> $tmp_salespersons </TD>
		</TR><TR>
		<TD ALIGN=RIGHT><B> Media: </B></TD>
		<TD> $ad_media &nbsp; <B>Impressions:</B> $ad_impressions  &nbsp;
	                              <B>CPM Rate:</B> $ad_cpmrate </TD>
		</TR><TR>
		<TD ALIGN=RIGHT><B> Web Sites: </B></TD>
		<TD> $tmp_sitenames </TD>
		</TR><TR>
		<TD ALIGN=RIGHT><B> Status: </B></TD>
		<TD> $ad_status &nbsp; <B>Paid Date:</B> $ad_PaidDate  &nbsp;
	                               <B>Received By:</B> $ad_recdby </TD>
		</TR>
		</TABLE> |;


# List all records:

    } else {

	$letterbar = qq| <B>List Index:</B> <A HREF="$thisscript">All</A> |;
	for $i (A..Z) {	    
	    $letterbar .= $alphaindex eq $i ? " [$i] "  : qq| <A HREF="$thisscript?alphaindex=$i">$i</A> |;
	}
	$page_contents = $letterbar;

	$table_title = "$numrecords Invoice Records";

	@table_headers = ( qq|<A HREF="$thisscript?sortby=campaign">Campaign ID</A>|,
			   qq|<A HREF="$thisscript?sortby=campname">Campaign Name</A>|,
			   qq|<A HREF="$thisscript?sortby=invoice">Invoice</A>|,
			   qq|<A HREF="$thisscript?sortby=StartDate">Start Date</A>|,
			   qq|<A HREF="$thisscript?sortby=StopDate">Stop Date</A>|,
			   qq|<A HREF="$thisscript?sortby=impressions">Impressions</A>|, 
			   qq|<A HREF="$thisscript?sortby=status">Status</A>| );

	foreach $tmp_key ( sort { &sortAlphaNumeric($a, $b) } keys %sort_list ) {

	    &getInfo( $tmp_key );

	    $table_data .= join "\t", ( qq|<A HREF="$thisscript?adcamp=$ad_campaign">$ad_campaign</A>|,
					qq|<A HREF="$thisscript?adcamp=$ad_campaign">$ad_campaign_name</A>|,
					qq|<A HREF="$thisscript?adinvoice=$ad_invoice">$ad_invoice</A>|,
					$ad_StartDate,
					$ad_StopDate,
					$ad_impressions,
					$ad_status . "\n" );

	} # foreach record

	&makeTable('HTML');
	$page_contents .= $table_contents;


    } # if op



# Print output web page

    &printWebpage; 



} # end of main



sub getInfo {

    my $tmp_key = shift;

    ( $ad_campaign,
      $ad_invoice,
      $ad_StartDate,
      $ad_StopDate,
      $ad_impressions,
      $ad_cpmrate,
      $ad_siteids,
      $ad_media,
      $ad_salespersons,
      $ad_comment,
      $ad_adids,
      $ad_status,
      $ad_PaidDate,
      $ad_recdby ) = split /\t/, $sort_list{ $tmp_key };

    $ad_campaign_name = $ad_campaign_info{ $ad_campaign }->{'name'};

    return;

} # sub getInfo


__END__
