#!/usr/bin/perl

#
# Ad system interface to allocate ads for email newsletters
#
# Notes:
#        - Not all web sites have an email newsletter configured in ad_config_lib.pl
#
#        - Each newsletter has two versions: HTML and plain text. The HTML newsletters
#          have a pair HREF/IMG tags linking to admail_rotate.pl which delivers a GIF
#          ad and redirects a click when an ad is clicked on. The plain text newsletters
#          have a hardcoded plain text sponsorship text block at the top of the newsletter.
#
#        - Because most HTML-supported email clients can not do IFRAME/ILAYER tags 
#          HTML newsletters can only include plain GIF images
#
#        - The /usr/local/adsystem/dat/ad_mail_alloc.dat file needs to readable by the
#          admail_rotate.pl script in order to deliver the ads and redirect clicks.
#
#        - I'm using the 'require .dat' trick to read/write each newsletter settings
#          as perl code because, I don't know, it seemed to make the admail_rotate.pl
#          script a lot more elegant. Doesn't seem to have any performance impact either.
#
#        - Right now the admail_rotate.pl script is in the /usr/local/adsystem/bin/
#          directory.
#
#
# Modified:  Kurt Gray -- Tue Aug  8 11:42:23 EDT 2000     First rev.
#            Kurt Gray -- Thu Aug 10 14:35:53 EDT 2000     Saves adtext files for each newsletter
#            Kurt Gray -- Tue Nov 28 11:04:20 EST 2000     Rewritten to work as part of /usr/local/adsystem on report server


use Date::Manip;
use Time::Local;          # converts ISO dates to eopch times for newsletter start and stop dates
use POSIX qw(strftime);

push @INC, '/usr/local/adsystem/lib/';
require 'ad_config_lib.pl';	        # Site-specific settings for adsystem control scripts
require 'ad_util_lib.pl';	        # Common functions for adsystem scripts

require '/usr/local/adsystem/dat/ad_mail_alloc.dat';    # Read all current newsletter settings


# Script config settings

    $largead_position          = 1;
    $siteid                    = $1 if $ENV{'QUERY_STRING'} =~ /siteid=(\w{7})/;
    $sitename                  = $sitesconfig{ $siteid . 'sitename' };
    $thisscript                = "$ad_mailedit_url\?siteid=$siteid";
    $allocfile                 = 'ad_mail_alloc.dat';
    $todayfullsysdate          = &Date::Manip::UnixDate("today", '%Y%m%d');


    &loginUser( $admin_access );


# Stop here if no siteid was specified 

    if ( $siteid !~ /^\w{7}$/ ) {
	print "<B>ERROR: Bad input.</B> siteid blank or invalid.</BODY></HTML>";
	exit;    
    }
    

# TODO: Move this bit of siteid hardcoding to ad_config_lib.pl

    my %newsletters = ( 'newsletter', 'Andover Update',
			'anim', 'Animation Factory Newsletter',
			'osdn', 'OSDN Newsletter');

# Read all CGI inputs

    $newsletter     = $1 if $sitesconfig{ $siteid . 'ad_newsletter' }              =~ /(\w+)/;
    $adid           = $1 if $form->{'in'}->{'adid'}                                =~ /(\w+)/;
    $adText         = $form->{'in'}->{'adtext'} if $form->{'in'}->{'adtext'}       =~ /\w+/;
    $default_adid   = $1 if $form->{'in'}->{'default_adid'}                        =~ /(\w+)/;
    $StartDate      = $form->{'in'}->{'StartDate'} if $form->{'in'}->{'StartDate'} =~ /\w+/;
    $StopDate       = $form->{'in'}->{'StopDate'} if $form->{'in'}->{'StopDate'}   =~ /\w+/;


# Fix American date input to ISO
			
    $StartDate      = &NiceDate2ISODate($StartDate) if $StartDate =~ /\d\/\d/;
    $StopDate       = &NiceDate2ISODate($StopDate) if $StopDate =~ /\d\/\d/;


# Convert StartDate and StopDate to Epcoh times before saving to admail_rotate.dat

    $StartDate      = &ISODate2Epochtime($StartDate) if $StartDate =~ /\d\/\w/;
    $StopDate       = &ISODate2Epochtime($StopDate) if $StopDate =~ /\d\/\w/;


# Already read current settings from "require ad_mail_rotate.dat" file

    if ( ! $newsletter ) {
	print "WARNING: The site $sitename does not have a newsletter specified. Defaulting to Andover Update allocation. \n";
    }

    $newsletter      ||= 'newsletter';                 # Default to Andover Update if no input
    $adid            ||= $adID{         $newsletter };
    $adText          ||= $adtext{       $newsletter };
    $StartDate       ||= $Startdate{    $newsletter };
    $StopDate        ||= $Stopdate{     $newsletter };
    $default_adid    ||= $default_adID{ $newsletter };
    $newsletter_name   = $newsletters{  $newsletter };



# Warn user if dates are mixed up

    if ( $StartDate > $StopDate ) {
	$error_messages .= "<B><FONT COLOR=RED>WARNING:</FONT></B> Start Date and Stop Date are backwards.<BR><BR>";
    }

    $currenttime = time;
    if ( $StartDate > $currenttime ) {
	$error_messages .= "<B>NOTICE:</B> Start Date hasn't occured yet. Default ad is being served now.<BR><BR>";
    }
    if ( $StopDate < $currenttime ) {
	$error_messages .= "<B>NOTICE:</B> Stop Date has past. Default ad is being served now.<BR><BR>";
    }

# Warn if adtext contains HTML tags (should only be plain text)

   if ( $adText =~ /<.+>/ ) {
       $error_messages .= "<B><FONT COLOR=RED>WARNING:</FONT></B> Sponsorship text should not contain HTML tags! Text only.<BR><BR>";
   }


# If Save button clicked then save new ad_mail_rotate.dat file

    if ( $form->{'in'}->{'GoButton'} =~ 'Save' ) {

	# Read current contents of datfile

	$file_data = '';
	if ( open INFILE, "< $datfilepath/$allocfile" ) {
	    while (<INFILE>) {
		$file_data .= $_;
	    }
	    close INFILE;
	} else {
	    $error_messages .= "ERROR: Could not read contents of $datfilepath/$allocfile : $! <BR>\n";
	}


	# Save the new email ad text file -- this file gets mirrored to all sites to OSDN.com can read its newsletter text

	if ( $adText =~ /\w+/ ) {
	    $adtextfilename = "$ad_htmlad_path/email_adtext\_$newsletter.txt";
	    if ( open(OUTFILE, "> $adtextfilename")) {
		print OUTFILE $adText;
		close OUTFILE;
	    } else {
		$error_messages .= "ERROR: Could not create $adtextfilename: $! <BR>\n";
	    }
	}


	# Update current settings for this newsletter

	$file_data =~ s/\$adID\{ '$newsletter' \} \= '\w{10}'\;/\$adID\{ '$newsletter' \} \= '$adid'\;/;
	$file_data =~ s/\$default_adID\{ '$newsletter' \} \= '\w{10}'\;/\$default_adID\{ '$newsletter' \} \= '$default_adid'\;/;
	$file_data =~ s/\$Startdate\{ '$newsletter' \} \= \d*\;/\$Startdate\{ '$newsletter' \} \= $StartDate\;/;
	$file_data =~ s/\$Stopdate\{ '$newsletter' \} \= \d*\;/\$Stopdate\{ '$newsletter' \} \= $StopDate\;/;
	$file_data =~ s/\$adtext\{ '$newsletter' \} \= qq\|[\r\n][^|]*[\n\r]+\|\;[\r\n]/\$adtext\{ '$newsletter' \} \= qq\|\n$adText\n\|\;\n/s;

        # Create a new allocation file
      
	if ( $user_access_level < $admin_access ) {
	    $error_messages .= "ERROR: Can not save file: User $username does not have ad system control privileges. <BR>\n"; 
	} else {
	    &BackupAndSaveFile( "$datfilepath/$allocfile" );
	}

    } # if Save



# Convert StartDate and StopDate from Epoch time to ISO date to show on web page

    $StartDate  = &Epochtime2ISODate($1) if $StartDate =~ /^(\d+)$/;
    $StopDate   = &Epochtime2ISODate($1) if $StopDate  =~ /^(\d+)$/;


# Make a list of adid's we can allocate to newsletters (no HTML ads allowed)

    if ( open( ADINFILE, "< $datfilepath/ad_info.dat" ) ) {	
	while (<ADINFILE>) {
	    chomp;
	
	    ($AdID,$ht,$wh,$tt,$at,$re) = split(/\t/, $_);
			
	    if ( $largead_position ) {
		next if $wh < 380;
	    } else {
		next if $wh > 140;
	    }
	    
	    if ( (substr($AdID, 4, 1)) > 4 ) {
		$error_messages .= "DEBUG: Skipping $AdID since newsletters can not run HTML ads.\n" if $debugging == 1;
		next;
	    }

	    if ( $verify_gifs_exists == 1 ) {
		$gif_filename = "$gif_filepath/$AdID.gif";
		if ( !( -e $gif_filename )) {
		    $gif_errors .= qq|WARNING: Can not allocate <A HREF="$ad_infoedit_url?adid=$AdID">$AdID</A> does not have a GIF image.<BR> \n|;
		    next;
		}
	    }	
		
		
	    $adids{$AdID} = $AdID;
		
	}
	close ADINFILE;


        # Make a droplist to choose adid for this newsletter

	foreach $AdID (sort keys %adids) {
	    if ( $AdID eq $adid ) {
		$adids_droplist .= "<OPTION VALUE=\"$AdID\" SELECTED>$AdID</OPTION>\n";
	    } else {
		$adids_droplist .= "<OPTION VALUE=\"$AdID\">$AdID</OPTION>\n";
	    }

            if ( $AdID eq $default_adid ) {
		$default_adids_droplist .= "<OPTION VALUE=\"$AdID\" SELECTED>$AdID</OPTION>\n";
	    } else {
		$default_adids_droplist .= "<OPTION VALUE=\"$AdID\">$AdID</OPTION>\n";
	    }
	}


    } else {
	 $error_messages .= "ERROR: Couldn't read $datfilepath/ad_info.dat: $! <BR> \n";
    }

	
# Print page top

$tmp_navbar = &userNavbar;

	print <<REST_OF_PAGE;
$tmp_navbar
$messages
$error_messages
$gif_errors

    <FONT SIZE=+1><B>$newsletter_name Sponsorship:</B></FONT> &nbsp; [<A HREF=\"$ad_mainmenu_url\">Return to Main Menu</A>]

    <FORM METHOD="POST" ACTION="$thisscript">
    <B>$newsletter_name Sponsorship GIF Banner:</B> <SELECT NAME="adid">$adids_droplist</SELECT>
    <BR>
    <B>Start Date:</B> <INPUT TYPE=TEXT NAME="StartDate" SIZE=14 VALUE="$StartDate"> &nbsp; 
    <B>Stop Date:</B> <INPUT TYPE=TEXT SIZE=14 NAME="StopDate" VALUE="$StopDate">
    <BR>
    <BR>
    <B>Sponsorship Text:</B><BR>
    <TEXTAREA NAME="adtext" ROWS=4 COLS=60>$adText</TEXTAREA>
    <BR>
    <BR>
    <B>Default ad (run after StopDate of the ad above):</B> <SELECT NAME="default_adid">$default_adids_droplist</SELECT>
    <BR>
    <BR>
    <INPUT TYPE=SUBMIT NAME="GoButton" VALUE="Save Changes to $newsletter_name">
    <INPUT TYPE="HIDDEN" NAME="UserName" VALUE="$username">
    <INPUT TYPE="HIDDEN" NAME="Password" VALUE="$password">
    </FORM>
 &nbsp; [<A HREF=\"$ad_mainmenu_url\">Return to Main Menu</A>]
    </BLOCKQUOTE>
    </BODY>
    </HTML>

REST_OF_PAGE







sub NiceDate2ISODate {

        local ($nicedate) = @_;
        ($mon,$day,$year) = split(/[\/\-]/, $nicedate);
        $day =~ s/^0//;
        $day = '0' . $day if $day < 10;
        $mon =~ s/^0//;
        %monthlist = qw/1 Jan 2 Feb 3 Mar 4 Apr 5 May 6 Jun 7 Jul 8 Aug 9 Sep 10 Oct 11 Nov 12 Dec/;
        $year = `/bin/date +%Y` unless $year =~ /\d\d\d\d/;
        $year =~ s/\W//g;
        
        return $day . '/' . $monthlist{$mon} . '/' . $year;

}  # Converts 2/03/98 to 03/Mar/1998



sub ISODate2SysDate {

        local ($ISODate) = @_;
        if ($ISODate =~ /(\d+)\/(\w+)\/(\d+)/) {
	    %monthlist = qw/Jan 01 Feb 02 Mar 03 Apr 04 May 05 Jun 06 Jul 07 Aug 08 Sep 09 Oct 10 Nov 11 Dec 12/;
	    return $3 . $monthlist{$2} . $1;
        } else {
	    return 0;
        } 

}  # Converts 02/Jan/1998 to 19980102 to do date comparisons



sub ISODate2Epochtime {

        local ($ISODate) = @_;
        if ($ISODate =~ /(\d+)\/(\w+)\/(\d+)/) {
	    %monthlist = qw/Jan 0 Feb 1 Mar 2 Apr 3 May 4 Jun 5 Jul 6 Aug 7 Sep 8 Oct 9 Nov 10 Dec 11/;
	    return timelocal(0, 0, 0, $1, $monthlist{$2}, $3);
        } else {
	    return 0;
        } 


}  # Converts 02/Jan/1998 to 998234231



sub Epochtime2ISODate {

        local ($TIME) = @_;
        if ($TIME =~ /(\d+)/) {
            return strftime("%d/%b/%Y", localtime($1));
        } else {
	    return 0;
        } 


}  # Converts 02/Jan/1998 to 998234231




__END__



















