#!/usr/bin/perl

# CGI and command line interface to get ad statistics reports
#
# TODO:
#        BUG:    Sort-by column links don't have ad_campaign, banner_id context included
#        ADD:    Client, Invoice, Sales Rep, Report Options form to top of HTML report
#        ADD:    Graphs
#        ADD:    stats by SysDate to breakdown and sort stats by date
#        CHANGE: userLogin redirect URL to "SCRIPT_NAME || ad_mainmenu_url" so any CGI script can do login
#        
#
# Modified:   Kurt Gray -- Fri Dec 22 12:41:09 EST 2000      First rev. intending to replace rp_ad.pl
#


use Date::Manip;

push @INC, '/usr/local/adsystem/lib/';
require 'ad_config_lib.pl';	        # Site-specific settings for adsystem control scripts
require 'ad_util_lib.pl';	        # Common functions for adsystem scripts
require 'ad_email_lib.pl';

&unbufferSTDOUT;


{

# Read command line options

    foreach $arg (@ARGV) {

	# Option to specify a site

	if ( $arg =~ /site=(\w+)/ ) {
	    $siteid = $1;

	# Debug option shows what's happening without writing new logs
	# and a Ctrl+C keystroke will stop script and dumps stats gathered so far

	} elsif ( $arg =~ /debug/ ) {
	    $debugging = 1;
	    # $dontsavelogs = 1;
	    # $SIG{INT} = dumpStats;

	# Option to get stats for a specific date (date=yesterday is useful for regenerating yesterday's stats)

	} elsif ( $arg =~ /date=(\d{6,8})\-?(\d{0,8})/ ) {
	    $StartDate_in = $1;
	    $StopDate_in  = $2;

	# Print summary stats only

	} elsif ( $arg =~ /showsummary/ ) {
	    $showsummary = 1;


        # Ad campaign to show stats for:

	} elsif ( $arg =~ /adcamp=(\w+)/ ) {
	    $ad_campaign = $1;

        # Ad banner to show stats for:

	} elsif ( $arg =~ /adid=(\w+)/ ) {
	    $ad_banner_id = $1;


        # Table data output format:

	} elsif ( $arg =~ /format=(\w+)/ ) {
	    $output_format = $1;

        # Option to breakout campaign stats by siteid or logdate:

	} elsif ( $arg =~ /breakdownby=(\w+)/ ) {
	    $breakdownby = $1;

	# Warn about unparsed inputs
	} else {
	    print "ERROR: $0 does not understand argument: $arg <BR>\n";
	    exit;
	} # if arg

    } # foreach ARGV



# If we didn't get any command line inputs then read CGI web inputs:

    if ( ( $ad_campaign || $StartDate_in || $siteid || $debugging ) ) {

	$offline_mode++;
	$output_format ||= 'CSV';   # Print ad report data to STDOUT, comma separated values

    } else {

	$online_mode++;

	# Login user and start printing output web page:

	&loginUser( $advertiser_access );

	# Adjust navbar links:

	# delete $nav_links->{ $sales_dept }->{'Ad Reports'};

	# Read CGI inputs:

	$StartDate_in   = $1 if $form->{'in'}->{ 'StartDate'   } =~ /([\d\/\-]+)/;
	$StopDate_in    = $1 if $form->{'in'}->{ 'StopDate'    } =~ /([\d\/\-]+)/;
	$ad_campaign_in = $1 if $form->{'in'}->{ 'adcamp'      } =~ /(\w{4})/;
	$ad_banner_in   = $1 if $form->{'in'}->{ 'adbanner'    } =~ /(\w{8,12})/;
	$siteid         = $1 if $form->{'in'}->{ 'siteid'      } =~ /(\w{7})/;
	$output_format  = $1 if $form->{'in'}->{ 'format'      } =~ /(\w{3,6})/;
	$sortby         = $1 if $form->{'in'}->{ 'sortby'      } =~ /(\w+)/;
	$breakdownby    = $1 if $form->{'in'}->{ 'breakdownby' } =~ /(\w+)/;
	$op             = $1 if $form->{'in'}->{ 'op'          } =~ /(\w+)/;
	$emailto        = $1 if $form->{'in'}->{ 'emailto'     } =~ /(\S{5,80})/;
	$daterange_confirmed = $1 if $form->{'in'}->{'dates_confirmed'} =~ /(\w)/;

        # Web site partners only stats for their own web sites:
	if ( $user_access_level == $partner_access ) {
	    $siteid = $1 if $ad_system_users{ $username }->{ 'siteids' } =~ /(\w+)/;
	}

    } # if CGI inputs


# Set script defaults:

    $ad_campaign                = $username if $user_access_level <= $advertiser_access;
    $ad_campaign              ||= $ad_campaign_in || substr( $ad_banner_in, 0, 4);

    $ad_id                      = $ad_banner_in || $ad_campaign;
    $site_initials              = $1 if $siteid =~ /^(\w{2})\w+/;
    $output_format            ||= 'HTML';
    $output_textonly            = 1 if $output_format =~ /SV/ || $offline_mode;
    $sortby                   ||= 'impressions';
    $impressions_cutoff_limit ||= 80;  # Don't show stats for any campaign, site, or banner with less than this many impressions
    $numberofdays_limit       ||= 200;
    $op                         = 'emailto' if $emailto;

    &getCampaignsInfo( $ad_campaign );   # Make global hash of all ad campaign ids => name, numberofads
    $ad_campaign_name         ||= $ad_campaign_info{ $ad_campaign }->{'name'};

    &setCampaignNavlinks;

    # Default date range to past 2 days

    $StopDate_in ||= &Date::Manip::UnixDate("today", '%y%m%d');   # logdate format is yymmdd
    if ( $StartDate_in !~ /\d\d/ ) {
	$tmp_date     = &Date::Manip::ParseDateString( "today" );
	$tmp_date     = &Date::Manip::DateCalc( $tmp_date, "- 1 days" );
	$StartDate_in = &Date::Manip::UnixDate("$tmp_date", '%y%m%d');
    }

    &parseDateInputs;  # Convert StartDate_in and StopDate_in to various formats and setup @logdates list

    if (( $numberofdays > $numberofdays_limit ) && ( $online_mode )) {
	if ( $daterange_confirmed ) {
	    $confirm_daterange = qq|<INPUT TYPE=CHECKBOX NAME="dates_confirmed" VALUE="T" CHECKED> Dates Confirmed|;
	} else {

	    $messages .= qq|WARNING: You're asking for more than $numberofdays_limit days worth of data:
                      $numberofdays days. If you're sure about this then click "Dates Confirmed"
		      then click submit button. You may also want to set Breakdown to "Normal"
		      and Output Format to "Text" for faster results. <BR>\n|;
	    
	    $confirm_daterange = qq|<INPUT TYPE=CHECKBOX NAME="dates_confirmed" VALUE="T"> Dates Confirmed|;

	    @logdates = ();

	}
    } # if too many days


# Setup list of report options to include ad report details links:

    %report_options  = ( 
			 'StartDate', $StartDate_yymmdd,
			 'StopDate', $StopDate_yymmdd,
			 'format', $output_format 
			 );

    $report_options{ 'siteid' }   = $1 if $siteid       =~ /(\w{7})/;
    $report_options{ 'adcamp' }   = $1 if $ad_campaign  =~ /(\w{4})/;
    $report_options{ 'adbanner' } = $1 if $ad_banner_in =~ /(\w{8,12})/;
    $report_options{ 'breakdownby' } = $1 if $breakdownby =~ /(\w+)/;
    $report_options{ 'sortby' } = $1 if $sortby =~ /(\w+)/;

    while (($key,$value) = each %report_options ) {
	push @report_settings, "$key\=$value";
    }

    $tmp_report_url = $ad_report_url . '?';
    $tmp_report_url .= ( join '&', @report_settings );



# Setup options form:

    if (( $online_mode ) && ( $output_format ne 'PRINT' )) {

	$page_contents .= qq|<FORM METHOD=POST ACTION="$ad_report_url">Ad Campaign ID: <INPUT NAME=adcamp SIZE=5 MAXLENGTH=4 VALUE="$ad_campaign"> $ad_campaign_name Banner ID: <INPUT SIZE=11 NAME=adbanner VALUE="$ad_banner_in"> from Start Date: <INPUT NAME=StartDate SIZE=8 MAXLENGTH=12 VALUE="$StartDate_nice"> &nbsp; to Stop Date: <INPUT NAME=StopDate SIZE=8 MAXLENGTH=12 VALUE="$StopDate_nice"> $confirm_daterange <BR>
Output Format: |;

	%drop_list = ( 'HTML',  'Normal',
		       'PRE',   'Text',
		       'CSV',   'CSV',
		       'TSV',   'TSV',
		       'PRINT', 'Printable' );

	&makeDropList( "format", "$output_format" );

	$page_contents .= qq| Sort by: |;

	%drop_list = ( 'adcamp',      'Campaign ID',
		       'campname',    'Campaign Name',
		       'impressions', 'Impressions',
		       'clicks',      'Clicks',
		       'clickthru',   'Click-Thru',
		       'clickable',   'Clickable',
		       'campname',    'Campaign Name',
                       'networkid',   'Network',
		       'siteid',      'Site Name',
		       'adbanner',    'Banner ID',
		       'bannername',  'Banner Name' );

	&makeDropList( "sortby", "$sortby" );

	$page_contents .= qq| Breakdown: |;

	%drop_list = ( ' ',         'Normal',
		       'siteid',    'by Site',
		       'logdate',   'by Date', );

	&makeDropList( "breakdownby", "$breakdownby" );

	if ( $ad_system_users{ $username }->{'email'} ) {
	    $page_contents .= qq| <INPUT TYPE=CHECKBOX NAME="op" VALUE="emailto"> Email this to me |;
	}

	$page_contents .= qq|  &nbsp; <INPUT TYPE=SUBMIT NAME=GoButton VALUE="Show Report"></FORM>\n|;

    } # if web options


# Read all adlog files in given date range:

    $stats_by_siteid           = {};
    $stats_by_campaign         = {};
    $stats_by_bannerid         = {};
    $breakdown_campaign_stats  = {};
    $breakdown_banner_stats    = {};

    $numberofmondays = int( $numberofdays / 7 ) || 1;

    foreach $logdate ( @logdates ) {

	$tmp_logdate  = &logdate2SysDate( $logdate );
	if ( $breakdownby eq 'weekday' ) { 
	    $tmp_logdate2 = &logdate2SysDateDow( $logdate );
	    $tmp_dow     = "$1 (x$numberofmondays)" if $tmp_logdate2 =~ / (\w{3})$/;
	}

	$adlogfile = $adlogpath . '/ads_' . $logdate . '.log';

	if ( open ( INFILE, "< $adlogfile" ) ) {
	    $messages .= "DEBUG: Reading $adlogfile... <BR>\n" if $debugging == 1;
	    while (<INFILE>) {
		chomp;
		($tmp_adid, $tmp_impressions, $tmp_clicks, $tmp_siteid) = split(/\t/, $_);

		# If campaign or banner ID specified then ignore stats from other banners or campaigns:

		next if ( $ad_id ) && ( $tmp_adid !~ /^$ad_id/ );

		# If a siteid was specified then ignore stats from other sites:

		if ( $siteid ) {
		    if ( $tmp_siteid ne $siteid ) {
			# TODO: Use stats_group instead of allocation group???
			if ( $sitesconfig{ $tmp_siteid . 'ad_stats_group' } ne $site_initials ) {
			    next;
			}
		    }

		# Group stats summary by site according to subsite stats group:
 
		} else {

		    if ( $sitesconfig{ $tmp_siteid . 'ad_stats_group' } =~ /([a-z][a-z])/ ) {
			$tmp_siteid = $1 . '000en';
		    }

		} # if siteid

		# Determine the campaign ID and ad type (HTML ad or normal ad)

		$tmp_campaign = substr( $tmp_adid, 0, 4 );
		$tmp_adtype   = substr( $tmp_adid, 4, 1 );

		# $messages .= "DEBUG: Tallying stats for $tmp_adid on $tmp_siteid on $logdate... <BR>\n" if $debugging == 1;

		$stats_by_bannerid{ $tmp_adid }->{ 'impressions' }     += $tmp_impressions;
		$stats_by_campaign{ $tmp_campaign }->{ 'impressions' } += $tmp_impressions;
		$stats_by_siteid{ $tmp_siteid }->{ 'impressions' }     += $tmp_impressions;
		$total_impressions                                     += $tmp_impressions;

		if ( $breakdownby ) {
		    $tmp_key3 = ( $breakdownby eq 'logdate' ) ? $tmp_logdate : $tmp_siteid;
		    $breakdown_campaign_stats{ $tmp_campaign }->{ $tmp_key3 }->{ 'impressions' } += $tmp_impressions;
		    $breakdown_banner_stats{ $tmp_adid }->{ $tmp_key3 }->{ 'impressions' } += $tmp_impressions;
		}

		# Count HTML ad impressions separately and don't count HTML ad clicks:
		# TODO: ...however there are many HTML ads we do track clicks for...

		if (( $tmp_adtype < 5 ) || ( $tmp_clicks > 0 )) {

		    $stats_by_bannerid{ $tmp_adid }->{ 'clicks' }      += $tmp_clicks;		
		    $stats_by_campaign{ $tmp_campaign }->{ 'clicks' }  += $tmp_clicks;
		    $stats_by_siteid{ $tmp_siteid }->{ 'clicks' }      += $tmp_clicks;
		    if ( $breakdownby ) {
			$breakdown_campaign_stats{ $tmp_campaign }->{ $tmp_key3 }->{ 'clicks' } += $tmp_clicks;
			$breakdown_banner_stats{ $tmp_adid }->{ $tmp_key3 }->{ 'clicks' } += $tmp_clicks;
		    }
		    $total_clicks                                      += $tmp_clicks;

		} else {

		    # $stats_by_bannerid{ $tmp_adid }->{ 'clicks' }                 = 'N/A';
		    $stats_by_bannerid{ $tmp_adid }->{ 'htmlad_impressions' }     += $tmp_impressions;
		    $stats_by_campaign{ $tmp_campaign }->{ 'htmlad_impressions' } += $tmp_impressions;
		    $stats_by_siteid{ $tmp_siteid }->{ 'htmlad_impressions' }     += $tmp_impressions;
		    if ( $breakdownby ) {
			$breakdown_campaign_stats{ $tmp_campaign }->{ $tmp_key3 }->{ 'htmlad_impressions' } += $tmp_impressions;
			$breakdown_banner_stats{ $tmp_adid }->{ $tmp_key3 }->{ 'htmlad_impressions' } += $tmp_impressions;
		    }
		    $total_htmlad_impressions                                     += $tmp_impressions;

		} # if adtype

	    } # while INFILE
	    close INFILE;

	} else {
	    $error_messages .= "WARNING: Could not read ad stats for day of $logdate: $adlogfile: $! <BR>\n";
	}


    } # foreach logdate



# Which table are we going to show now?

    
    if ( $ad_banner_in ) {
	if ( ! $output_textonly ) {
	    $page_contents .= qq|<BLOCKQUOTE> $ad_banner_info{ $ad_banner_in }->{'HTML'} </BLOCKQUOTE>
	        $ad_banner_info{ $ad_banner_in }->{'infotable'}|;
	}
    }

    $page_contents .= '<PRE>' if $output_textonly && $online_mode;

    if ( $ad_id ) {
	$tmp_report_url =~ s/\&?adbanner\=$ad_banner_in//;
	$tmp_report_url =~ s/\&?siteid\=$siteid//;
	$nav_links->{ $advertiser_access }->{ "$ad_campaign_name Ad Report" } = $tmp_report_url;
	%statsby_hash  = %stats_by_bannerid;
	&makeAdStatsTable( 'Banner ID', 'adbanner', 'Ad Creative', 'bannername' );
    } else {
	%statsby_hash  = %stats_by_campaign;
	&makeAdStatsTable( 'Campaign ID', 'adcamp', $breakdownby, 'campname' );
    }

    $attachment_data .= $table_contents;

    %statsby_hash  = %stats_by_siteid;
    &makeAdStatsTable( 'Web Site', 'siteid', 'Network', 'networkid' );

    $attachment_data .= $table_contents;

    $page_contents .= '</PRE>' if $output_textonly && $online_mode;


# Print output:

    if ( $offline_mode ) {

	print $page_contents;
	print $messages;
	print $error_messages;


# If we're in CGI mode then print output as a fancy web page:

    } else {

	if ( $op eq 'emailto' ) {
	    $from_addr       = $email_admin;
	    $to_addr         = $emailto || $ad_system_users{$username}->{'email'};
	    $email_subject   = "[OSDN Stats] $table_title";
	    $email_message   = qq|This report was sent to you by $ad_system_users{$username}->{'name'} <$ad_system_users{$username}->{'email'}> via the 
OSDN Traffic Stats System at $system_baseurl
The attached file is formatted as a $output_format table.\n|;
	    $tmp_output_format   = lc( $output_format );
	    $attachment_filename = "osdn_ad_stats.$tmp_output_format";
	    &SendEmail;
	}

	&printWebpage( $output_format );


    } # if offline




}  # end of main script



# makeAdStatsTable: Creates formatted summary stats table of the given stats hash
# 
#       Inputs:    $statsby_label   (local)   Label of left-most table column
#                  $statsby_index   (local)   Key for column sortby link option
#                  %statsby_hash    (global)  Stats hash to be formatted into a table
#                  $output_format   (global)  How to format the table: CSV, TSV, HTML, or PRE
#
#       Outputs:   $page_contents   (global)  Appends table text to page_contents
#

sub makeAdStatsTable {

    my $statsby_labelA = shift;
    my $statsby_indexA = shift;
    my $statsby_labelB = shift;
    my $statsby_indexB = shift;


    $table_data        =  '';
    $table_contents    =  '';
    $tmp_table_name    = '"' . ( $ad_banner_info{ $ad_banner_in }->{'name'} || $ad_banner_in ) . '" Ad Banner' if $ad_banner_in;
    $tmp_table_name  ||= '"' . ( $ad_campaign_name || $ad_campaign ) . '" Campaign';
    $table_title       =  qq|$tmp_table_name Stats by $statsby_labelA from $StartDate_ISO to $StopDate_ISO, $numberofdays days|;

    if ( $output_textonly ) {
	@table_headers   =  ( $statsby_labelA, $statsby_labelB, 'Impressions', 'ClickTrack', 'Clicks', 'Click-Thru %' );
    } else {
	@table_headers   =  (
			     qq|<A HREF="$tmp_report_url&sortby=$statsby_indexA">$statsby_labelA</A>|,
			     qq|<A HREF="$tmp_report_url&sortby=$statsby_indexB">$statsby_labelB</A>|,
			     qq|<A HREF="$tmp_report_url&sortby=impressions">Impressions</A>|,
			     qq|<A HREF="$tmp_report_url&sortby=clickable">ClickTrack</A>|,
			     qq|<A HREF="$tmp_report_url&sortby=clicks">Clicks</A>|,
			     qq|<A HREF="$tmp_report_url&sortby=clickthru">Click-Thru %</A>|,
			     );
    } # if output_format


    # Create a sortable hash of stats-by-site:

    %sorttable_hash = ();

    foreach $tmp_key ( sort keys %statsby_hash ) {

	if ( $statsby_indexA eq 'siteid' ) { 
	    $tmp_nameA             = $sitesconfig{ $tmp_key . 'sitename' } || $tmp_siteid;
	    $tmp_nameB             = $sitesconfig{ $tmp_key . 'site_content_group' };
	} elsif ( $statsby_indexB eq 'campname' ) {
	    $tmp_nameA             = $tmp_key;
	    $tmp_nameB             = $ad_campaign_info{ $tmp_key }->{'name'} || $tmp_key;
	} elsif ( $statsby_indexB eq 'bannername' ) {
	    $tmp_nameA             = $tmp_key;
	    $tmp_nameB             = $ad_banner_info{ $tmp_key }->{'name'} || $tmp_key;
	} else {
	    $tmp_nameA             = $tmp_key;
	    $tmp_nameB             = $tmp_key;
	}

	$tmp_impressions           = $statsby_hash{ $tmp_key }->{ 'impressions' };

	next if ( ( $impressions_cutoff_limit > 1 ) && ( $tmp_impressions <  $impressions_cutoff_limit ) );

	$tmp_htmlad_impressions    = $statsby_hash{ $tmp_key }->{ 'htmlad_impressions' };
	$tmp_clickable_impressions = ( $tmp_impressions - $tmp_htmlad_impressions ) * 1 || 'N/A';
	$tmp_clicks                = $statsby_hash{ $tmp_key }->{ 'clicks' } || '0';
	$tmp_click_thru            = $tmp_clickable_impressions > 1  ? 
	                             substr( ( ( $tmp_clicks / $tmp_clickable_impressions ) * 100 ), 0, 4 ) : 0;

	$tmp_report_url             =~ s/\&?$statsby_indexA\=$tmp_key//;
	$tmp_report_linkA          = $tmp_report_url . "\&$statsby_indexA\=$tmp_key";


	if ( $sortby eq 'impressions' ) {
	    $tmp_key2 = "$tmp_impressions$tmp_key";
     	} elsif ( $sortby eq 'clicks' ) {    
	    $tmp_key2 = "$tmp_clicks$tmp_key";
	} elsif ( $sortby eq 'clickable' ) {
	    $tmp_key2 = "$tmp_clickable_impressions$tmp_key";
	} elsif ( $sortby eq 'clickthru' ) {
	    $tmp_key2 = "$tmp_click_thru$tmp_key";
	} elsif ( $sortby eq $statsby_indexB ) {
	    $tmp_key2 = "$tmp_nameB$tmp_key";
	    $alphasort++;
	} else {
	    $tmp_key2 = "$tmp_nameA$tmp_key";
	    $alphasort++;
	} # if sortby

	$tmp_nameB = substr($tmp_nameB, 0, 18) if $output_format eq 'PRINT';


	# If breakdown campaigns by site the add table rows for each site:

	if ( $statsby_indexA eq 'adcamp' ) {

	    if ( $breakdownby =~ /\w\w/ ) {

		foreach $tmp_key3 ( sort keys %{ $breakdown_campaign_stats{ $tmp_key } } ) {

		    $tmp_impressionsB           = $breakdown_campaign_stats{ $tmp_key }->{ $tmp_key3 }->{ 'impressions' };
		    $tmp_htmlad_impressionsB    = $breakdown_campaign_stats{ $tmp_key }->{ $tmp_key3 }->{ 'htmlad_impressions' };
		    $tmp_clickable_impressionsB = ( $tmp_impressionsB - $tmp_htmlad_impressionsB ) * 1 || 'N/A';
		    $tmp_clicksB                = $breakdown_campaign_stats{ $tmp_key }->{ $tmp_key3 }->{ 'clicks' } || '0';
		    $tmp_click_thruB            = $tmp_clickable_impressionsB > 1  ? 
	                             substr( ( ( $tmp_clicksB / $tmp_clickable_impressionsB ) * 100 ), 0, 4 ) : 0;
		    
		    $tmp_adname = "$tmp_key, $tmp_nameB";
		    		    
		    $sorttable_hash{ $tmp_key2 } .= join "\t", ( 
								 $tmp_adname,
								 $sitesconfig{ $tmp_key3 . 'sitename' } || &SysDate2ISODateDow( $tmp_key3 ),
								 $tmp_impressionsB,
								 $tmp_clickable_impressionsB,
								 $tmp_clicksB,
								 "$tmp_click_thruB\n" 
								 );

		} #foreach siteid

		$breakdown_total_label = $output_textonly ? 'TOTAL:' : '<B>TOTAL:</B>';
		$breakdown_row_spacer  = ( join "\t", @table_headers ) . "\n";


	    } # if breakdown by site

	} elsif ( $statsby_indexA eq 'adbanner' ) {

	    if ( $breakdownby =~ /\w\w/ ) {

		foreach $tmp_key3 ( sort keys %{ $breakdown_banner_stats{ $tmp_key } } ) {

		    $tmp_impressionsB           = $breakdown_banner_stats{ $tmp_key }->{ $tmp_key3 }->{ 'impressions' };
		    $tmp_htmlad_impressionsB    = $breakdown_banner_stats{ $tmp_key }->{ $tmp_key3 }->{ 'htmlad_impressions' };
		    $tmp_clickable_impressionsB = ( $tmp_impressionsB - $tmp_htmlad_impressionsB ) * 1 || 'N/A';
		    $tmp_clicksB                = $breakdown_banner_stats{ $tmp_key }->{ $tmp_key3 }->{ 'clicks' } || '0';
		    $tmp_click_thruB            = $tmp_clickable_impressionsB > 1  ? 
			                          substr( ( ( $tmp_clicksB / $tmp_clickable_impressionsB ) * 100 ), 0, 4 ) : 0;

		    $tmp_adname = "$tmp_key, $tmp_nameB";
		    
		    $sorttable_hash{ $tmp_key2 } .= join "\t", ( 
								 $tmp_adname,
								 $sitesconfig{ $tmp_key3 . 'sitename' } || &SysDate2ISODateDow( $tmp_key3 ),
								 $tmp_impressionsB,
								 $tmp_clickable_impressionsB,
								 $tmp_clicksB,
								 "$tmp_click_thruB\n" 
								 );

		} #foreach siteid

		$breakdown_total_label = $output_textonly ? 'TOTAL:' : '<B>TOTAL:</B>';
		$breakdown_row_spacer  = ( join "\t", @table_headers ) . "\n";

	    } # if breakdown by site

	} else {

	    $breakdown_total_label = '';
	    $breakdown_row_spacer  = '';	    

	} # if table is adcamp

        # Create sortable table array data:


	if ( $output_format !~ /SV/ ) {
	    $sorttable_hash{ $tmp_key2 } .= join "\t", ( 
						     $output_textonly ? $tmp_nameA : qq|<A HREF="$tmp_report_linkA">$tmp_nameA</A>|,
						     ( $output_textonly ? $tmp_nameB : qq|<A HREF="$tmp_report_linkA">$tmp_nameB</A>|) . $breakdown_total_label,
						     $tmp_impressions,
						     $tmp_clickable_impressions,
						     $tmp_clicks,
						     "$tmp_click_thru\n$breakdown_row_spacer" 
						     );
	}


    } # foreach $tmp_siteid


    # Sort stats hash and convert it a table:

    foreach $tmp_key ( sort { &sortAlphaNumeric($a,$b) } keys %sorttable_hash ) {
	$table_data .= $sorttable_hash{ $tmp_key };
    }


    $total_clickable_impressions   = ( $total_impressions - $total_htmlad_impressions ) * 1;
    $total_click_thru              = $total_clicks > 1 ? 
	                             substr( ( ( $total_clicks / $total_clickable_impressions ) * 100 ), 0, 4 ) :
		                     $total_clicks;
	
    $table_data .= join "\t", ( ' ',
				$output_textonly ? 'TOTALS:' : '<B>TOTALS:</B>',
				$total_impressions,
				$total_clickable_impressions,
				$total_clicks,
				"$total_click_thru\n" 
				);

    &makeTable( $output_format );
    $page_contents .= $table_contents;


}  # sub makeAdStatsTable




__END__







