#!/usr/bin/perl

# CGI and command line interface to compare ad stats for
# a given site to it's past ad allocation in order to verify
# whther the ad stats logs contain extranious ad impressions.
#
# Modified:   Kurt Gray -- Fri Dec 22 12:41:09 EST 2000      First rev. based on ad_report.pl
#


use Date::Manip;

push @INC, '/usr/local/adsystem/lib/';
require 'ad_config_lib.pl';	        # Site-specific settings for adsystem control scripts
require 'ad_util_lib.pl';	        # Common functions for adsystem scripts
require 'ad_email_lib.pl';

&unbufferSTDOUT;


{

# Read command line options

    foreach $arg (@ARGV) {

	# Option to specify a site

	if ( $arg =~ /site=(\w+)/ ) {
	    $siteid = $1;

	# Debug option shows what's happening without writing new logs
	# and a Ctrl+C keystroke will stop script and dumps stats gathered so far

	} elsif ( $arg =~ /debug/ ) {
	    $debugging = 1;
	    # $dontsavelogs = 1;
	    # $SIG{INT} = dumpStats;

	# Option to get stats for a specific date (date=yesterday is useful for regenerating yesterday's stats)

	} elsif ( $arg =~ /date=(\d{6,8})\-?(\d{0,8})/ ) {
	    $StartDate_in = $1;
	    $StopDate_in  = $2;

	# Print summary stats only

	} elsif ( $arg =~ /showsummary/ ) {
	    $showsummary = 1;


        # Ad campaign to show stats for:

	} elsif ( $arg =~ /adcamp=(\w+)/ ) {
	    $ad_campaign = $1;

        # Ad banner to show stats for:

	} elsif ( $arg =~ /adid=(\w+)/ ) {
	    $ad_banner_id = $1;


        # Table data output format:

	} elsif ( $arg =~ /format=(\w+)/ ) {
	    $output_format = $1;

        # Option to breakout campaign stats by siteid or logdate:

	} elsif ( $arg =~ /fixlogs/ ) {
	    $rewrite_adlogs++;

	# Warn about unparsed inputs
	} else {
	    print "ERROR: $0 does not understand argument: $arg <BR>\n";
	    exit;
	} # if arg

    } # foreach ARGV



# If we didn't get any command line inputs then read CGI web inputs:

    if ( ( $ad_campaign || $StartDate_in || $siteid || $debugging ) ) {

	$offline_mode++;
	$output_format ||= 'CSV';   # Print ad report data to STDOUT, comma separated values

    } else {

	$online_mode++;

	# Login user and start printing output web page:

	&loginUser( $admin_access );

	# Adjust navbar links:

	# delete $nav_links->{ $sales_dept }->{'Ad Reports'};

	# Read CGI inputs:

	$StartDate_in   = $1 if $form->{'in'}->{ 'StartDate'   } =~ /([\d\/\-]+)/;
	$StopDate_in    = $1 if $form->{'in'}->{ 'StopDate'    } =~ /([\d\/\-]+)/;
	$ad_campaign_in = $1 if $form->{'in'}->{ 'adcamp'      } =~ /(\w{4})/;
	$ad_banner_in   = $1 if $form->{'in'}->{ 'adbanner'    } =~ /(\w{8,12})/;
	$siteid         = $1 if $form->{'in'}->{ 'siteid'      } =~ /(\w{7})/;
	$output_format  = $1 if $form->{'in'}->{ 'format'      } =~ /(\w{3,6})/;
	$sortby         = $1 if $form->{'in'}->{ 'sortby'      } =~ /(\w+)/;
	$breakdownby    = $1 if $form->{'in'}->{ 'breakdownby' } =~ /(\w+)/;
	$op             = $1 if $form->{'in'}->{ 'op'          } =~ /(\w+)/;
	$emailto        = $1 if $form->{'in'}->{ 'emailto'     } =~ /(\S{5,80})/;
	$daterange_confirmed = $1 if $form->{'in'}->{'dates_confirmed'} =~ /(\w)/;

	$thisscript = $ENV{'SCRIPT_NAME'};

    } # if CGI inputs


# Verify inputs:

    if ( $siteid !~ /\w+/ ) {
	print "ERROR: This script requires a siteid input. \n";
	exit;
    }



# Set script defaults:

    $ad_campaign                = $username if $user_access_level <= $advertiser_access;
    $ad_campaign              ||= $ad_campaign_in || substr( $ad_banner_in, 0, 4);

    $ad_id                      = $ad_banner_in || $ad_campaign;
    $site_initials              = $1 if $siteid =~ /^(\w{2})\w+/;
    $output_format            ||= 'HTML';
    $output_textonly            = 1 if $output_format =~ /SV/ || $offline_mode;
    $sortby                   ||= 'impressions';
    $impressions_cutoff_limit ||= 80;  # Don't show stats for any campaign, site, or banner with less than this many impressions
    $numberofdays_limit       ||= 150;
    $op                         = 'emailto' if $emailto;

    &getCampaignsInfo( $ad_campaign );   # Make global hash of all ad campaign ids => name, numberofads
    $ad_campaign_name         ||= $ad_campaign_info{ $ad_campaign }->{'name'};

    &setCampaignNavlinks;

    # Default date range to past 2 days

    $StopDate_in ||= &Date::Manip::UnixDate("today", '%y%m%d');   # logdate format is yymmdd
    if ( $StartDate_in !~ /\d\d/ ) {
	$tmp_date     = &Date::Manip::ParseDateString( "today" );
	$tmp_date     = &Date::Manip::DateCalc( $tmp_date, "- 1 days" );
	$StartDate_in = &Date::Manip::UnixDate("$tmp_date", '%y%m%d');
    }

    &parseDateInputs;  # Convert StartDate_in and StopDate_in to various formats and setup @logdates list



# Setup list of report options to include ad report details links:

    %report_options  = ( 
			 'StartDate', $StartDate_yymmdd,
			 'StopDate', $StopDate_yymmdd,
			 'format', $output_format 
			 );

    $report_options{ 'siteid' }   = $1 if $siteid       =~ /(\w{7})/;
    $report_options{ 'adcamp' }   = $1 if $ad_campaign  =~ /(\w{4})/;
    $report_options{ 'adbanner' } = $1 if $ad_banner_in =~ /(\w{8,12})/;
    $report_options{ 'breakdownby' } = $1 if $breakdownby =~ /(\w+)/;
    $report_options{ 'sortby' } = $1 if $sortby =~ /(\w+)/;

    while (($key,$value) = each %report_options ) {
	push @report_settings, "$key\=$value";
    }

    $tmp_report_url = $thisscript . '?';
    $tmp_report_url .= ( join '&', @report_settings );



# Setup options form:

    if (( $online_mode ) && ( $output_format ne 'PRINT' )) {

	$page_contents .= qq|<FORM METHOD=POST ACTION="$thisscript">Site ID: <INPUT SIZE=8 NAME=siteid VALUE="$siteid"> from Start Date: <INPUT NAME=StartDate SIZE=8 MAXLENGTH=12 VALUE="$StartDate_nice"> &nbsp; to Stop Date: <INPUT NAME=StopDate SIZE=8 MAXLENGTH=12 VALUE="$StopDate_nice"> $confirm_daterange <BR>
Output Format: |;

	%drop_list = ( 'HTML',  'Normal',
		       'PRE',   'Text',
		       'CSV',   'CSV',
		       'TSV',   'TSV',
		       'PRINT', 'Printable' );

	&makeDropList( "format", "$output_format" );

	# $page_contents .= qq| Sort by: |;

	%drop_list = ( 'adcamp',      'Campaign ID',
		       'campname',    'Campaign Name',
		       'impressions', 'Impressions',
		       'clicks',      'Clicks',
		       'clickthru',   'Click-Thru',
		       'clickable',   'Clickable',
		       'campname',    'Campaign Name',
                       'networkid',   'Network',
		       'siteid',      'Site Name',
		       'adbanner',    'Banner ID',
		       'bannername',  'Banner Name' );

	# &makeDropList( "sortby", "$sortby" );

	# $page_contents .= qq| Breakdown: |;

	%drop_list = ( ' ',         'Normal',
		       'siteid',    'by Site',
		       'logdate',   'by Date', );

	# &makeDropList( "breakdownby", "$breakdownby" );

	if ( $ad_system_users{ $username }->{'email'} ) {
	    $page_contents .= qq| <INPUT TYPE=CHECKBOX NAME="op" VALUE="emailto"> Email this to me |;
	}

	$page_contents .= qq|  &nbsp; <INPUT TYPE=SUBMIT NAME=GoButton VALUE="Show Report"></FORM>\n|;

    } # if web options



# Read all ad_allocation files for the given web site

    @ad_positions = split ',', $sitesconfig{ $siteid . 'ad_positions' };

    if ( ! @ad_positions ) {
	print "ERROR: Can not scan current ad allocation files for $siteid: No ad positions found in sitesconfig.\n";
	exit;
    }

    # Read all current ad allocation files for this siteid:

    foreach $ad_position ( @ad_positions ) {
	$allocfile = "$datfilepath/ad\_$ad_position\_alloc.dat\.$siteid";
	if ( open INFILE, "< $allocfile" ) {
	    while (<INFILE>) {
		( $tmp_adid, $tmp_impressions, $tmp_stopontarget, $tmp_startdate, $tmp_stopdate ) = split /\t/, $_;
		$ads_allocated{ $tmp_adid } += $tmp_impressions;
	    }
	    close INFILE;
	    $messages .= "DEBUG: Scanned $allocfile <BR>\n" if $debugging == 1;
	} else {
	    $error_messages .= "ERROR: Could not read $allocfile: $!<BR>"; 
	}
    } #foreach ad_position

    # Read all previous ad allocation files for this siteid:

    if ( opendir DIR, $backupfilepath ) {
	@allocfiles = grep /alloc\.dat\.$siteid/, readdir DIR;
	if ( ! @allocfiles ) {
	    print "WARNING: Did not find any previous ad allocation files in $backupfilepath for $siteid. <BR>\n";
	} else {
	    foreach $tmp_allocfile ( @allocfiles ) {
		$allocfile = "$backupfilepath/$tmp_allocfile";
		if ( open INFILE, "< $allocfile" ) {
		    while (<INFILE>) {
			( $tmp_adid, $tmp_impressions, $tmp_stopontarget, $tmp_startdate, $tmp_stopdate ) = split /\t/, $_;
			$ads_allocated{ $tmp_adid } += $tmp_impressions;
		    }
		    close INFILE;
		    $messages .= "DEBUG: Scanned $allocfile <BR>\n" if $debugging == 1;
		} else {
		    $error_messages .= "ERROR: Could not read $allocfile: $!<BR>"; 
		}
	    } # foreach allocfile
	} # if @allocfiles
	closedir DIR;
    } else {
	$error_messages .= "ERROR: Could not scan dir $backupfilepath for previous ad allocation files: $! <BR>\n";
    } # if opendir backupfilepath


# Read all adlog files in given date range:

    $stats_by_siteid           = {};
    $stats_by_campaign         = {};
    $stats_by_bannerid         = {};
    $breakdown_campaign_stats  = {};
    $breakdown_banner_stats    = {};

    $numberofmondays = int( $numberofdays / 7 ) || 1;

    foreach $logdate ( @logdates ) {

	$tmp_logdate  = &logdate2SysDate( $logdate );
	if ( $breakdownby eq 'weekday' ) { 
	    $tmp_logdate2 = &logdate2SysDateDow( $logdate );
	    $tmp_dow     = "$1 (x$numberofmondays)" if $tmp_logdate2 =~ / (\w{3})$/;
	}

	$adlogfile = $adlogpath . '/ads_' . $logdate . '.log';

	if ( open ( INFILE, "< $adlogfile" ) ) {
	    $total_reported_today = 0;
	    $total_not_allocated_today = 0;
	    $new_adlog_data = '';
	    $messages .= "DEBUG: Reading $adlogfile... <BR>\n" if $debugging == 1;
	    while ($line = <INFILE>) {
		chomp $line;
		($tmp_adid, $tmp_impressions, $tmp_clicks, $tmp_siteid) = split(/\t/, $line);

		if ( $tmp_siteid ne $siteid ) {
		    $new_adlog_data .= $line;
		    $new_adlog_data .= "\n";
		    next;
		}

		# Determine the campaign ID and ad type (HTML ad or normal ad)

		$tmp_campaign = substr( $tmp_adid, 0, 4 );
		$tmp_adtype   = substr( $tmp_adid, 4, 1 );
		
		# $messages .= "DEBUG: Tallying stats for $tmp_adid on $tmp_siteid on $logdate... <BR>\n" if $debugging == 1;

		$stats_by_bannerid{ $tmp_adid }->{ 'reported' }  += $tmp_impressions;
		if ( $ads_allocated{ $tmp_adid } ) {
		    $new_adlog_data .= $line;
		    $new_adlog_data .= "\n";
		} else {
		    $stats_by_bannerid{ $tmp_adid }->{ 'not_allocated' } += $tmp_impressions;
		    $total_not_allocated_today += $tmp_impressions;
		    $total_not_allocated_by_banner += $tmp_impressions;
		}
		$total_reported_today += $tmp_impressions;
		$total_reported_by_banner += $tmp_impressions;
	
	    } # while INFILE
	    
	    $stats_by_date{ $logdate }->{ 'reported' } = $total_reported_today;
	    $stats_by_date{ $logdate }->{ 'not_allocated' } = $total_not_allocated_today;

	    $total_reported += $total_reported_today;
	    $total_not_allocated += $total_not_allocated_today;

	    close INFILE;

	    # Save corrected ad log file:

	    if (( $total_not_allocated_today > 10 ) && ( $rewrite_adlogs )) {
		$new_adlog_file = "/tmp/ads\_$logdate.log";
		if ( open OUTFILE, "> $new_adlog_file" ) {
		    print OUTFILE $new_adlog_data;
		    close OUTFILE;
		    $messages .= "NOTICE: Saved corrected ad log file: $new_adlog_file <BR>\n"; 
		} else {
		    $error_messages .= "ERROR: Could not create $new_adlog_file : $! <BR>\n";
		}
	    } # if new_adlog_data

	} else {
	    $error_messages .= "WARNING: Could not read ad stats for day of $logdate: $adlogfile: $! <BR>\n";
	}


    } # foreach logdate



# Print stats by date:

    $table_data = "Logdate\tReported Impressions\tNot Allocated\tCorrectly Delivered\n";

    foreach $logdate ( sort keys %stats_by_date ) {

	$tmp_impressions_reported = $stats_by_date{ $logdate }->{ 'reported' };
	$tmp_impressions_not_allocated = $stats_by_date{ $logdate }->{ 'not_allocated' };
	$tmp_delivered_correctly = $tmp_impressions_reported - $tmp_impressions_not_allocated;

	$tmp_logdate = &logdate2ISODate( $logdate );

	$table_data .= join "\t", ( $tmp_logdate,
				    $tmp_impressions_reported,
				    $tmp_impressions_not_allocated,
				    $tmp_delivered_correctly,
				    "\n" );

    } # foreach logdate

    $total_delivered_correctly = $total_reported - $total_not_allocated;
    $table_data .= join "\t", ( '<B>TOTAL</B>',
				$total_reported,
				$total_not_allocated,
				$total_delivered_correctly,
				"\n" );

    &makeTable( $output_format );
    $page_contents .= $table_contents;



# Print stats by Ad ID:

    $table_data = "Logdate\tReported Impressions\tNot Allocated\tCorrectly Delivered\n";

    foreach $adid ( sort keys %stats_by_bannerid ) {

	$tmp_impressions_reported = $stats_by_bannerid{ $adid }->{ 'reported' };
	$tmp_impressions_not_allocated = $stats_by_bannerid{ $adid }->{ 'not_allocated' };
	$tmp_delivered_correctly = $tmp_impressions_reported - $tmp_impressions_not_allocated;

	$table_data .= join "\t", ( $adid,
				    $tmp_impressions_reported,
				    $tmp_impressions_not_allocated,
				    $tmp_delivered_correctly, 
				    "\n" );

    } # foreach logdate

    $total_delivered_correctly = $total_reported_by_banner - $total_not_allocated_by_banner;
    $table_data .= join "\t", ( '<B>TOTAL</B>',
				$total_reported_by_banner,
				$total_not_allocated_by_banner,
				$total_delivered_correctly,
				"\n" );

    &makeTable( $output_format );
    $page_contents .= $table_contents;
    &printWebpage( $output_format );

}

__END__










