#!/usr/bin/perl

#
# This CGI script is the interface used to 
# set the reserved pages on each web site.
#
# Notes:
#
#
# Modified:   Kurt Gray -- Mon Nov 13 14:22:05 EST 2000     First rev. adapted from ad_reservationedit.pl on each remote site
#             Kurt Gray -- Thu Apr 19 12:15:55 EDT 2001     Added impressions input

use Date::Manip;

push @INC, '/usr/local/adsystem/lib/';
require 'ad_config_lib.pl';	        # Site-specific settings for adsystem control scripts
require 'ad_util_lib.pl';	        # Common functions for adsystem scripts


# Script settings and base inputs:

    $siteid                    = $1 if $ENV{'QUERY_STRING'} =~ /siteid=(\w{7})/;
    $site_initials             = substr( $siteid, 0, 2 );
    $sitename                  = $sitesconfig{ $siteid . 'sitename' };
    @ad_height_restrictions    = split( ',', $sitesconfig{ $siteid . 'ad_layout' });
    $can_run_megabanners       = 0 if $sitesconfig{ $siteid . 'ad_restrictions' } =~ /MEGABANNERS=0/;
    @ad_positions_list         = (sort keys %adtypes);
    $thisscript                = "$ad_reservations_url\?siteid=$siteid";
    $allocfile                 = "ad_reservations_alloc.dat\.$siteid";
    $adtype                    = $adtypes{$ad_position};

    $yesterday                 = &Date::Manip::UnixDate("yesterday", '%y%m%d');
    $today                     = &Date::Manip::UnixDate("today", '%y%m%d');
    $todayfullsysdate          = &Date::Manip::UnixDate("today", '%Y%m%d');

    $yesterday_adlog           = $adlogpath . '/ads_' . $yesterday . '.log';  
    $pagetrafficlog            = $weblogpath . '/$siteid/pages_' . $yesterday . '.log';
    

# Start printing output web page

    &loginUser( $admin_access );

    if ( ! $todayfullsysdate ) {
	print "DATE ERROR: Could not determine today's full sysdate. ";
	exit;
    }
    
    if ( ! $siteid ) {
	print "INPUT ERROR: siteid input missing or invalid. ";
	exit;
    }


# Make a list of web sites that share allocations with this web site

    $allocation_shared_with = '';
    foreach $tmp_logname ( sort keys %logname2siteid ) {
	$tmp_siteid = $logname2siteid{$tmp_logname};
	next if $sitesconfig{ $tmp_siteid . 'ad_allocation_group' } ne $site_initials;
	$allocation_shared_with .= " " . $sitesconfig{ $tmp_siteid . 'sitename' } . " ";
    }

    if ( $allocation_shared_with ) {
	$allocation_shared_with = "\n <BR> $sitename includes: $sitename $allocation_shared_with";
    }


# Read ad impressions delivered to each ad yesterday on all sites that share ad allocations with this web site

    if ( open ( INFILE, "< $yesterday_adlog" ) ) {
	while (<INFILE>) {
	    chomp;
	    ($adid,$impr,$cliks,$tmp_siteid) = split(/\t/, $_);
	    if ( $tmp_siteid ne $siteid ) {
		if ( $sitesconfig{ $tmp_siteid . 'ad_allocation_group'} ne $site_initials ) {
		    next;
		}
	    }
	    $ad_impressions_yesterday{$adid} += $impr;
	}
	close INFILE;
    } else {
	$error_messages .= "<BR>ERROR: Could not read ad stats log $yesterday_adlog: $! ";
    }
	

# TODO: Read pages_(yesterday).log and note how many impressions this page delivered yesterday


# Notify user of any ad type restrictions for this web site

    if ( $can_run_megabanners != 1 ) {
	$messages .= "<BR><BR>NOTICE: This site does not accomodate megabanners.<BR>";
    }


# Build a hash of ads available in ad_info.dat for this ad position

    if ( open( ADINFILE, "< $adinfofile" ) ) {
	while (<ADINFILE>) {
	    chomp;
	    ($AdID,$ht,$wh,$tt,$at,$re) = split(/\t/,$_);
		
	    if ( $can_run_megabanners != 1 ) {
		if ( (substr($AdID, 4, 1)) eq '6' ) {
		    $error_messages .= "<BR>Skipping $AdID since this site can not run megabanners.\n" if $debugging == 1;
		    next;
		}
	    }
	    
	    if ( $verify_gifs_exists == 1 ) {
		$gif_filename = "$gif_filepath/$AdID.gif";
		if ( !( -e $gif_filename )) {
		    $gif_errors .= "<BR>NOTICE: <A HREF=\"$ad_infoedit_url\?adid=$AdID\" TARGET=TomThumb>$AdID</A> does not have a GIF image in $gif_filepath\n";
		    next;
		}
	    }	
	    $AdList{$AdID} = $AdID;
	}
	close ADINFILE;
	@adids2 = (sort keys %AdList);
	
    } else {
	$error_messages .= "<BR>ERROR: Could not read ad_info file $adinfofile : $! <BR> ";
    }  # if ad_info.dat


# Read all CGI form inputs

    $file_data = '';
    $i = '';
    
    foreach $key ( keys %{$form->{'in'}} ) {

	next if $key !~ /^ad_pagename_(\d+)$/;
	
	$i = $1;
	
	if ( $i !~ /\d+/ ) {
	    $error_messages .= "<BR> ad_pagename form key invalid : $key ";
	    next;
	}

	$ad_kill          = $form->{'in'}->{ 'ad_kill_'      .  $i };
	$ad_pagename      = $form->{'in'}->{ 'ad_pagename_'  .  $i };
	$ad_position      = $form->{'in'}->{ 'ad_position_'  .  $i };
	$ad_id            = $form->{'in'}->{ 'ad_id_'        .  $i };
	$ad_startdate     = $form->{'in'}->{ 'ad_startdate_' .  $i };
	$ad_stopdate      = $form->{'in'}->{ 'ad_stopdate_'  .  $i };
	$ad_impressions   = $form->{'in'}->{ 'ad_impressions_'  .  $i };

	$ad_startdate     = &NiceDate2ISODate( $ad_startdate ) if $ad_startdate =~ /\d\/\d/;
	$ad_stopdate      = &NiceDate2ISODate( $ad_stopdate  ) if $ad_stopdate  =~ /\d\/\d/;
	
	if ( $ad_pagename !~ /\w+/ ) {
	    # $messages .= "<BR>WARNING: Entry ignored : No password : $i  $ad_pagename   $ad_pagename_name   $ad_password ";
	    next;
	}


	if ( $ad_id !~ /\w+/ ) {
	    $messages .= "<BR>WARNING: Entry ignored : No ad_id read : $ad_pagename, $ad_position ";
	    next;
	}


	if ( $ad_kill eq 'T' ) {
	    $messages .= "<BR>NOTICE: Entry removed : $ad_pagename, $ad_position, $ad_id ";
	    next;
	} 

	$file_data .= "$ad_pagename\t$ad_position\t$ad_id\t$ad_startdate\t$ad_stopdate\t$ad_impressions\n";
	
    } # foreach form input


# If no form inputs read then read from file

    if (( $i < 1 ) && ( $file_data !~ /\w+/ )) {
	
	if ( open( INFILE, "< $datfilepath/$allocfile" ) ) { 
	    while (<INFILE>) {
		$file_data .= $_;
	    }
	    close INFILE;
	} else {
	    $error_messages .= "<BR>ERROR: Could not read reserved page file $datfilepath/$allocfile : $! ";
	}

    } # if file_data


# Produce a web form to edit file_data
    
    $i = 0;
    foreach $line ( sort ( split /\n/, $file_data ) ) {
	
	$i++;
	
	if ( $reserved_pages_limit > 1 ) {
	    if ( $i > $reserved_pages_limit ) {
		$error_messages .= "<BR>WARNING: Number of reserved pages exceeds server limit of $reserved_pages_limit ";
		$error_messages .= "<BR>Reserved pages listed beyond this limit were not saved in this list.";
		last;
	    }
	}

	( $ad_pagename, $ad_position, $ad_id, $ad_startdate, $ad_stopdate, $ad_impressions ) = split /\t/, $line;
	
	$ad_positions_droplist      = "<SELECT NAME=\"ad_position\_$i\">";
	for $tmp_ad_position ( @ad_positions_list ) {
	    $ad_positions_droplist .= "<OPTION VALUE=$tmp_ad_position";
	    $ad_positions_droplist .= ' SELECTED' if $tmp_ad_position == $ad_position;
	    $ad_positions_droplist .= '>' . $adtypes{$tmp_ad_position};
	}
	$ad_positions_droplist     .= "</SELECT>";

	if ( $AdList{ $ad_id } ) {
	    $page_contents .= "\n<INPUT NAME=\"ad_kill\_$i\" TYPE=CHECKBOX VALUE=\"T\">   ";
	    $page_contents .= "<INPUT NAME=ad_pagename\_$i SIZE=15 VALUE\=$ad_pagename >  ";
	    $page_contents .= $ad_positions_droplist;
	    $page_contents .= "<INPUT TYPE=HIDDEN NAME=ad_id\_$i VALUE\=$ad_id > ";
	    $page_contents .= "<A HREF=\"$ad_infoedit_url\?adid=$ad_id\" TARGET=TomThumb>$ad_id</A>\t";
	} else {
	    $page_contents .= "\n<INPUT NAME=\"ad_kill\_$i\" TYPE=CHECKBOX VALUE=\"T\" CHECKED>   ";
	    $page_contents .= "<INPUT NAME=ad_pagename\_$i SIZE=15 VALUE\=$ad_pagename >  ";
	    $page_contents .= $ad_positions_droplist;
	    $page_contents .= "<INPUT TYPE=HIDDEN NAME=ad_id\_$i VALUE\=$ad_id > ";
	    $page_contents .= " <STRIKE>$ad_id</STRIKE> \t";
	}

	$page_contents .= "<INPUT NAME=\"ad_impressions\_$i\" SIZE=11 MAXLENGTH=11 VALUE=\"$ad_impressions\">  ";
	$page_contents .= "<INPUT NAME=\"ad_startdate\_$i\" SIZE=11 MAXLENGTH=11 VALUE=\"$ad_startdate\">  ";
	$page_contents .= "<INPUT NAME=\"ad_stopdate\_$i\" SIZE=11 MAXLENGTH=11 VALUE=\"$ad_stopdate\">";
	$page_contents .= '    ' . $ad_impressions_yesterday{$ad_id};
	
    } # foreach line


# Stop here if we don't have any data to display

    if ( $i < 1 ) {
	$messages .= "<BR>NOTICE: Didn't read any reserved pages $allocfile data from CGI or file on disk";
	# exit;
    }


# TODO: Check $admin_access_level and don't allow underprivleged users to save this file
# Save new ad password file data

    if ( $form->{'in'}->{'GoButton'} =~ /Save Changes/ ) {
      
	if ( $user_access_level < $admin_access ) {
	    $error_messages .= "ERROR: Can not save file: User $username does not have ad system control privileges. <BR>\n"; 
	} else {
	    &BackupAndSaveFile( "$datfilepath/$allocfile" );
	}

    } # if save changes


    $total = $i;
    $i++;  # increase inputs counter by number of new inputs
    

    $ad_positions_droplist      = "<SELECT NAME=\"ad_position\_$i\">";
    for $tmp_ad_position ( @ad_positions_list ) {
	$ad_positions_droplist .= "<OPTION VALUE=$tmp_ad_position";
	$ad_positions_droplist .= ' SELECTED' if $tmp_ad_position == $ad_position;
	$ad_positions_droplist .= '>' . $adtypes{$tmp_ad_position};
    }
    $ad_positions_droplist     .= "</SELECT>";


    print "<FONT SIZE=+1><B>$sitename Reserved Pages: $total Total</B></FONT> 
&nbsp; [<A HREF=\"$ad_allocedit_url\">Return to Main Menu</A>]
<BR> $messages $gif_errors <B><FONT COLOR=RED> $error_messages </FONT></B>
</FONT><PRE><FORM METHOD=POST ACTION=$thisscript>

<B>New:\t\tPagename\tAd Position\t\t\tAd ID\t\tImpressions\tStartDate\tStopDate</B>
        <INPUT NAME=ad_pagename\_$i SIZE=15 > $ad_positions_droplist <SELECT NAME=\"ad_id\_$i\">";

    foreach $tmp_ad_id ( sort keys %AdList ) {
	print "<OPTION VALUE=$tmp_ad_id>$tmp_ad_id</OPTION>\n";
    }

    print "</SELECT> <INPUT NAME=\"ad_impressions\_$i\" SIZE=11 MAXLENGTH=11 VALUE=\"$ad_impressions\"> <input name=\"ad_startdate\_$i\" SIZE=11 MAXLENGTH=11>  <input name=\"ad_stopdate\_$i\" SIZE=11 MAXLENGTH=11>
        <INPUT TYPE=SUBMIT NAME=GoButton VALUE=\" Save Changes for $sitename \">

<B>Kill?:\t\tPagename\tAd Position\t\tAd ID\t\tImpressions\tStartDate\tStopDate\tDelivered Yesterday</B>
$page_contents

       <INPUT TYPE=\"SUBMIT\" NAME=GoButton value=\" Save Changes for $sitename \">

<B>$total Reserved Pages for $sitename</B>

[<A HREF=\"$ad_allocedit_url\">Return to Main Menu</A>]

</FORM>
</PRE>
</BLOCKQUOTE>
</FONT>
</BODY>
</HTML>\n";



sub NiceDate2ISODate {

    local ($nicedate) = @_;
    ($mon,$day,$year) = split(/[\/\-]/, $nicedate);
    $day =~ s/^0//;
    $day = '0' . $day if $day < 10;
    $mon =~ s/^0//;
    %monthlist = qw/1 Jan 2 Feb 3 Mar 4 Apr 5 May 6 Jun 7 Jul 8 Aug 9 Sep 10 Oct 11 Nov 12 Dec/;
    $year = `/bin/date +%Y` unless $year =~ /\d\d\d\d/;
    $year =~ s/\W//g;
        
    return $day . '/' . $monthlist{$mon} . '/' . $year;

}  # Converts 2/03/98 to 03/Mar/1998


__END__

