#!/usr/bin/perl

#
# Cron script to send scheduled ad system emails 
#
# Modified:  Kurt Gray -- Thu Jan 11 15:03:27 EST 2001      First rev.


push @INC, '/usr/local/adsystem/lib/';
require 'ad_config_lib.pl';	        # Site-specific settings for adsystem control scripts
require 'ad_util_lib.pl';	        # Common functions for adsystem control scripts
require 'ad_email_lib.pl';	        # Email send functions for adsystem control scripts

{

# Read command line options

    foreach $arg (@ARGV) {

	# Option to specify a site

	if ( $arg =~ /site=(\w+)/ ) {
	    $siteid = $1;

	# Debug option shows what's happening without writing new logs
	# and a Ctrl+C keystroke will stop script and dumps stats gathered so far

	} elsif ( $arg =~ /debug/ ) {
	    $debugging = 1;
	    # $dontsavelogs = 1;
	    # $SIG{INT} = dumpStats;

	# Option to get stats for a specific date (date=yesterday is useful for regenerating yesterday's stats)

	} elsif ( $arg =~ /date=(\d{6,8})\-?(\d{0,8})/ ) {
	    $StartDate_in = $1;
	    $StopDate_in  = $2;

	# Send all email to this address:

	} elsif ( $arg =~ /allmailto=([\w\/\.\-\@]+)/ ) {
	    $email_allmailto = $1;

	# Warn about unparsed inputs
	} else {
	    print "ERROR: $0 does not understand argument: $arg \n";
	    exit;
	} # if arg
    }


# Script defaults:

    $email_logfile ||= '/usr/local/adsystem/local/logs/email_log';


# Figure out today's date:

    ( $today_SysDate, $today_dow, $today_hour )  = ( $1, $2, $3 ) if `/bin/date +'%Y%m%d %a %H'` =~ /^(\d+) (\w{3}) (\d+)/;
    $today_mday                     = substr( $today_SysDate, 6, 1);
    $today_lastdom                  = $today_mday == 3 ? 1 : '';
    $today_ISODate                  = &SysDate2ISODate( $today_SysDate );

    if ( $debugging == 1 ) {
	print "DEBUG: \$today_SysDate = $today_SysDate, \$today_dow = $today_dow, \$today_hour = $today_hour\n";
	print "DEBUG: \$today_ISODate = $today_ISODate\n";
	print "DEBUG: \$today_mday    = $today_mday, \$today_lastdom = $today_lastdom\n";
    }
    

# Read in list of emails already sent today:

    $sent_mail = {};
    if ( open INFILE, "< $email_logfile" ) {
	$line = <INFILE>;
	$email_logdate = $1 if $line =~ /^(\d+)/;
	while ($line = <INFILE>) {
	    ( $email_id,
	      $email_type,
	      $email_to,
	      $email_from,
	      $email_sent ) = split /\t/, $line;
	    next if ! $email_sent;
	    $sent_mail{ $email_id } = $line;
	}
	close INFILE;
	print "DEBUG: Read " . (keys %sent_mail)  . " sent mail records from $email_logfile \n" if $debugging == 1;
    } else {
	print "DEBUG: $email_logfile not read : $! \n" if $debugging == 1;
    }

    print "DEBUG: \$email_logdate = $email_logdate\n" if $debugging == 1;
    if ( $email_logdate ne $today_SysDate ) {
	`/bin/cp /dev/null $email_logfile`;
	$sent_mail = {};
	print "DEBUG: Email log date changed: Email log reset.\n" if $debugging == 1;
    } 


    $new_sent_mail = {};

# Read list of notes to be sent today:

    &getUserInfo;

    if ( open INFILE, "< $notes_idxfile" ) {
	print "DEBUG: Scanning $notes_idxfile for dated NOTES...\n" if $debugging == 1;
	$numbersent = 0;
	while (<INFILE>) {
	    chomp;
	    ( $note_id,
	      $note_parentid,
	      $note_context,
	      $note_type,
	      $note_private,
	      $tmp_username,
	      $note_subject,
	      $note_emailat,
	      $note_emailcc,
	      $upload_filename ) = split /\t/, $_;

	    if ( ! $note_emailat ) {
		print "DEBUG: Skipping note $note_id: No Email settings\n" if $debugging_verbose == 1;
		next;
	    }

	    # Don't send it again if it was already sent today:

	    if ( $sent_mail{ $note_id } ) {
		print "DEBUG: Skipping note $note_id: Already sent.\n" if $debugging == 1;
		next;
	    }

	    if ( $note_emailat eq 'EmailMonthly' ) {
		if ( ! $today_lastdom ) {
		    print "DEBUG: Skipping note $note_id: Not last day of the month.\n" if $debugging == 1;
		    next;
		}
	    } elsif ( $note_emailat =~ /EmailWeekly\d(\w{3})/ ) {
		$email_dow = $1;
		if ( $today_dow ne $email_dow ) {
		    print "DEBUG: Skipping note $note_id: Day of week $email_dow is not today.\n" if $debugging == 1;
		    next;
		}
	    } elsif ( $note_emailat =~ /EmailDaily/ ) {
		# continue;
	    } elsif ( &ISODate2SysDate( $note_emailat ) ne $today_SysDate ) {
		print "DEBUG: Skipping note $note_id: $note_emailat ne $today_ISODate.\n" if $debugging == 1;
		next;
	    }
	    
	    # Note email date matches date criteria, send it:

	    $from_addr       = $ad_system_users{$tmp_username}->{'email'} || $email_admin;
	    $to_addr         = $ad_system_users{$tmp_username}->{'email'};
	    $to_addr        .= ",$note_emailcc" if $note_emailcc;
	    $email_subject   = "$email_subject_prefix REMINDER: $note_subject";
	    $tmp_attachment_link = $upload_filename ? 
		qq|Attachment: $system_baseurl$notes_uploadbaseurl/$upload_filename| : '';
	    $note_message_filepath = "$notes_messagepath/$note_id.txt";

	    if ( -f "$note_message_filepath" ) {
		$note_message = &ReadFile("$note_message_filepath");
	    } else {
		$note_message = "(no message body)";
	    }

	    $email_message  = qq|The following message is an $note_emailat scheduled reminder posted in the OSDN Ad System:\n\n|;
	    $email_message .= qq|$note_message\n\n$tmp_attachment_link\nPosted at: $system_baseurl$ad_notes_url?note_id=$note_id\n|;
	    &SendEmail;
	    $new_sent_mail{ $note_id } = join "\t", ( $note_id,
						      'NOTE',
						      $to_addr,
						      $from_addr,
						      time,
						      "\n" );

	    print "DEBUG: Sent mail: $new_sent_mail{ $note_id }" if $debugging == 1;
	    
	    $numbersent++;

	} # while INFILE
	close INFILE;
	print "DEBUG: $numbersent email NOTES found not already sent.\n" if $debugging == 1;
    } else {
	print "DEBUG: Could not read notes idx file $notes_idxfile : $! \n" if $debugging == 1;
    }
    

# Append or overwrite today's email_log:

    if ( open OUTFILE, ">> $email_logfile" ) {
	print OUTFILE "$today_SysDate $today_hour: Ad System email scheduler finished: $numbersent emails sent.\n";
	while ( ( $email_id, $email_info ) = each %new_sent_mail ) {
	    print OUTFILE $email_info;
	}
	print OUTFILE $error_messages;
	close OUTFILE;
	print "DEBUG: Email log updated.\nDone\n." if $debugging == 1;
    } else {
	print "ERROR: Could not append to email log $email_logfile : $! \n";
    }
    



}

__END__


