#!/usr/bin/perl

# Shell script to upload/download ad dat files and ad logs to/from remote web sites
#
# Notes:
#
#
# Modified:      Kurt Gray -- Tue Oct 24 17:21:07 EDT 2000     First rev, adapted from rp_getremotelogs.pl
#                Kurt Gray -- Fri Oct 27 11:20:49 EDT 2000     Added functionality to upload ad*.dat files
#                Kurt Gray -- Sun Nov  5 13:35:18 EST 2000     Trying to consolidate all ad stuff under adsystem/ directory
#                Kurt Gray -- Mon Nov  6 11:34:24 EST 2000     Moved adsystem directory to /usr/local/adsystem/
#                Kurt Gray -- Mon Nov 20 15:02:44 EST 2000     Added functionality to uploadbannerfiles
#                Kurt Gray -- Fri Feb 16 10:35:50 EST 2001     Added functionality to downloadtrafficlogs
#                Kurt Gray -- Tue Feb 20 11:58:35 EST 2001     Removed GteCatprodFiles (obsolete category-page tracking)


# ad_config_lib contains list of web all sites and configuration options for each.

use Net::FTP;
use Date::Manip;

push @INC, '/usr/local/adsystem/lib/';
require 'ad_config_lib.pl';	        # Site-specific settings for adsystem control scripts


# Read command line options

    foreach $arg (@ARGV) {

	# Option to specify a site

	if ( $arg =~ /site=(\w+)/ ) {
	    $siteid = $1;

	# Debug option shows what's happening without writing new logs
	# and a Ctrl+C keystroke will stop script and dumps stats gathered so far

	} elsif ( $arg =~ /debug/ ) {
	    $debugging = 1;
	    # $dontsavelogs = 1;
	    # $SIG{INT} = dumpStats;

	# Option to get stats for a specific date (date=yesterday is useful for regenerating yesterday's stats)

	} elsif ( $arg =~ /date=(\d{6,8})\-?(\d{0,8})/ ) {
	    $date_input = $1;
	    $date_stop = $2;

        # Number of lines to read in access_log before stopping (useful for debugging)

	} elsif ( $arg =~ /lines=(\d+)/ ) {
	    $lines_max = $1;

	# Specify a base directory web sites are listed under

	} elsif ( $arg =~ /sitesdir=([\w\/]+)/ ) {
	    $sitesdir = $1;

	# Option to start reading access_log at this line (debugging option, not implemented)

	} elsif ( $arg =~ /linestart=(\d+)/ ) {
	    $line_start = $1;

        # Option to run script wihtout saving logs (useful when debugging)

	} elsif ( $arg =~ /dontsavelogs/ ) {
	    $dontsavelogs = 1;

	# Option to write number_of_hits per IP column in visitors_(date).log

	} elsif ( $arg =~ /positions=([\d,]+)/ ) {
	    @positions_list_input = split(/,/, $1);

        # Download catprod.dat files from older Andover.Net sites: dc000en, mb000en, and sh000en

	} elsif ( $arg =~ /dontgetcatprod/ ) {
	    $getcatprodfiles = 'NO';

	# Print summary stats of total FTP and Rsync transactions

	} elsif ( $arg =~ /showsummary/ ) {
	    $showsummary = 1;

	# Option to only download the ad rotation and allocation files for this web site

	} elsif ( $arg =~ /downloaddatfiles/ ) {
	    $op = 'downloaddatfiles';

	# Option to only download traffic summary logs from pagecount server

	} elsif ( $arg =~ /downloadtrafficlogs/ ) {
	    $op = 'downloadtrafficlogs';

	# Option to only upload the ad rotation and allocation files for this web site

	} elsif ( $arg =~ /uploaddatfiles/ ) {
	    $op = 'uploaddatfiles';

	# Option to only upload the ad banner image files for this web site

	} elsif ( $arg =~ /uploadbannerfiles/ ) {
	    $op = 'uploadbannerfiles';

	# Option to only upload the ad banner HTML layer files for this web site

	} elsif ( $arg =~ /uploadhtmladfiles/ ) {
	    $op = 'uploadhtmladfiles';

	# Option to only upload the ad rotation and allocation files for this web site

	} elsif ( $arg =~ /datfilepath=([\w\/\.\-]+)/ ) {
	    $datfilepath = $1;

	# Option to rsync ad dat files using wildcards

	} elsif ( $arg =~ /wildcards/ ) {
	    $send_using_wildcards = 1;

	# Warn about unparsed inputs
	} else {
	    print "ERROR: $0 does not understand argument: $arg \n";
	    exit;
	} # if arg
    }


# Script default settings:

    $ftp_timeout                    ||=   20;      # How long we'll wait for an ftp connection
    $rsync_timeout                  ||=   20;      # How long we'll wait for an rsync connection
    $getcatprodfiles                ||=    1;      # Whether to download catprod.dat from older Andover.Net sites
    @logdates                         =   ();      # Global list of logdates determines ad logs to download
    @transfer_filenames               =   ();      # Which files are going to be sent
    $number_of_file_transfers_limit ||=   50;      # Reasonable limit on number of logdates to process
    $time_started                     = time;      # Note of elapsed time run this script for debugging

# TODO: Move local path settings into ad_config_lib.pl

    $weblogpath     ||= '/usr/local/apache/sites/report/logs';
    $adlogpath      ||= '/usr/local/adsystem/logs';
    $datfilepath    ||= '/usr/local/adsystem/dat';
    $ad_banner_path ||= '/usr/local/apache/sites/report/htdocs/banner';
    $ad_htmlad_path ||= '/usr/local/apache/sites/report/htdocs/htmlad';
    $adinfofile     ||= "$datfilepath/ad_info.dat";



# Turn off buffering on STDOUT so each print statement displays immediately

    $old_fh = select STDOUT;
    $| = 1;
    select $old_fh;


# Figure out dates of ad logs to grab

    if ( $date_input =~ /(\d{6})$/ ) {

	push @logdates, $1;
	$logdate = $date_input;

	# push range of logdates on logdate list if $date_stop defined

	$numberofdays = 1;
	
	if (( $date_stop =~ /\d{8}/ ) && ( $date_input =~ /\d{8}/ )) {
	     
	    print "DEBUG: Parsing logdate range $date_input - $date_stop ...\n" if $debugging == 1;
	    while ( $logdate ne $date_stop ) {
		$logdate = &Date::Manip::ParseDateString( $logdate );
		$logdate = &Date::Manip::DateCalc( $logdate, "+ 1 days" );
		$logdate = &Date::Manip::UnixDate("$logdate", '%Y%m%d');
		if ( $logdate =~ /^\d{2}(\d{6})/ ) {
		    push @logdates, $1;
		} else {
		    print "ERROR: Date::Manip returned blank or odd date : $logdate\n";
		}
		if ( $numberofdays++ > $number_of_file_transfers_limit ) {
		    print "NOTICE: Number of logdates $numberofdays exceeds ";
		    print "number_of_file_transfers_limit $number_of_file_transfers_limit : ";
		    print "last date retrieved will be $logdate \n";
		    last;
		} 
	    }
	} # if date_stop
	
	# If no logdate input then figure out whether to get today's ad log or yesterday's

    } else {

	$hour = `/bin/date +%H`;                                            # Hour format is hh 
	$hour =~ s/\W+//;

	if ($hour =~ /00|01/) {
	    $logdate = &Date::Manip::UnixDate("yesterday", '%y%m%d');       # Date format is yyMMDD
	} else {
	    $todaydate = &Date::Manip::UnixDate("today", '%y%m%d');         # Date format is yyMMDD
	    $logdate = $todaydate;
	}
	push @logdates, $logdate;
	
    } # if logdate


    print "\nDEBUG: \@logdates list set to ", @logdates, " = $numberofdays logdates \n" if $debugging == 1;
    $numberofdays = @logdates;


    
# Get web traffic logs only?

    if ( $op eq 'downloadtrafficlogs' ) {
	# &getWebTrafficLogsFTP;
	&getWebTrafficLogsRsync;
	$log_messages .= qq|$numberofsites logdirs\t$successful_transfers transfer sessions\t|;
	&logMessages;
	exit;
    }



# Clean out previous ads_logdate.temp files so we don't accidently double-append stats
#TODO: Wouldn't it be safer to name ad temp file ads_logdate.timestamp.temp???

     &deleteAdTempFiles;   

  
# If user specified a siteid, then transfer ad files just for this web site

    if ( $siteid ) {

	&readSiteConfig;
	print "\nDEBUG: SINGLE SITE MODE: Looking at $siteid ($sitename) ...\n" if $debugging == 1;
	&transferFiles;

    } else {

        # Make a global list of all siteid's

	foreach $logname ( sort keys %logname2siteid ) {
	    push @allsiteids, $logname2siteid{$logname};
	}

	foreach $siteid ( sort @allsiteids ) {

	    next if $siteid !~ /\w\w000en/;
	    &readSiteConfig;
	    print "\nDEBUG: ALL SITES MODE: Looking at $sitename ($siteid) ...\n" if $debugging == 1;

	    if ( $ad_status ne 'ACTIVE' ) {
		if ( $ad_status ne 'LogsOnly' ) {
		    print "DEBUG: Not bothering with $siteid : ad_status = '$ad_status' \n" if $debugging == 1;
		    next;
		}
	    }

	    print "DEBUG: $siteid is ACTIVE: Looking at transfer settings for $siteid... \n" if $debugging == 1;

	    if ( &transferFiles ) {
		&appendAdTempFile;   # foreach logdate: append siteid/ads_logdate.log >> ads_logdate.temp
	    }

	} # foreach siteid
	
	&moveAdTempFile;    # foreach logdate: move ads_logdate.temp ==> ads_logdate.log

    } # foreach siteid


    if (( $op eq 'uploadbannerfiles' ) || ( $op eq 'uploadhtmladfiles' )) {
	if ( $number_of_new_files > 0 ) {
	    $log_messages = qq|$numberofsites ad servers\t$successful_transfers upload sessions\t$number_of_new_files new ads|;
	} else {
	    $log_messages = "No new ads to upload\t\t";
	}
    } else {
	$log_messages .= qq|$numberofsites ad servers\t$successful_transfers transfer sessions\t|;
    }
    &logMessages;


# Done

     exit;





# Any special ad system behavious with this web site comes from ad_config_lib.pl

sub readSiteConfig {

    $sitename            = $sitesconfig{ $siteid . 'sitename'            };
    $ad_status           = $sitesconfig{ $siteid . 'ad_status'           };
    $ad_allocation_group = $sitesconfig{ $siteid . 'ad_allocation_group' };
    $ad_info_upload      = $sitesconfig{ $siteid . 'ad_info_upload'      };
    $ad_banner_upload    = $sitesconfig{ $siteid . 'ad_banner_upload'    };
    $ad_htmlad_upload    = $sitesconfig{ $siteid . 'ad_htmlad_upload'    };
    $ad_htmlad_set       = $sitesconfig{ $siteid . 'ad_htmlad_set'       };
    $ad_log_download     = $sitesconfig{ $siteid . 'ad_log_download'     };

    if ( @positions_list_input ) {
	@positions_list = @positions_list_input;
    } else {
	@positions_list = split(/,/, $sitesconfig{$siteid.'ad_positions'});
    }

    print "WARNING: siteid '$siteid' does not have a sitename in \%sitesconfig. \n" if !$sitename;

    if ( $debugging == 2 ) {
	print "\n";
	print "DEBUG: \%sitesconfig $siteid ad_status             = '$ad_status' \n";
	print "DEBUG: \%sitesconfig $siteid ad_allocation_group   = '$ad_allocation_group' \n";
	print "DEBUG: \%sitesconfig $siteid ad_info_upload        = '$ad_info_upload' \n";
	print "DEBUG: \%sitesconfig $siteid ad_banner_upload      = '$ad_banner_upload' \n";
	print "DEBUG: \%sitesconfig $siteid ad_htmlad_upload      = '$ad_htmlad_upload' \n";
	print "DEBUG: \%sitesconfig $siteid ad_htmlad_set         = '$ad_htmlad_set' \n";
	print "DEBUG: \%sitesconfig $siteid ad_log_download       = '$ad_log_download' \n";
	print "DEBUG: \%sitesconfig $siteid \@positions_list       = '", @positions_list, "' \n";
	print "\n";
    }

} # end readSiteConfig






# Use FTP or SSH to download the desired files

sub transferFiles {

       %transfer_filenames    = ();       # Global array of remote => local filenames to send
       $transfer_protocol     = '';
       $transfer_host         = '';
       $transfer_port         = '';
       $transfer_userpass     = '';
       $transfer_remotedir    = '';
       $filename_excludes     = '';
    my $ad_position           = '';


    if ( $op eq 'uploaddatfiles' ) {

	$transfer_direction  = 'upload';
	@logdates            = ();   # Don't append process ad log files

	if ( &parseTransferSettings($ad_info_upload) < 1 ) {
	    print "DEBUG: transferFiles : ignoring $siteid : transfer_settings string for siteid is blank. \n" if $debugging == 1;
	    return 0;
	}

	# The following sites still have separate cgi-bin dat directories or use FTP:

	if ( $transfer_protocol eq 'FTP' ) {

	    foreach $ad_position ( @positions_list ) {
		$transfer_filenames{ "ad\_$ad_position\_alloc.dat"  }  =  "$datfilepath/ad\_$ad_position\_alloc.dat\.$siteid";
		$transfer_filenames{ "ad\_$ad_position\_rotate.dat" }  =  "$datfilepath/ad\_$ad_position\_rotate.dat\.$siteid";
	    }

	    $transfer_filenames{ "ad_reservations_alloc.dat"        }  =  "$datfilepath/ad_reservations_alloc.dat\.$siteid";
	    $transfer_filenames{ "ad_reservations.dat"              }  =  "$datfilepath/ad_reservations.dat\.$siteid";

	    if ( $sitesconfig{$siteid . 'ad_info_lite'} ) {
		$tmp_adinfo2 = $sitesconfig{$siteid . 'ad_info_lite'};
		$transfer_filenames{  "$tmp_adinfo2"                }  =  "$datfilepath/$tmp_adinfo2\.$siteid";
	    } else {
		$transfer_filenames{ "ad_info2.dat"                 }  =  "$datfilepath/ad_info2.dat\.$siteid";
	    }

	    $transfer_filenames{ "ad_info.dat"                      }  =  "$datfilepath/ad_info.dat";

	} else {

	    $transfer_filenames{  'ad_*'  } = "$datfilepath/ad_\*";

	} # if siteid



    # Upload ad banner files (GIF images)

    } elsif ( $op eq 'uploadbannerfiles' ) {

	$transfer_direction  = 'upload';
	$getcatprodfiles     = 0;    # Don't bother downloading catprod files
	@logdates            = ();   # Don't append process ad log files

	if ( &parseTransferSettings($ad_banner_upload) < 1 ) {
	    print "DEBUG: transferFiles : ignoring $siteid : transfer_settings string for siteid is blank. \n" if $debugging == 1;
	    return 0;
	}

# TODO: Don't call listNewFiles for every site. Call it once.

	if ( &listNewFiles( $ad_banner_path ) < 1 ) {
	    print "DEBUG: Not bothering uploading banner image files to $siteid : No new files in $ad_banner_path \n" if $debugging == 1;
	    return 0;
	}

	if ( $transfer_protocol eq 'FTP' ) {
	    %transfer_filenames = %new_files;
	} else {
	    $transfer_filenames{  '*.gif'  }  = "$ad_banner_path/\*.gif";
	} # if FTP


    # Upload ad banner files (HTML ad layers)

    } elsif ( $op eq 'uploadhtmladfiles' ) {

	$transfer_direction  = 'upload';
	$getcatprodfiles     = 0;    # Don't bother downloading catprod files
	@logdates            = ();   # Don't append process ad log files

	if ( &parseTransferSettings($ad_htmlad_upload) < 1 ) {
	    print "DEBUG: transferFiles : ignoring $siteid : transfer_settings string for siteid is blank. \n" if $debugging == 1;
	    return 0;
	}

	$tmp_htmlad_path = $ad_htmlad_path;
	if ( $ad_htmlad_set ) {
	    $tmp_htmlad_path  = "$ad_htmlad_path/../$ad_htmlad_set";
	}

# TODO: Don't call listNewFiles for every site. Call it once.

	if ( &listNewFiles( $tmp_htmlad_path ) < 1 ) {
	    print "DEBUG: Not bothering uploading HTML ad files to $siteid : No new files in $tmp_htmlad_path \n" if $debugging == 1;
	    return 0;
	}

	if ( $transfer_protocol eq 'FTP' ) {
	    %transfer_filenames = %new_files;
	} else {
	    $transfer_filenames{ '*.*' }  = "$tmp_htmlad_path/\*.*";
	} # if FTP


    # Download ad logs for each logdate

    } else {

	$op = 'downloadlogs';

	if ( &parseTransferSettings($ad_log_download) < 1 ) {
	    print "DEBUG: transferFiles : ignoring $siteid : transfer_settings string for siteid is blank. \n" if $debugging == 1;
	    return 0;
	}

	foreach $logdate ( @logdates ) {
	    $transfer_filenames { "ads\_$logdate.log" } = "$adlogpath/ads\_$logdate.log\.$siteid";
	}
	
    } # if uploaddatfiles



    # Don't bother if no files in transfer list

    $number_of_transfers = ( keys %transfer_filenames );

    if ( $number_of_transfers < 1 ) {
	print "DEBUG: Not bothering connecting to $siteid : No files to send. \n" if $debugging == 1;
	return 0;
    }


    # Count number of ACTIVE web sites with valid download/upload strings

    $numberofsites++;


    # Now use the appropriate protocol to transmit the files

    if ( $transfer_protocol eq 'FTP' ) {
	&transferViaFTP( $transfer_host, $transfer_userpass, $transfer_remotedir );    
    } elsif ( $transfer_protocol eq 'RSYNC' ) {
	&transferViaRsync( $transfer_host, $transfer_port, $transfer_remotedir );
    } else {
	print "ERROR: transferFiles: No handler for the '$transfer_protocol' protocol. Fix the upload/download settings for $siteid\n";
	return 0;
    }

    return 1;

} # sub transferFiles




sub parseTransferSettings {

    $transfer_settings = shift;

    # Parse the upload/download string for this web site

    if ( $transfer_settings =~ /^(FTP):([\w\.]+):([\w\-]+):([\w\-\/\.]+)/ ) {

	$transfer_protocol  = $1;
	$transfer_host      = $2;
	$transfer_userpass  = $3;
	$transfer_remotedir = $4;

	print "DEBUG: parseTransferSettings: $transfer_protocol, $transfer_host, $transfer_userpass, $transfer_remotedir \n" if $debugging == 1;

    } elsif ( $transfer_settings =~ /^(RSYNC):([\w\.\-]+):(\d*):([\w\-\.]+)/ ) {

	$transfer_protocol  = $1;
	$transfer_host      = $2;
	$transfer_port      = $3;
	$transfer_remotedir = $4;

	print "DEBUG: parseTransferSettings: $transfer_protocol, $transfer_host, $transfer_port, $transfer_remotedir \n" if $debugging == 1

    } else {

	print "DEBUG: parseTransferSettings did not parse transfer_settings string for $siteid: '$transfer_settings' \n" if $debugging == 1;
	return 0;

    } # if transfer_settings

    return 1;

} # sub parseTransferSettings





# Use FTP to upload or download ad_remote_files from remote site
#
# transferViaFTP arguments are ( siteid, host_address, user-pass, remote_dir ),
#                                (global) siteid : determines the local source directory
#                                (global) trnasfer_filenames : list of files to transmit (base filenames) 
#                                (global) transfer_direction : flag to upload or download

sub transferViaFTP {

    my ( $host_address, $remote_dope, $remote_dir ) = @_;
    my ($user,$pass) = split '1', $remote_dope;
    $user = &hop($user); $pass = &hop($pass);	
    
    my $local_filename  = '';
    my $remote_filename = '';
    my $response        = '';

    # Create a new FTP connection and do your business

    print "\nDEBUG: transferViaFTP $siteid : FTP connecting to $host_address [$user] ... \n" if $debugging == 1;

    $ftp = Net::FTP->new($host_address, Timeout => $ftp_timeout);    
    if ($ftp) {
	
	$ftp->login($user,$pass);
	$response = $ftp->message;
	$ftp->cwd($remote_dir);
        if ( $ftp->code ne '250' ) {
	    print "ERROR: transferViaFTP $siteid : FTP CWD error : " . $ftp->code . " : " . $ftp->message . "\n";
	    $ftp->quit;
	    $transfer_failures++;
	    return;
	}
	$response = $ftp->message . "PWD: " . $ftp->pwd . "\n";
	print "DEBUG: FTP $host_address> $response" if $debugging == 1;

	foreach $remote_filename ( sort keys %transfer_filenames ) {

	    $local_filename = $transfer_filenames{$remote_filename};

	    # Can't use filename wildcards with Net::FTP
	    if (( $remote_filename =~ /\*/ ) || ( $local_filename =~ /\*/ )) {
		print "ERROR: transferViaFTP can not send $remote_filename : ";
		print "Apparently Net\::FTP does not support mput or mget. Damn. \n";
		next;
	    }

	    if ( $dontsavelogs == 1 ) {

		print "NOTICE: Not actually sending $siteid:$remote_filename : dontsavelogs == 1 \n";

	    } else {

		if ( $transfer_direction eq 'upload' ) {
		    $ftp->put( "$local_filename", "$remote_filename" );
		} else {
		    $ftp->get( "$remote_filename", "$local_filename" );
		}

		if ( $ftp->code ne '226' ) {
		    print "ERROR: transferViaFTP $siteid : FTP GET error : " . $ftp->code . " : " . $ftp->message . "\n";
		    $transfer_failures++;
		    # Don't close connection on this error; we may have more files in transfer list.
		}

		$response = $ftp->code . ': ' . $ftp->message;
		print "DEBUG: FTP $host_address>  $response" if $debugging == 1;
		$successful_transfers++;

	    } # if dontsavelogs

	} # foreach filename

        # Close FTP site

	$ftp->quit;
	print "DEBUG: FTP $host_address> " . $ftp->message if $debugging == 1;
	
    } else {
	print "ERROR: transferViaFTP $siteid could not FTP connect to $host_address\:$user within $ftp_timeout seconds.\n";
	$transfer_failures++;
    }

    return;

}  # sub transferViaFTP



# Use rsync command to upload or download files from remote web site
#
# Notes:
#         - See "man rsync" for other rsync tricks such as listing remote filegroup:
#         - Interesting rsync command options:
#                          --size-only    =  Only use file size differences to compare files   : Wouldn't use for ad*.dat
#                          - u            =  Don't overwrite files with newer timestamp        : OK... if timestamp is preserved on transit
#                          - v/q          =  Verbose/Quiet mode                                :
#                          - n            =  Dry run: show what rsync would do                 : Useful when dontsavelogs == 1
#                          --progress     =  show transfer progress                            : Useful when debugging == 1
#                          - S            =  Deal with sparse file effeciently (send in bulk?) : Better mode for ad*.dat files?
#                          - a            =  Archive mode: Use recursion and reserve permisions: Useful default mode, I suppose
#                          --timeout=TIME =  Set IO timeout in seconds                         : Should test with a dead host
#                          - z            =  Compress data being sent                          : May not matter for our puroses here
#
#         - Read RSYNC_SSH_TUNNEL_HOWTO.txt for why we rsync to localhost:port instead of remote host
#
# transferViaRsync arguments are ( host_address, port, remote_filegroup ),
#                                (global) siteid : determines the local source directory 
#                                (global) transfer_filenames : list of files to transmit (base filenames) 
#                                (global) transfer_direction : flag to upload or download

sub transferViaRsync {

    my ( $host_address, $port, $filegroup ) = @_;
        
    print "DEBUG: transferViaRsync $siteid connecting to $host_address:$port... \n" if $debugging == 1;
    
    my $filename_source      =  '';
    my $filename_destination =  '';
    my $remote_filename      =  '';
    my $local_filename       =  '';
    my $response             =  '';
    my $n_flag               =  '';    # Whether to do dry run or not
    my $u_flag               =  'u';   # Default rsync to only transmit if source file is newer
    my $portspec             =  '';

    $portspec = "--port=$1" if $port =~ /(\d+)/;


    foreach $remote_filename ( sort keys %transfer_filenames ) {
	
	$local_filename = $transfer_filenames{$remote_filename};

	if ( $transfer_direction eq 'upload' ) {
	    $u_flag                     = 'u';     # Only upload if source files are newer than remote
	    $filename_source            = $local_filename;
	    $remote_filename = '' if $local_filename =~ /\*/;   # For wildcards destination can only be a directory
	    $filename_destination       = "$host_address\::$filegroup/$remote_filename";
	} else {
	    $u_flag                     = '';      # Always download despite who has the newer file     
	    $filename_source            = "$host_address\::$filegroup/$remote_filename";
	    $local_filename = '.' if $remote_filename =~ /\*/;   # For wildcards destination can only be a directory
	    $filename_destination       = $local_filename;
	}

	if ( $dontsavelogs == 1 ) {
	    print "NOTICE: rsync $siteid running in dry mode : dontsavelogs == 1 \n";
	    $n_flag = 'n';
	} 

	if ( $debugging == 1 ) {
	    print "DEBUG: Shell executing 'rsync --timeout=$rsync_timeout $portspec -avz$u_flag$n_flag $filename_excludes ";
	    print "$filename_source $filename_destination' ... \n";
	}

	$response = qx| rsync --timeout=$rsync_timeout $portspec -avz$u_flag$n_flag $filename_excludes $filename_source $filename_destination |;

	print "DEBUG: rsync output:\n $response \n" if $debugging == 1;


	# Parse any errors from rsync command

	if ( $response =~ /ERROR/ ) {
	    print "ERROR: rysnc command $siteid unhappy : $response \n";
	    $transfer_failures++;
	} else {
	    $successful_transfers++;
	}

    } # foreach filename

    return;

}  # sub transferViaRsync



# listNewFiles: Given a path argument, list full path and base filenames of 
#               files newer than a given number of days
#
#               Arguments:  path to find new files in
#
#               Returns:    $number_of_new_files (local)
#                           %new_files (global)

sub listNewFiles {

    my $tmp_path            = shift;
    my $day_limit           = 1;
    my $tmp_filename        = '';
    my $tmp_filepath        = '';
    my $tmp_modtime         = 0;
       $number_of_new_files = 0;
       %new_files           = ();

    if ( opendir DIR, "$tmp_path" ) {
	print "DEBUG: Looking for $tmp_path files newer than $day_limit days...\n" if $debugging == 1;
	@tmp_filenames = grep /\w\.\w/, (readdir DIR);
	closedir DIR;
	foreach $tmp_filename ( @tmp_filenames ) {
	    $tmp_filepath = "$tmp_path/$tmp_filename";
	    $tmp_modtime = ( -M "$tmp_filepath" );
	    if ( $tmp_modtime <= 0 ) {
		print "WARNING: listNewFiles: Could not get modtime for file $tmp_filepath : skipping it. \n";
		next;
	    }
	    # print "DEBUG: Checking $tmp_filepath mod date: $tmp_modtime > 2 \n";
	    next if ( $tmp_modtime > $day_limit );
	    print "DEBUG: listNewFiles: $tmp_filepath is considered new: $tmp_modtime <= $day_limit \n" if $debugging == 1;
	    $new_files{ $tmp_filename } = $tmp_filepath;
	}
	$number_of_new_files = ( keys %new_files );
    } else {
	print "ERROR: Can not opendir $tmp_path to look for new files : $! \n";
    }

    return $number_of_new_files;

} # sub listNewFiles






# Remove the previous stale ads_(logdate).temp file if there is one (there normally wouldn't be any, but just in case)
# TODO: May as well delete all ads_*.temp files in local logdir.

sub deleteAdTempFiles {

    foreach $logdate ( @logdates ) {
	$adstempfilename = "$adlogpath/ads\_$logdate.temp";
	if ( -f "$adstempfilename" ) {
	    if ( unlink "$adstempfilename" ) {
		print "DEBUG: Deleted previous $adstempfilename file.\n" if $debugging == 1;
	    } else {
		print "ERROR: $0 quit : Could not delete previous $adstempfilename file : $!\n";
		exit;  # Better quit here to avoid double-inserting ad stats 
	    }
	} else {
	    print "DEBUG: No previous $adstempfilename file found.\n" if $debugging == 1;
	}
    } # foreach logdate, delete stale tempfiles

} # deleteAdTempFiles




# Append $siteid/ads_(logdate).log log data to $adstempfile
# If this script is not collecting ad logs then the @logdates was probably nulled to avoid this step.

sub appendAdTempFile {

    foreach $logdate ( @logdates ) {
	$adstempfilename = "$adlogpath/ads\_$logdate.temp";
	if ( -f "$adlogpath/ads\_$logdate.log\.$siteid" ) {
	    if ( open INFILE, "< $adlogpath/ads\_$logdate.log\.$siteid" ) {
		if ( open OUTFILE, ">> $adstempfilename" ) {
		    while (<INFILE>) {
			print OUTFILE;
		    }
		    close OUTFILE;
		    print "DEBUG: $adlogpath/ads\_$logdate.log\.$siteid appended to $adstempfilename \n" if $debugging == 1;
		} else {
		    print "ERROR: Could not append to $adstempfilename : $! \n"; 
		}
		close INFILE;
	    } else {
		print "ERROR: Could not append to $adstempfilename : $! \n"; 
	    }
	} else {
	    # The following notice is a DEBUG because some sites may be ACTIVE but not have its own ad log file
	    print "DEBUG: Did not append $siteid ad data to $adstempfilename : log file does not exist \n" if $debugging == 1;
	}
    } # foreach logdate, append to adlog tempfile

} # sub appendAdTempFile



# Move the consolidated $adstempfile to adlogs/ads_(logdate).log
# If this script is not collecting ad logs then the @logdates was probably nulled to avoid this step.

sub moveAdTempFile {

    foreach $logdate ( @logdates ) {
	$adstempfilename = "$adlogpath/ads\_$logdate.temp";
	if ( -f "$adstempfilename" ) {
	    if ( $dontsavelogs == 1 ) {
		print "NOTICE: Not moving $adstempfilename ==> $adlogpath/ads\_$logdate.log : dontsavelogs == 1 \n";
	    } else {
		if ( rename "$adstempfilename", "$adlogpath/ads\_$logdate.log" ) {
		    print "DEBUG: Moved $adstempfilename ==> $adlogpath/ads\_$logdate.log\n" if $debugging == 1;
		} else {
		    print "ERROR: Could not rename $adstempfilename ==> $adlogpath/ads\_$logdate.log : $! \n";
		}
	    }
	} else {
	    print "ERROR: Did not move $adstempfilename ==> $adlogpath/ads\_$logdate.log : file did not exist.\n";
	}
    } # foreach logdate, move logfile

} # sub moveAdTempFile






# sub getWebTrafficLogsRsync: Downloads and parses web traffic logs from page count web server via Rsync
#
# Inputs:   @trafficlogs_remote_logdirs (global) list of log subdirectories to download traffic logs from
#           $trafficlogs_host (global) network address or hostname of the traffic logs RSYNC host
#           $trafficlogs_module (global) RSYNC module name of the traffic logs directory on remote host
#
# Outputs:  Writes downloaded and parsed traffic logs in $weblogpath
#

sub getWebTrafficLogsRsync {

    $numberofsites = @trafficlogs_remote_logdirs;

    my $summary_tmp  = '/tmp/summary.temp';   # Temporary download files
    my $pages_tmp    = '/tmp/pages.temp';


    foreach $logdate ( @logdates ) {

	%tmp_pagespersiteid    =  ();
	$tmp_visitors_logdata  =  '';
	$logdate_newformat     =  &logdate2SysDate( $logdate );
	
	foreach $logdir ( @trafficlogs_remote_logdirs ) {
	    
	    $source_file = "$trafficlogs_host\:\:$trafficlogs_module/$logdir/logs/summary_" . $logdate_newformat . '.log';
	    $dest_file   = $summary_tmp;
	    `/bin/cp /dev/null $dest_file`;

	    if ( $debugging == 1 ) {
		print "DEBUG: Shell executing 'rsync --timeout=20 -avz$u_flag$n_flag $source_file $dest_file' ";
		$n_flag = 'n' if $dontsavelogs == 1;
	    }

	    $response = qx| rsync --timeout=20 -avz$u_flag$n_flag $source_file $dest_file |;
	    
	    print "DEBUG: rsync output:\n $response \n" if $debugging == 1;


	    # Parse any errors from rsync command

	    if (( $response =~ /ERROR/ ) || ( $response =~ /No such file/ )) {

		print "ERROR: rysnc command unhappy : $response \n";
		$transfer_failures++;

	    } else {

		$successful_transfers++;

		# Convert the traffic summary data to common log format:
		
		if ( open( INFILE, "< $summary_tmp" ) ) {
		    while (<INFILE>) {
			chomp;
			next if $_ !~ /^\w+/;
			($tmp_sitename, $tmp_pages, $tmp_visitors) = split("\t", $_);
			$tmp_siteid = $logname2siteid{$tmp_sitename} || $tmp_sitename;
			$logline = join ',', ( $logdate,
					       $tmp_visitors,
					       $tmp_pages,
					       "$tmp_siteid\n" );
			print "DEBUG: Adding $logline to tmp_visitors_logdata\n" if $debugging == 1;
			$tmp_visitors_logdata .= $logline;

		    }
		    close INFILE;
		    
		} else {
		    print "ERROR: getWebTrafficLogs: Could not read traffic summary file $summary_tmp : $! \n";
		}
		
	    } # if ERROR
	    

	    # Get pages_19990825.log for this group of sites:
	    	    
	    $source_file = "$trafficlogs_host\:\:$trafficlogs_module/$logdir/logs/pages_" . $logdate_newformat . '.log';
	    $dest_file   = $pages_tmp;
	    `/bin/cp /dev/null $dest_file`;

	    if ( $debugging == 1 ) {
		print "DEBUG: Shell executing 'rsync --timeout=20 -avz$u_flag$n_flag $source_file $dest_file' ";
		$n_flag = 'n' if $dontsavelogs == 1;
	    }

	    $response = qx| rsync --timeout=20 -avz$u_flag$n_flag $source_file $dest_file |;
	    
	    print "DEBUG: rsync output:\n $response \n" if $debugging == 1;


	    # Parse any errors from rsync command

	    if (( $response =~ /ERROR/ ) || ( $response =~ /No such file/ )) {

		print "ERROR: rysnc command unhappy : $response \n";
		$transfer_failures++;

	    } else {
		
		$successful_transfers++;
		
		if (open(INFILE, "< $pages_tmp")) {
		    while (<INFILE>) {
			next if $_ !~ /^\w+/;
			($tmp_sitename, $tmp_pagename, $tmp_hits) = split("\t", $_);
			$tmp_siteid = $logname2siteid{$tmp_sitename} || $tmp_sitename;
			$tmp_pagespersiteid{$tmp_siteid} .= "$tmp_siteid\t$tmp_pagename\t$tmp_hits";
		    }
		    close INFILE;
		} else {
		    print "ERROR: getWebTrafficLogs: Could not read page hits log $pages_tmp : $! \n";
		}
		
	    } # if ERROR

	    
	} # foreach $logdir on this logdate
	
	
	# Move the traffic summary log for this logdate into action:
	
	$visitors_logfile = "$weblogpath/visitors/visitors$logdate.log";
	if ( open(OUTFILE, "> $visitors_logfile")) {
	    print OUTFILE $tmp_visitors_logdata;
	    close OUTFILE;
	    print "DEBUG: getWebTrafficLogsRsync: Saved $visitors_logfile\n" if $debugging == 1;
	} else {
	    print "ERROR: getWebTrafficLogsRsync: Could not append converted summary data for $logdate to $visitors_logfile : $!\n";
	}

	
	# Split out the page hits log data for this logdate by each web site:
	
	foreach $tmp_siteid ( keys %tmp_pagespersiteid ) {
	    
	    $pages_logfile = "$weblogpath/$tmp_siteid/pages\_$logdate.log";
	    
	    if (open(OUTFILE, "> $pages_logfile")) {
		print OUTFILE $tmp_pagespersiteid{$tmp_siteid};
		close OUTFILE;
		print "DEBUG: getWebTrafficLogsRsync: Saved $pages_logfile\n" if $debugging == 1;
	    } else {
		print "ERROR: getWebTrafficLogsRsync: Could not write page traffic log for $tmp_siteid, $pages_logfile: $! \n";
	    }
		
	}  # foreach siteid

    }  # foreach $logdate
	    
    return 1;

} # getWebTrafficLogsRsync




# sub getWebTrafficLogsFTP: Downloads and parses web traffic logs from page count web server via FTP
#
# Inputs:   @trafficlogs_remote_logdirs (global) list of log subdirectories to download traffic logs from
#           $trafficlogs_host (global) network address or hostname of the traffic logs FTP host
#           $trafficlogs_user (global) FTP username of the logs host
#           $trafficlogs_pass (global) FTP password of the logs host
#
# Outputs:  Writes downloaded and parsed traffic logs in $weblogpath
#

sub getWebTrafficLogsFTP {

    $numberofsites = @trafficlogs_remote_logdirs;
    
    my $summary_tmp  = '/tmp/summary.temp';   # Temporary download files
    my $pages_tmp    = '/tmp/pages.temp';


    
# Create a new FTP connection and do your business

    print "DEBUG: getWebTrafficLogs: FTP connecting to $trafficlogs_host...\n" if $debugging == 1;
    $ftp = Net::FTP->new( $trafficlogs_host, Timeout => $ftp_timeout );

    if ( $ftp ) {
	
	$ftp->login( $trafficlogs_user, $trafficlogs_pass );
	print "DEBUG: FTP: " . $ftp->code . ' ' . $ftp->message . "\n" if $debugging == 1;

	if ( $ftp->code ne '230' ) {
	    print "ERROR: Could not login to traffic log server: " . $ftp->message . "\n";
	    $transfer_failures++;
	    return 0;
	}

    } else {
	
	print "ERROR: getWebTrafficLogs: FTP could not connect to $trafficlogs_host\:$trafficlogs_user within $ftp_timeout seconds. \n\n";
	$transfer_failures++;
	return 0;
	
    } # if ftp


    foreach $logdate ( @logdates ) {

	%tmp_pagespersiteid    =  ();
	$tmp_visitors_logdata  =  '';
	$logdate_newformat     =  &logdate2SysDate( $logdate );
	
	foreach $logdir ( @trafficlogs_remote_logdirs ) {
	    
	    $source_dir = "/$logdir/logs";
	    # $ftp->cwd( $source_dir );
	    # print "DEBUG: FTP: CWD $source_dir..." . $ftp->message . "\n" if $debugging == 1;
	    

	    # Download traffic summary log for this group of sites for this logdate:
	    
	    $summary_log = "$source_dir/summary_" . $logdate_newformat . '.log';
	    
	    $ftp->get( $summary_log, $summary_tmp );
	    print "DEBUG: getWebTrafficLogs: FTP GET $summary_log... " . $ftp->message . "\n" if $debugging == 1;
	    
	    if ( $ftp->code ne '226' ) {

		print "ERROR: getWebTrafficLogs: GET $summary_log : " . $ftp->code . " : " . $ftp->message . "\n";
		$transfer_failures++;
	    
	    } else {

		$successful_transfers++;
		
		# Convert the traffic summary data to common log format:
		
		if ( open( INFILE, "< $summary_tmp" ) ) {
		    while (<INFILE>) {
			chomp;
			next if $_ !~ /^\w+/;
			($tmp_sitename, $tmp_pages, $tmp_visitors) = split("\t", $_);
			$tmp_siteid = $logname2siteid{$tmp_sitename} || $tmp_sitename;
			$tmp_visitors_logdata .= join ',', ( $logdate,
							     $tmp_visitors,
							     $tmp_pages,
							     "$tmp_siteid\n" );
		    }
		    close INFILE;
		    
		} else {
		    print "ERROR: getWebTrafficLogs: Could not read traffic summary file $summary_tmp : $! \n";
		}
		
	    } # if ftp->code
	    

	    # Get pages_19990825.log for this group of sites:
	    
	    $pages_log = "$source_dir/pages_" . $logdate_newformat . '.log';
	    
	    $ftp->get( $pages_log, $pages_tmp );
	    print "DEBUG: getWebTrafficLogs: FTP GET $pages_log... " . $ftp->message . "\n" if $debugging == 1;
	    
	    if ( $ftp->code ne '226' ) {
		
		print "ERROR: getWebTrafficLogs: GET $pages_log : " . $ftp->code . " : " . $ftp->message . "\n";
		$transfer_failures++;
		
	    } else {
		
		$successful_transfers++;
		
		if (open(INFILE, "< $pages_tmp")) {
		    while (<INFILE>) {
			next if $_ !~ /^\w+/;
			($tmp_sitename, $tmp_pagename, $tmp_hits) = split("\t", $_);
			$tmp_siteid = $logname2siteid{$tmp_sitename} || $tmp_sitename;
			$tmp_pagespersiteid{$tmp_siteid} .= "$tmp_siteid\t$tmp_pagename\t$tmp_hits";
		    }
			close INFILE;
		} else {
		    print "ERROR: getWebTrafficLogs: Could not read page hits log $pages_tmp : $! \n";
		}
		
	    } # if ftp->code

	    
	} # foreach $logdir on this logdate
	
	
	# Move the traffic summary log for this logdate into action:
	
	$visitors_logfile = "$weblogpath/visitors/visitors$logdate.log";
	if ( open(OUTFILE, "> $visitors_logfile")) {
	    print OUTFILE $tmp_visitors_logdata;
	    close OUTFILE;
	    print "DEBUG: getWebTrafficLogs: Saved $visitors_logfile\n" if $debugging == 1;
	} else {
	    print "ERROR: Could not append converted summary data for $logdate to $visitors_logfile : $!\n";
	}

	
	# Split out the page hits log data for this logdate by each web site:
	
	foreach $tmp_siteid ( keys %tmp_pagespersiteid ) {
	    
	    $pages_logfile = "$weblogpath/$tmp_siteid/pages\_$logdate.log";
	    
	    if (open(OUTFILE, "> $pages_logfile")) {
		print OUTFILE $tmp_pagespersiteid{$tmp_siteid};
		close OUTFILE;
		print "DEBUG: getWebTrafficLogs: Saved $pages_logfile\n" if $debugging == 1;
	    } else {
		print "ERROR: getWebTrafficLogs: Could not write page traffic log for $tmp_siteid, $pages_logfile: $! \n";
	    }
		
	}  # foreach siteid

    }  # foreach $logdate


    # Close FTP site

    $ftp->quit;
    print "DEBUG: getWebTrafficLogs: FTP disconnecting from $host... " . $ftp->message . "\n" if $debugging == 1;
	    
    return 1;

} # sub getWebTrafficLogsFTP




sub logdate2SysDate {

    my $logdate = shift;
    if ( $logdate =~ /^(\d{2})(\d{4})$/ ) {
	my ( $yy, $mm_dd ) = ( $1, $2 );
	my $YY = ( $yy >= 70 ) ? '19' : '20';
	return "$YY$yy$mm_dd";
    } else {
	return $logdate;
    }

}  # logdate2SysDate: Converts 991123 to 19991123, and 000224 to 20000224



# sub logMessages: Append $log_messages (global) to ad system status log

sub logMessages {

    $time_elapsed = time - $time_started;
    if (( $debugging == 1 ) || ( $showsummary == 1 )) {
	$numberofsites *= 1;
	$successful_transfers *= 1;
	$transfer_failures *= 1;
	print "\n";
	print "DEBUG: SUMMARY: $numberofsites total active ad servers found\n";
	print "                $successful_transfers successful file transfers in $time_elapsed seconds\n";
	print "                $transfer_failures failed file transfers \n" if $transfer_failures > 0;
	print "\n";
    } # showsummary


    if ( $system_logfile ) {
	my $date = `/bin/date`; chomp $date;
	my $dow = $1 if $date =~ /^(\w{3}) /;
	$system_logfile = "$system_logfile.$dow";
	if ( ( -M "$system_logfile" ) > 3 ) {
	    `/bin/cp /dev/null $system_logfile`;  # Zero out log on date roll over
	}
	if ( open LOGFILE, ">> $system_logfile" ) {
	    print LOGFILE qq|$date\t[$op]\t|;
	    print LOGFILE $log_messages;
	    print LOGFILE qq|\t$transfer_failures transfer ERRORs| if $transfer_failures > 0;
	    print LOGFILE qq|\t$time_elapsed seconds\n|;
	    close LOGFILE;
	}
    } # logfile

} # sub logMessages


__END__




