#!/usr/bin/perl

#
# CGI interface for checking the local ad system logs
#
#
# Modified:  Kurt Gray -- Thu Jan 18 15:08:30 EST 2001     First rev.

push @INC, '/usr/local/adsystem/lib/';
require 'ad_config_lib.pl';	        # Site-specific settings for adsystem control scripts
require 'ad_util_lib.pl';	        # Common functions for adsystem scripts

{

# Script settings and base inputs:

    $thisscript           ||= $ENV{'SCRIPT_NAME'};


# Print top of output web page and check user login

    $dontlogthispage++;

    &loginUser( $super_admin_access );


# Read inputs:

    my $logfile  = $1 if $form->{'in'}->{'logfile'} =~ /(\w+)/;
    my $dow      = $1 if $form->{'in'}->{'dow'} =~ /(\w{3})/;
    my $startat  = $1 if $form->{'in'}->{'startat'} =~ /(\d+)/;
    my $op       = $1 if $form->{'in'}->{'op'} =~ /(\w+)/;





# Get today's day of week and current log file names:

    my @daysofweek = ('Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun');

    my $date      = `/bin/date`; chomp $date;
    my $today_dow = $1 if $date =~ /^(\w{3}) /;

    $dow     ||= $today_dow;
    $logfile ||= 'users_log';

    my $line_groups ||= 15;

    $users_logfile  = "$users_logfile.$dow";
    $system_logfile = "$system_logfile.$dow";

    my %logfiles = ( 'email_log', $email_logfile,
		     'users_log', $users_logfile,
		     'system_log', $system_logfile );


    my $uptime_info = `/usr/bin/uptime` if $op eq 'showuptime';
    $nav_links->{ $super_admin_access }->{'System Uptime'} = qq|$thisscript?op=showuptime|;


# Make log browser navbar:

    $lognav .= qq|<TABLE><TR>|;
    foreach $tmp_logfile ( sort keys %logfiles ) {
	next if $tmp_logfile eq 'email_log';
	$lognav .= qq|<TD ALIGN=RIGHT><B>$tmp_logfile:</B></TD>|;
	foreach $tmp_dow ( @daysofweek ) {
	    $logfile_path = "$adlocalpath/logs/$tmp_logfile.$tmp_dow";  # TODO: Should not assume adlocalpath here
	    next if ( ! ( -f "$logfile_path" ));                   # Don't show link to non-existent log files
	    if (( $tmp_dow eq $dow ) && ( $tmp_logfile eq $logfile )) {
		$lognav .= qq|<TD><B>[<A HREF="$thisscript?logfile=$tmp_logfile&dow=$tmp_dow">$tmp_dow</A>]</B></TD>|;
	    } else {
		$lognav .= qq|<TD><A HREF="$thisscript?logfile=$tmp_logfile&dow=$tmp_dow">$tmp_dow</A></TD>|;
	    }
	} # foreach dow
	$lognav .= qq|<TD WIDTH=30>&nbsp;</TD>\n|;
    } # foreach logfile
    $lognav .= qq|<TD ALIGN=RIGHT><B>email_log:</B></TD><TD><A HREF="$thisscript?logfile=email_log">$today_dow</A></TD>
	    </TR>
	    </TABLE>\n<BR> $uptime_info |;

    $page_contents .= $lognav;





# Read the current log file into a table:

    $logfile_path = $logfiles{ $logfile };
    
    # Read entire file:

    $histogram = {};
    if ( open INFILE, "< $logfile_path" ) {
	while ($line = <INFILE>) {
	    $numoflines++;
	    ( $cell1, $cell2, $cell3, $cell4 ) = split /\t/, $line;
	    $histograms->{ 'column2' }->{ $cell2 }++;
	    $histograms->{ 'column3' }->{ $cell3 }++;
	    $histograms->{ 'column4' }->{ $cell4 }++;
	    $file_data .= $line;
	}
	close INFILE;	
    } else {
	$error_messages .= "ERROR: Can not read log $logfile_path : $!\n";
	&printWebpage;
	exit;
    }


    foreach $column ( sort keys %{$histograms} ) {
	$histographs .= qq|<TABLE CELLPADDING=0 CELLSPACING=0><TH COLSPAN=3>Histogram of $logfile_path $column:</TH>\n|;
 	foreach $cell ( sort { &sortAlphaNumeric( $histograms->{$column}->{$a}, $histograms->{$column}->{$b} ) } keys %{$histograms->{$column}} ) {
	    $hits = $histograms->{$column}->{$cell};
	    $cell_width = $hits * 3;
	    $histographs .= qq|<TR><TD>$cell</TD><TD>$hits</TD><TD>
		<TABLE CELLPADDING=0 CELLSPACING=0 BORDER=1><TR><TD BGCOLOR=6666FF WIDTH=$cell_width>.</TD></TR></TABLE></TD></TR>\n|;
	}
	$histographs .= qq|</TABLE>\n|;
    }


    # Set start at stop region:

    $tail_number = $numoflines - $line_groups;
    $startat   ||= $tail_number;                # Default is tail of file
    if ( $line_groups > 1 ) {
	$startat = $startat * 1;
	$stopat  = $startat + $line_groups;
    }

    # Set previous lines link:

    if ( $stopat > 1 ) {
	if ( $startat > 1 ) {
	    $line_number = $startat - $line_groups;
	    $table_data .= qq|[<A HREF="$thisscript?logfile=$logfile&dow=$dow&startat=1">Head</A>] &nbsp; |;
	    $table_data .= qq|<A HREF="$thisscript?logfile=$logfile&dow=$dow&startat=$line_number">Previous $line_groups lines...</A>\n|;
	}
    }


    # Read selected lines from file_data:

    $line_number = 0;
    foreach $line ( (split /\n/, $file_data) ) {
	$line_number++;
	next if (( $startat > 1 ) && ( $line_number < $startat ));
	$table_data .= $line . "\n";
	last if (( $stopat > 1 ) && ( $line_number >= $stopat ));
    }
    $table_data .= "$date (now)\t";

    # Set next lines link:

    if ( $stopat > 1 ) {
	if ( ( $line_number + 1 ) < $numoflines ) {
	    $table_data .= qq| <A HREF="$thisscript?logfile=$logfile&dow=$dow&startat=$line_number">Next $line_groups lines...</A> |;
	    $table_data .= qq| &nbsp; [<A HREF="$thisscript?logfile=$logfile&dow=$dow&startat=$tail_number">Tail</A>] |;
	}
    }

    $lines_label = qq| - Lines $startat to $stopat out of $numoflines |;
    $table_title = "$logfile_path $lines_label ";
    
    &makeTable( 'HTML' );
    $page_contents .= $table_contents;
    $page_contents .= $histographs;


# Show web page:

    &printWebpage;


}

