#!/usr/bin/perl

# CGI and command line interface to get web site traffic statistics reports
#
# TODO:
#        ADD:    Graphs
#        ADD:    stats by SysDate to breakdown and sort stats by date
#        CHANGE: userLogin redirect URL to "SCRIPT_NAME || ad_mainmenu_url" so any CGI script can do login
#        
#
# Modified:   Kurt Gray -- Wed Jan  3 14:58:24 EST 2001     First rev. adapted from ad_report.pl
#


use Date::Manip;

push @INC, '/usr/local/adsystem/lib/';
require 'ad_config_lib.pl';	# Site-specific settings for adsystem control scripts
require 'ad_util_lib.pl';	# Common functions for adsystem scripts
require 'ad_email_lib.pl';	# Send email functions
require 'ad_graph_lib.pl';

&unbufferSTDOUT;


{

# Read command line options

    foreach $arg (@ARGV) {

	# Option to specify a site

	if ( $arg =~ /site=(\w+)/ ) {
	    $siteid = $1;

	# Debug option shows what's happening without writing new logs
	# and a Ctrl+C keystroke will stop script and dumps stats gathered so far

	} elsif ( $arg =~ /debug/ ) {
	    $debugging = 1;
	    # $dontsavelogs = 1;
	    # $SIG{INT} = dumpStats;

	# Option to get stats for a specific date (date=yesterday is useful for regenerating yesterday's stats)

	} elsif ( $arg =~ /date=(\d{6,8})\-?(\d{0,8})/ ) {
	    $StartDate_in = $1;
	    $StopDate_in  = $2;

	# Print summary stats only

	} elsif ( $arg =~ /showsummary/ ) {
	    $showsummary = 1;


        # Ad campaign to show stats for:

	} elsif ( $arg =~ /adcamp=(\w+)/ ) {
	    $ad_campaign = $1;

        # Ad banner to show stats for:

	} elsif ( $arg =~ /adid=(\w+)/ ) {
	    $ad_banner_id = $1;


        # Table data output format:

	} elsif ( $arg =~ /format=(\w+)/ ) {
	    $output_format = $1;

        # Option to breakout campaign stats by siteid or logdate:

	} elsif ( $arg =~ /breakdownby=(\w+)/ ) {
	    $breakdownby = $1;

	# Warn about unparsed inputs
	} else {
	    print "ERROR: $0 does not understand argument: $arg <BR>\n";
	    exit;
	} # if arg

    } # foreach ARGV



# If we didn't get any command line inputs then read CGI web inputs:

    if ( ( $StartDate_in || $siteid || $debugging ) ) {

	$offline_mode++;
	$output_format ||= 'CSV';   # Print ad report data to STDOUT, comma separated values

    } else {

	$online_mode++;

	# Login user and start printing output web page:

	&loginUser( $partner_access );

	# Adjust navbar links:

	# delete $nav_links->{ $editorial_staff }->{'Web Traffic Stats'};

	# Read CGI inputs:

	$StartDate_in  = $1 if $form->{'in'}->{ 'StartDate'   } =~ /([\d\/\-]+)/;
	$StopDate_in   = $1 if $form->{'in'}->{ 'StopDate'    } =~ /([\d\/\-]+)/;
	$ad_campaign   = $1 if $form->{'in'}->{ 'adcamp'      } =~ /(\w{4})/;
	$ad_banner_id  = $1 if $form->{'in'}->{ 'adbanner'    } =~ /(\w{10})/;
	$siteid        = $1 if $form->{'in'}->{ 'siteid'      } =~ /(\w{7})/;
	$output_format = $1 if $form->{'in'}->{ 'format'      } =~ /(\w{3,6})/;
	$sortby        = $1 if $form->{'in'}->{ 'sortby'      } =~ /(\w+)/;
	$op            = $1 if $form->{'in'}->{ 'op'          } =~ /(\w+)/;
	$emailto       = $1 if $form->{'in'}->{ 'emailto'     } =~ /(\S{5,80})/;
	$breakdownby   = $1 if $form->{'in'}->{ 'breakdownby' } =~ /(\w+)/;
	$stats_group   = $1 if $form->{'in'}->{ 'stats_group' } =~ /([\w\.\- ]+)/;
 
	$daterange_confirmed = $1 if $form->{'in'}->{'dates_confirmed'} =~ /(\w)/;

	# Web site partners only stats for their own web sites:
	if ( $user_access_level == $partner_access ) {
	    $siteid = $1 if $ad_system_users{ $username }->{ 'siteids' } =~ /(\w+)/;
	}


    } # if CGI inputs


# Set script defaults:

    $site_initials        = $1 if $siteid =~ /^(\w{2})\w+/;
    $sitename             = $sitesconfig{ $siteid . 'sitename' };
    $output_format      ||= 'HTML';
    $output_textonly      = 1 if $output_format =~ /SV/ || $offline_mode;
    $ad_campaign        ||= ( substr( $ad_banner_id, 0, 4) || 'All' );
    $sortby             ||= 'pages';
    $breakdownby        ||= 'logdate';
    $breakdownby          = '' if $breakdownby eq '_none'; 
    $op                   = 'emailto' if $emailto;
    $site_content_group ||= $stats_group || 'OSDN';

    # Default date range to past 7 days

    $StopDate_in ||= &Date::Manip::UnixDate("today", '%y%m%d');   # logdate format is yymmdd
    if ( $StartDate_in !~ /\d\d/ ) {
	$tmp_date     = &Date::Manip::ParseDateString( "today" );
	$tmp_date     = &Date::Manip::DateCalc( $tmp_date, "- 6 days" );
	$StartDate_in = &Date::Manip::UnixDate("$tmp_date", '%y%m%d');
    }

    &parseDateInputs;  # Convert StartDate_in and StopDate_in to various formats and setup @logdates list

    if (( $numberofdays > 90 ) && ( $online_mode )) {
	if ( $daterange_confirmed ) {
	    $confirm_daterange = qq|<INPUT TYPE=CHECKBOX NAME="dates_confirmed" VALUE="T" CHECKED> Dates Confirmed|;
	} else {

	    $messages .= qq|WARNING: You're asking for more than 90 days worth of data:
                      $numberofdays days. If you're sure about this then click "Dates Confirmed"
		      then click submit button. You may also want to set Breakdown to "Normal"
		      and Output Format to "Text" for faster results. <BR>\n|;
	    
	    $confirm_daterange = qq|<INPUT TYPE=CHECKBOX NAME="dates_confirmed" VALUE="T"> Dates Confirmed|;

	    @logdates = ();

	}
    } # if too many days


# Setup list of report options to include ad report details links:

    %report_options  = ( 
			 'StartDate', $StartDate_yymmdd,
			 'StopDate', $StopDate_yymmdd,
			 'format', $output_format 
			 );

    $report_options{ 'siteid' }   = $1 if $siteid       =~ /(\w{7})/;
    $report_options{ 'adcamp' }   = $1 if $ad_campaign  =~ /(\w{4})/;
    $report_options{ 'adbanner' } = $1 if $ad_banner_id =~ /(\w{10})/;

    while (($key,$value) = each %report_options ) {
	push @report_settings, "$key\=$value";
    }

    $traffic_report_url .= '?' . ( join '&', @report_settings );



# Setup options form:

    if (( $online_mode ) && ( $output_format ne 'PRINT' )) {

	$page_contents .= qq|<FORM METHOD=POST ACTION="$traffic_report_url">|;
	$page_contents .= qq|Web Site: |;
	&getActiveSitenames;
	%drop_list = reverse %sitenames;
	&makeDropList( "siteid", $siteid );
	$page_contents .= qq| Start Date: <INPUT NAME=StartDate SIZE=8 MAXLENGTH=12 VALUE="$StartDate_nice">
	                      Stop Date: <INPUT NAME=StopDate SIZE=8 MAXLENGTH=12 VALUE="$StopDate_nice"> $confirm_daterange 
	                                 <INPUT TYPE=SUBMIT NAME=GoButton VALUE="Show Report"><BR> Breakdown: |;

	%drop_list = ( '_none',     'Normal',
		       'siteid',    'by Site',
		       'logdate',   'by Date',
		       'weekday',   'by Weekday');
	&makeDropList( "breakdownby", "$breakdownby" );

	$page_contents .= qq| Sort by: |;
	%drop_list = ( 'networkid',   'Network',
		       'siteid',      'Site Name',
		       'pages',       'Page Hits',
		       'visitors',    'Visitors',
		       'ppv',         'PPV',
		       'impressions', 'Impressions',
		       'clicks',      'Clicks',
		       'clickthru',   'ClickThru',
		       'adspp',       'AdsPerPage' );
	&makeDropList( "sortby", "$sortby" );

	$page_contents .= qq| Output As: |;
	%drop_list = ( 'HTML',  'HTML',
		       'PRE',   'Text',
		       'CSV',   'CSV',
		       'TSV',   'TSV',
		       'GRAPH', 'Bar Graph',
		       'PRINT', 'Printable' );
	&makeDropList( "format", "$output_format" );

	$page_contents .= qq| <INPUT TYPE=CHECKBOX NAME="op" VALUE="emailto"> Email this to me </FORM>\n|;

    } # if web options


# Read all log files in given date range:

    $stats_by_siteid  = {};
    $breakdown_stats  = {};
    $graph_data       = {};


    $numberofmondays = int( $numberofdays / 7 ) || 1;

    foreach $logdate ( @logdates ) {

	$tmp_logdate  = &logdate2SysDate( $logdate );
	if ( $breakdownby eq 'weekday' ) { 
	    $tmp_logdate2 = &logdate2SysDateDow( $logdate );
	    $tmp_dow     = "$1 (x$numberofmondays)" if $tmp_logdate2 =~ / (\w{3})$/;
	}

	$trafficlogfile =  $weblogpath . '/visitors/visitors' . $logdate . '.log';

	if ( open ( INFILE, "< $trafficlogfile" ) ) {
	    $messages .= "DEBUG: Reading $trafficlogfile... <BR>\n" if $debugging == 1;
	    while (<INFILE>) {
		chomp;
		($tmp_date, $tmp_visitors, $tmp_pages, $tmp_siteid) = split(',', $_);

		# Ignore stats from sites that are not of the site group we're interested in

		if ( $site_content_group ) {
		    next if $sitesconfig{ $tmp_siteid . 'site_content_group' } ne $site_content_group;
		}

		# If a siteid was specified then ignore stats from other sites:

		if ( $siteid ) {
		    if ( $tmp_siteid ne $siteid ) {
			if ( $sitesconfig{ $tmp_siteid . 'traffic_stats_group' } ne $site_initials ) {
			    next;
			} 
		    }

		# Group stats summary by site according to subsite stats group:
 
		} else {

		    $tmp_subsiteid = $tmp_siteid;
		    if ( $sitesconfig{ $tmp_siteid . 'traffic_stats_group' } =~ /([a-z][a-z])/ ) {
			$tmp_siteid = $1 . '000en';
		    }

		} # if siteid

		# $messages .= "DEBUG: Tallying stats for $tmp_siteid on $logdate... <BR>\n" if $debugging == 1;

		$stats_by_siteid{ $tmp_siteid }->{ 'pages' }     += $tmp_pages;
		$total_pages                                     += $tmp_pages;
		$stats_by_siteid{ $tmp_siteid }->{ 'visitors' }  += $tmp_visitors;
		$total_visitors                                  += $tmp_visitors;

		if ( $output_format eq 'GRAPH' ) {
		    $graph_data{ $tmp_logdate }->{ $sitesconfig{ $tmp_siteid . 'sitename' } || $tmp_siteid } += $tmp_pages;
		}

		if ( $breakdownby ) {
		    $tmp_key3 = $tmp_subsiteid;
		    $tmp_key3 = $tmp_logdate if ( $breakdownby eq 'logdate' );
		    $tmp_key3 = $tmp_dow if ( $breakdownby eq 'weekday' );
		    $breakdown_stats{ $tmp_siteid }->{ $tmp_key3 }->{ 'pages' }    += $tmp_pages;
		    $breakdown_stats{ $tmp_siteid }->{ $tmp_key3 }->{ 'visitors' } += $tmp_visitors;
		}

	    } # while INFILE
	    close INFILE;

	} else {
	    $error_messages .= "WARNING: Could not read traffic stats log for day of $logdate: $trafficlogfile: $! <BR>\n";
	}


	next if ( $output_format eq 'GRAPH' );  # Skip ad logs if doing bar graph


	# Read ad stats for this logdate:

	$adlogfile = $adlogpath . '/ads_' . $logdate . '.log';

	if ( open ( INFILE, "< $adlogfile" ) ) {
	    $messages .= "DEBUG: Reading $adlogfile... <BR>\n" if $debugging == 1;
	    while (<INFILE>) {
		chomp;
		($tmp_adid, $tmp_impressions, $tmp_clicks, $tmp_siteid) = split(/\t/, $_);


		# Ignore stats from sites that are not of the site group we're interested in

		if ( $site_content_group ) {
		    next if $sitesconfig{ $tmp_siteid . 'site_content_group' } ne $site_content_group;
		}

		# If a siteid was specified then ignore stats from other sites:

		if ( $siteid ) {
		    if ( $tmp_siteid ne $siteid ) {
			# TODO: Use stats_group instead of allocation group???
			if ( $sitesconfig{ $tmp_siteid . 'ad_stats_group' } ne $site_initials ) {
			    next;
			}
		    }

		# Group stats summary by site according to subsite stats group:
 
		} else {

		    $tmp_subsiteid = $tmp_siteid;
		    if ( $sitesconfig{ $tmp_siteid . 'ad_stats_group' } =~ /([a-z][a-z])/ ) {
			$tmp_siteid = $1 . '000en';
		    }

		} # if siteid

		$stats_by_siteid{ $tmp_siteid }->{ 'impressions' }     += $tmp_impressions;
		$total_impressions                                     += $tmp_impressions;

		if ( $breakdownby ) {
		    $tmp_key3 = $tmp_subsiteid;
		    $tmp_key3 = $tmp_logdate if ( $breakdownby eq 'logdate' );
		    $tmp_key3 = $tmp_dow if ( $breakdownby eq 'weekday' );
		    $breakdown_stats{ $tmp_siteid }->{ $tmp_key3 }->{ 'impressions' }    += $tmp_impressions;
		}

		$tmp_adtype   = substr( $tmp_adid, 4, 1 );
		if (( $tmp_adtype < 5 ) || ( $tmp_clicks > 1 )) {

		    $stats_by_siteid{ $tmp_siteid }->{ 'clicks' }      += $tmp_clicks;
		    $breakdown_stats{ $tmp_siteid }->{ $tmp_key3 }->{ 'clicks' } += $tmp_clicks;
		    $total_clicks                                      += $tmp_clicks;

		} else {

		    $stats_by_siteid{ $tmp_siteid }->{ 'htmlad_impressions' }     += $tmp_impressions;
		    $breakdown_stats{ $tmp_siteid }->{ $tmp_key3 }->{ 'htmlad_impressions' } += $tmp_impressions;
		    $total_htmlad_impressions                                     += $tmp_impressions;

		} # if adtype

	    } # while INFILE
	    close INFILE;

	} else {
	    $error_messages .= "WARNING: Could not read ad stats for day of $logdate: $adlogfile: $! <BR>\n";
	}


    } # foreach logdate



# Which table are we going to show now?

    %statsby_hash  = %stats_by_siteid;
    $page_contents .= '<PRE>' if $output_textonly && $online_mode;

    # Show graph of page impressions by date:

    if ( $output_format eq 'GRAPH' ) {
	$graph_url = &makeGraph( "Total Page Impressions by Date", 710, 400 );
	if ( $graph_url ) {
	    $page_contents .= qq|<IMG SRC="$graph_url" WIDTH=710 HEIGHT=400 ALT="Bar Graph">|;
	}
    } else {
	&makeTrafficStatsTable( 'Web Site', 'siteid', 'Network', 'networkid' );
    }

    $page_contents .= '</PRE>' if $output_textonly && $online_mode;


# Print output:

    if ( $offline_mode ) {

	print $page_contents;
	print $messages;
	print $error_messages;


# If we're in CGI mode then print output as a fancy web page:

    } else {

	if ( $op eq 'emailto' ) {
	    $from_addr       = $email_admin;
	    $to_addr         = $emailto || $ad_system_users{$username}->{'email'};
	    $email_subject   = "[OSDN Stats] $table_title";
	    $email_message   = qq|This report was sent to you by $ad_system_users{$username}->{'name'} <$ad_system_users{$username}->{'email'}> via the 
OSDN Traffic Stats System at $system_baseurl
The attached file is formatted as a $output_format table.\n|;
	    $attachment_data     = $table_contents;
	    $tmp_output_format   = lc( $output_format );
	    $attachment_filename = "osdn_web_traffic.$tmp_output_format";
	    &SendEmail;
	}

	&printWebpage( $output_format );

    } # if offline




}  # end of main script



# makeTrafficStatsTable: Creates formatted summary stats table of the given stats hash
# 
#       Inputs:    $statsby_label   (local)   Label of left-most table column
#                  $statsby_index   (local)   Key for column sortby link option
#                  %statsby_hash    (global)  Stats hash to be formatted into a table
#                  $output_format   (global)  How to format the table: CSV, TSV, HTML, or PRE
#
#       Outputs:   $page_contents   (global)  Appends table text to page_contents
#

sub makeTrafficStatsTable {

    my $statsby_labelA = shift;
    my $statsby_indexA = shift;
    my $statsby_labelB = shift;
    my $statsby_indexB = shift;


    $table_data        =  '';
    $table_contents    =  '';
    $tmp_table_name    = $sitename || $siteid || 'OSDN Network';
    $table_title       = qq|$tmp_table_name Traffic from $StartDate_ISO to $StopDate_ISO, $numberofdays Days|;

    if ( $output_textonly ) {
	@table_headers   =  ( $statsby_labelA, $breakdownby || $statsby_labelB, 'Pages', 'Visitors', 'PPV', 
			      'Ad Impressions', 'APP', 'Ad Clicks', 'ClickThru' );
    } else {
	@table_headers   =  (
			     qq|<A HREF="$traffic_report_url&sortby=$statsby_indexA">$statsby_labelA</A>|,
			     $breakdownby || qq|<A HREF="$traffic_report_url&sortby=$statsby_indexB">$statsby_labelB</A>|,
			     qq|<A HREF="$traffic_report_url&sortby=pages">Pages</A>|,
			     qq|<A HREF="$traffic_report_url&sortby=visitors">Visitors</A>|,
			     qq|<A HREF="$traffic_report_url&sortby=ppv">PPV</A>|,
			     qq|<A HREF="$traffic_report_url&sortby=impressions">Ad Impressions</A>|,
			     qq|<A HREF="$traffic_report_url&sortby=adspp">APP</A>|,
			     qq|<A HREF="$traffic_report_url&sortby=clicks">Ad Clicks</A>|,
			     qq|<A HREF="$traffic_report_url&sortby=clickthru">ClickThru</A>|,
			     );
    } # if output_format


    # Create a sortable hash of stats-by-site:

    %sorttable_hash = ();

    foreach $tmp_key ( sort keys %statsby_hash ) {

	if ( $statsby_indexA eq 'siteid' ) { 
	    $tmp_nameA             = $sitesconfig{ $tmp_key . 'sitename' } || $tmp_siteid;
	    $tmp_nameB             = $sitesconfig{ $tmp_key . 'site_content_group' };
	} else {
	    $tmp_nameA             = $tmp_key;
	    $tmp_nameB             = $tmp_key;
	}

	$tmp_pages                 = $statsby_hash{ $tmp_key }->{ 'pages' };
	next if ( ( $pages_cutoff_limit > 1 ) && ( $tmp_pages <  $pages_cutoff_limit ) );
	$tmp_visitors              = $statsby_hash{ $tmp_key }->{ 'visitors' } || '0';
	$tmp_ppv                   = $tmp_visitors > 1  ? 
	                             substr( ( $tmp_pages / $tmp_visitors ), 0, 4 ) : 0;

	$tmp_impressions           = $statsby_hash{ $tmp_key }->{ 'impressions' };
	$tmp_htmlad_impressions    = $statsby_hash{ $tmp_key }->{ 'htmlad_impressions' };
	$tmp_clickable_impressions = ( $tmp_impressions - $tmp_htmlad_impressions ) * 1 || 'N/A';
	$tmp_clicks                = $statsby_hash{ $tmp_key }->{ 'clicks' } || '0';
	$tmp_click_thru            = $tmp_clickable_impressions > 1  ? 
	                             substr( ( ( $tmp_clicks / $tmp_clickable_impressions ) * 100 ), 0, 4 ) : 0;
	$tmp_adspp                 = $tmp_pages > 1 ? substr( ( $tmp_impressions / $tmp_pages ), 0, 3 ) : 0;

	$traffic_report_url  =~ s/\&?$statsby_indexA\=$tmp_key//;
	$tmp_report_linkA    = $traffic_report_url . "\&$statsby_indexA\=$tmp_key";


	if ( $sortby eq 'pages' ) {
	    $tmp_key2 = "$tmp_pages$tmp_key";
	} elsif ( $sortby eq 'impressions' ) {
	    $tmp_key2 = "$tmp_impressions$tmp_key";
	} elsif ( $sortby eq 'clicks' ) {
	    $tmp_key2 = "$tmp_clicks$tmp_key";
	} elsif ( $sortby eq 'clickable' ) {
	    $tmp_key2 = "$tmp_clickable_impressions$tmp_key";
	} elsif ( $sortby eq 'clickthru' ) {
	    $tmp_key2 = "$tmp_click_thru$tmp_key";
	} elsif ( $sortby eq 'adspp' ) {
	    $tmp_key2 = "$tmp_adspp$tmp_key";
	} elsif ( $sortby eq 'visitors' ) {
	    $tmp_key2 = "$tmp_visitors$tmp_key";
	} elsif ( $sortby eq 'pagename' ) {
	    $tmp_key2 = "$tmp_pagename$tmp_key";
	} elsif ( $sortby eq 'ppv' ) {
	    $tmp_key2 = "$tmp_ppv$tmp_key";
	} elsif ( $sortby eq $statsby_indexB ) {
	    $tmp_key2 = "$tmp_nameB$tmp_key";
	    $alphasort++;
	} else {
	    $tmp_key2 = "$tmp_nameA$tmp_key";
	    $alphasort++;
	} # if sortby


	# If breakdown campaigns by site the add table rows for each site:

	if ( $breakdownby =~ /\w\w/ ) {
	    foreach $tmp_key3 ( sort keys %{ $breakdown_stats{ $tmp_key } } ) {

		$tmp_pagesB                 = $breakdown_stats{ $tmp_key }->{ $tmp_key3 }->{ 'pages' };
		$tmp_visitorsB              = $breakdown_stats{ $tmp_key }->{ $tmp_key3 }->{ 'visitors' } || '0';
		$tmp_ppvB                   = $tmp_visitorsB > 1  ? 
		                              substr( ( $tmp_pagesB / $tmp_visitorsB ), 0, 4 ) : 0;

		$tmp_impressionsB           = $breakdown_stats{ $tmp_key }->{ $tmp_key3 }->{ 'impressions' };
		$tmp_htmlad_impressionsB    = $breakdown_stats{ $tmp_key }->{ $tmp_key3 }->{ 'htmlad_impressions' };
		$tmp_clickable_impressionsB = ( $tmp_impressionsB - $tmp_htmlad_impressionsB ) * 1 || 'N/A';
		$tmp_clicksB                = $breakdown_stats{ $tmp_key }->{ $tmp_key3 }->{ 'clicks' } || '0';
		$tmp_click_thruB            = $tmp_clickable_impressionsB > 1  ? 
		                              substr( ( ( $tmp_clicksB / $tmp_clickable_impressionsB ) * 100 ), 0, 4 ) : 0;
		$tmp_adsppB                 = $tmp_pagesB > 1 ? substr( ( $tmp_impressionsB / $tmp_pagesB ), 0, 3 ) : 0;

		$sorttable_hash{ $tmp_key2 } .= join "\t", ( $output_textonly ? $tmp_nameA : qq|<A HREF="$tmp_report_linkA">$tmp_nameA</A>|,
						    $sitesconfig{ $tmp_key3 . 'sitename' } || &SysDate2ISODateDow($tmp_key3),
						    $tmp_pagesB,
						    $tmp_visitorsB,
						    $tmp_ppvB,
						    $tmp_impressionsB,
						    $tmp_adsppB,
						    $tmp_clicksB,
						    "$tmp_click_thruB\n" );

		} #foreach siteid
		$breakdown_total_label = $output_textonly ? 'TOTAL' : ' <B>TOTAL:</B>';
		$breakdown_row_spacer  = ( join "\t", @table_headers ) . "\n";
	} else {
	    $breakdown_total_label = '';
	    $breakdown_row_spacer  = '';	    
	} # if table is adcamp

        # Create sortable table array data:

	if ( $output_format !~ /SV/ ) {
	    $sorttable_hash{ $tmp_key2 } .= join "\t", ( 
						     $output_textonly ? $tmp_nameA : qq|<A HREF="$tmp_report_linkA">$tmp_nameA</A>|,
						     $breakdown_total_label ? $breakdown_total_label : 
						     ( $output_textonly ? $tmp_nameB : qq|<A HREF="$tmp_report_linkA">$tmp_nameB</A>|),
						     $tmp_pages,
						     $tmp_visitors,
						     $tmp_ppv,
						     $tmp_impressions,
						     $tmp_adspp,
						     $tmp_clicks,
						     "$tmp_click_thru\n$breakdown_row_spacer" 
						     );
	}


    } # foreach $tmp_siteid


    # Sort stats hash and convert it a table:

    foreach $tmp_key ( sort { &sortAlphaNumeric($a,$b) } keys %sorttable_hash ) {
	$table_data .= $sorttable_hash{ $tmp_key };
    }

    $total_ppv                     = $total_visitors > 1 ? 
                                     substr( ( $total_pages / $total_visitors ), 0, 4 ) : '0';

    $total_clickable_impressions   = ( $total_impressions - $total_htmlad_impressions ) * 1;
    $total_click_thru              = $total_clicks > 1 ? 
	                             substr( ( ( $total_clicks / $total_clickable_impressions ) * 100 ), 0, 4 ) :
		                     $total_clicks;

    $total_adspp                    = $total_pages > 1 ? substr( ( $total_impressions / $total_pages ), 0, 3 ) : 0;

    $table_data .= join "\t", ( ' ',
				$output_textonly ? 'TOTALS:' : qq|<B>TOTALS:</B>|,
				$total_pages,
				$total_visitors,
				$total_ppv,
				$total_impressions,
				$total_adspp,
				$total_clicks,
				"$total_click_thru\n" );

    &makeTable( $output_format );
    $page_contents .= $table_contents;


}  # sub makeTrafficStatsTable




__END__







