use strict;

package FormParse;

##########################################################
sub new {
	my $class = shift;
	my $self = {};

	$self->{maxdata} = 131072; # 2^17 bytes
	return(bless($self,$class));
}

##########################################################
sub Parse {
	my $self = shift;

	my($type,$len);

	$self->{type} = $ENV{'CONTENT_TYPE'};
	$self->{len} = $ENV{'CONTENT_LENGTH'};
	$self->{meth} = $ENV{'REQUEST_METHOD'};

	# die "too much data\n" if($len > $self->{maxdata});

	if (!defined $self->{meth} 
		|| $self->{meth} eq '' 
		|| $self->{meth} eq 'GET' 
		|| $self->{meth} eq 'HEAD' 
		|| $self->{type} eq 'application/x-www-form-urlencoded') 
	{
		$self->ParseNormal();
	} 
	elsif($self->{type} =~  m#^multipart/form-data#) {
		$self->ParseMulti();
	}
}	

##########################################################
# ParseNormal(\$in) (private method)
#	inputs
#	returns
#
##########################################################
sub ParseNormal {
	my $self = shift;
	
	my ($key, $val, $i, $input, @in,$errflag,$got);
	# Read in text
	if (!defined $self->{meth} || $self->{meth} eq '') {
		$input = $ENV{'QUERY_STRING'};
	} 
	elsif($self->{meth} eq 'GET' || $self->{meth} eq 'HEAD') {
		$input = $ENV{'QUERY_STRING'}; 
	} 
	elsif ($self->{meth} eq 'POST') {
		if (($got = read(STDIN, $input, $self->{len}) != $self->{len})) {
			$errflag="FormParse: Short Read: wanted $self->{len}, got $got\n";
		}
	} else {
		die "FormParse: $! Unknown request method: $self->{meth}\n";
	}

	@in = split(/[&;]/,$input);

	for $i(0 .. $#in) {
		# convert plus to space
		$in[$i] =~ s/\+/ /g;
		# split into key and value
		($key, $val) = split(/=/,$in[$i],2); # splits on the first =.
		    
		# Convert %XX from hex numbers to alphanumeric
		$key =~ s/%([A-Fa-f0-9]{2})/pack("c",hex($1))/ge;
		$val =~ s/%([A-Fa-f0-9]{2})/pack("c",hex($1))/ge;
		    
		# Associate key and value
		$self->{in}->{$key} .= "\0" if (defined($self->{in}->{$key})); # \0 is the multiple separator
		$self->{in}->{$key} .= $val;
	}
}

##########################################################
# ParseMulti(\$in) (private method)
#	inputs
#	returns
#
##########################################################
sub ParseMulti {
	my $self = shift;

	my($delim,$inputs,@parts,$key,$filename);

	$delim = $1 if <STDIN> =~ /(\S+)/;
	
	while(<STDIN>) {
		$inputs .= $_;
	}

	# if ( my $debugging = 1 ) {
	#    print "Content-type: text/plain\n\n";
	#    print "DEBUG: CGI INPUT = \n $inputs";
	# }

	(@parts) = split /\x0d\n$delim/, $inputs;

	for(@parts) {
	    if ($_ =~  s/Content-\w+\: \w+\-\w+\; name=\"([^\"].*?)\"\; filename=\"([^\"].*?)\"\x0d\n//) {
		$key = $1;
		$filename = $2;
		s/Content-\w+\: \w+\/\w+\x0d\n\x0d\n(.*?)/$1/;
		s/\n\-+\w+\-\-\x0d$//;
		s/\x0d\n//;
		if ($filename) {
		    $self->{in}->{$key}->{value} = $_;
		    $self->{in}->{$key}->{filename} = $filename;
		    # if ( my $debugging = 1 ) {
		    #	print "DEBUG: FILE INPUT '$filename' = " . length . " bytes \n";
		    # }
		} else {
		    $self->{in}->{$key} = $_;
		}
	    } elsif ($_ =~ s/Content-\w+\: form\-data\; name=\"([^\"].*?)\"\x0d\n\x0d\n//) {
		$key = $1;
		s/\x0d\n//;
		$self->{in}->{$key} = $_;
		# if ( my $debugging = 1 ) {
		#     print "DEBUG: CGI INPUT '$key' = '$_'\n";
		# }
	    }
	}

	# if ( my $debugging = 1 ) {
	#    print "DEBUG: CGI INPUT PARSING DONE. delim = '$delim'";
	#    exit
	# }

} # sub ParseMulti

1;

__END__

=head1 NAME

FormParse - a simple object oriented CGI form input parser

=head1 SYNOPSIS

  FormParse is a simple CGI form parser for reading web page 
  form inputs and can also read data from INPUT TYPE=FILE in
  multipart forms.

  Example:

  use FormParse;
  my $p = new FormParse;
  $p->Parse;
  
  ...then you can access your HTML form inputs

  $catname = $p->{'in'}->{"dog"};
  $dogname = $p->{'in'}->{"cat"};


  If you are using a multi-part form, and one of your values is 
  a file, then it will work as such:

  $filename = $p->{formkey}->{filename}
  $filecontents = $p->{formkey}->{value}


  How to dump all form inputs:

	for (keys(%{$p->{'in'}})) {
		if (ref{$p->{'in'}->{$_})) {
			print "key: $_ 	value: $p->{'in'}->{$_}\n";
		}

	}


=head1 COPYRIGHT

Copyright 1999, Patrick Galbraith 

This library is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.



