
# OSDN email messages and send mail functionality is all here in this lib.
#
# Notes:
#
#	Most of these functions use global variables instead of passing around parameters
#
#
# Modified:  Kurt Gray -- Wed Jan 10 11:07:50 EST 2001      Based on osdn_email_lib.pl, based on server51_email_lib.pl



use Net::SMTP;
use Net::DNS;
use MIME::Base64;


# Confirmation email to new memeber signup

sub sendAccountActivatedEmail {

    $from_addr       = $email_admin;
    $to_addr         = $user_email;
    $email_subject   = "$email_subject_prefix $user_name login info";
    
    $email_message   = qq|Greetings,

An Ad System login account has been created for you. 

$system_baseurl

Your ad system username is: $user_id
Your ad system password is: $user_password

Make sure that cookies are enabled in your browser
otherwise you\'ll be sent back to the login screen.

Do not share this login info with anyone, especially people
outside of our organization. If someone you know needs 
access to this system then tell them to send an email 
to <$email_admin> so that a separate login account 
will be created for them.

The ad system includes our network traffic stats, ad stats,
ad campaign info, current ad rotations for each web site, 
a basic notes system, and all the backend plumbing for 
placing ads and gathering stats. User functionality is 
slightly different depending on your department.

Question, comments, major maladjustments, bugs, and
donations address to: <$email_admin>

...and if it's not in there yet, then I'm working on it.

Kurt

    |;

    
    &SendEmail;

} # sub SendAccountActivatedEmail




#
# Generic send email funtion uses a direct SMTP connection to to_addr MX host
#
# Arguments:  attachment_filename   (global)  Name of file attachment (optional)
#             encoding_type         (local)  Send 'base64' if email_message is binary data
#             attachment_data       (global) Part message to be sent as an attachment
#
# Read input from global variables: to_addr, from_addr, email_subject, email_message
#

sub SendEmail {

    my $encoding_type           = shift;
    my $attachment_separator    = '';
    my $message_mime_header     = '';
    my $message_mime_attachment = '';

    # Options to suppress all email set in server51_db_config.lib

    if ( $suppress_all_email == 1 ) {
	$error_messages .= "DEBUG: SendMail deactivated! $email_subject -> $to_addr not sent <BR>\n";
	return 1;
    }
    
    # Separate the message into MIME multipart if there is attachment data:

    if ( $attachment_data ) {

	if ( $attachment_filename =~ /\.html$/i ) {
	    if ( $attachment_data !~ /<HTML>/i ) {
		$attachment_data = qq|<HTML>
		    <HEAD>
			<TITLE> $table_title </TITLE>
		        <BASE HREF="$system_baseurl">
		    </HEAD>
		    <BODY BGCOLOR=\#FFFFFF>
			$attachment_data
		    </BODY>
		    </HTML>|;
	    }
	    $encoding_type = 'base64';
	}
	
	my $date = `/bin/date`; chomp $date;
	my $time = time;

	$attachment_filename ||= 'adsystem_data.txt';
	$attachment_separator  = "$time.NextPart_000_0028_" . $$;
	$encoding_type       ||= ( $attachment_filename =~ /\.gif|\.doc|\.xls$/i ) ? 'base64' : 'quoted-printable';
	$email_message       ||= qq|See attached file: $attachment_filename\n|;

	if ( $encoding_type eq 'base64' ) {
	    $attachment_data = MIME::Base64::encode_base64( $attachment_data );
	}

        # MIME-Version: 1.0 Headers:

	$attachment_header = qq|
Message-ID: <AdSystem.Jan122001.WHSDGSLDHSLD.$time\@$ENV{'HOSTNAME'}$ENV{'SERVER_NAME'}>
MIME-Version: 1.0
Content-Type: multipart/mixed; boundary="$attachment_separator"
|;

	$message_mime_header = qq|This is a multi-part message in MIME format.

--$attachment_separator
Content-Type: text/plain; charset="us-ascii"
|;

	$message_mime_attachment = qq|
--$attachment_separator
Content-Type: text/plain; charset="us-ascii"; name="$attachment_filename"
Content-Transfer-Encoding: $encoding_type
Content-Disposition: attachment; filename="$attachment_filename"

$attachment_data
--$attachment_separator--|;

    } # if attachment


    # Expand ad system usernames to email addresses:
    if ( $to_addr =~ /,/ ) {
	my @tmp_addresses = ();
	my $tmp_address   = '';
	my $tmp_addressex = '';
	foreach $tmp_address ( (split ',', $to_addr) ) {
	    $tmp_addressex    = '';
	    if ( $tmp_address =~ /^(\w+)$/ ) {
		my $tmp_username  = $1;
		$tmp_addressex    = $ad_system_users{ $tmp_username }->{'email'};	    
		if ( $tmp_addressex !~ /\w+/ ) {
		    $error_messages .= "ERROR: SendEmail email address unknown for user '$tmp_username' <BR>\n";
		    next;	    
		} else {
		    # $error_messages .= "DEBUG: SendEmail tmp_address $tmp_address EXPANDED to $tmp_addressex <BR>\n";
		}
	    } else {
		# $error_messages .= "DEBUG: SendEmail tmp_address $tmp_address is more than a word <BR>\n";
	    }
	    push @tmp_addresses, $tmp_addressex || $tmp_address;
	} # foreach address
	$to_addr = join ',', @tmp_addresses;
    } else {
	# $error_messages .= "DEBUG: SendEmail to_addr $to_addr does not match a comma <BR>\n";
    }

    # Compose the entire email message:

    $entire_message = qq|From: $from_addr
To: $to_addr
Subject: $email_subject$attachment_header
$message_mime_header
$email_message
$message_mime_attachment|;



    foreach $to_addrr ( (split ',', $to_addr) ) {

	if ( $email_allemailto =~ /\w+/ ) {
	    $error_messages .= "DEBUG: SendMail rerouted: all email sent to $email_allemailto instead of $to_addrr <BR>\n";
	    $to_addrr        = $email_allemailto;
	}

	# Parse each email address:
        
	if ( $to_addrr =~ /([\w\-\.]+)\@([\w\-\.]+)/ ) {

	    $my_username = $1;
	    $my_domain   = $2;

	} else {
	    $error_messages .= "ERROR: SendEmail could not parse email address: $to_addrr <BR>\n";
	    next;
	}
    
	# DNS lookup email server for this domain:

	$msg = '';
	@my_servers = &get_mail_server( $my_domain );    
	if ( $msg ) {
	    $error_messages .= "ERROR: SendEmail can not find a mail server for domain $my_domain : $msg <BR>\n";
	    next;
	}

	# Attempt a connection to the mail server. Multiple attempts will be made if needed.
    
	$smtp = &server_connect( @my_servers );
	if (!$smtp) {
	    $error_messages .= "ERROR: SendEmail could not establish an SMTP connection to $my_domain.<BR>\n";
	    next;
	} else {
	    # Send the email
	    $rval = '';
	    $rval = &send_mail( $smtp, $from_addr, $to_addrr );
	    # Disconnect from the SMTP host.
	    $smtp->quit;
	}  # if not error_messages
	
	if ( $rval == 1 ) {
	    $error_messages .= "NOTICE: SendEmail sent \"$email_subject\" --> $to_addrr.<BR>\n";
	} elsif( $rval == 0 ) {
	    $error_messages .= "ERROR: SendEmail got an SMTP error $code: \"$msg\" when trying to mail \"$email_subject\" --> $to_addrr.<BR>\n";
	} else {
	    $error_messages .= "ERROR: SendEmail SMTP connection stalled sending $email_subject to $to_addrr.<BR>\n";
	} # if rval

    } #foreach to_addrr


} # sub SendEmail




###########################################
#        get_mail_server()                # 
###########################################

sub get_mail_server{

    my($domain) = @_;
    my($res,$rr,$exch,$addr,$query);
    my(@mx,@results);

    @results = ();

    $res = new Net::DNS::Resolver;
    @mx = mx($res, $domain);
    
    if(@mx) {

        foreach $rr (@mx) {
            # Found the MX record
            push(@results, $rr->exchange); 
        }

    } else {

        # Look for an A record.
        
        $query = $res->search($domain);
        if ($query) {
            foreach $rr ($query->answer) {
                if($rr->type eq "A"){
                    push(@results,$rr->address);
                }
            }
        }
    }

    $msg = $res->errorstring unless(@results);
    return @results;

} #end get_server_name()


################################################
#               server_connect()               # 
################################################
sub server_connect{    
    my (@remote_array) = @_;
    my ($smtp);
    my ($num_tries) = 2;
    my ($tries) = 0;
    my ($remote) = '';
    
    # Globals

    while($tries < $num_tries) {        
        foreach $remote (@remote_array) {

            # Hack for hotmail
            $num_tries = 10 if($remote =~ /hotmail/); 

            undef $smtp;            
            $smtp = Net::SMTP->new($remote, Hello => 'andover.net', Timeout => 120); # Add Debug => 1 for debugging SMTP
            
            if (defined($smtp)) {               
                # Check it's status.
                unless(($code = $smtp->code) == 250) {
                        $msg = $smtp->message;
                    chop $msg;
                    $smtp->quit;
                    undef $smtp;
                }
                
            } else {

                sleep(30) if($remote =~ /hotmail/);

            }
            
            return $smtp if($smtp); # Got a connection.
            
        } #End foreach
        
        if ((++$tries) < $num_tries) { 
            # Wait a few seconds and try again.
            sleep 10;
        } else {
            undef $smtp;
        }
        
    } # End while

    return $smtp;
    
}

#################################################
#                       send_mail()             #
#################################################

sub send_mail{
    
    my ($smtp,$from_addr,$to_addr) = @_;
    my ($name,$all_mail);
    
    
    # Globals
    undef $msg;
    undef $code;
    $phase = 'NONE';

######################
# MAIL FROM: command.#
######################
    
    $smtp->mail($from_addr);
    
    $code = $smtp->code;
    
    if($code == 503){
        
        # 503 means server doesn't want a new name.  It thinks it already knows what our FROM ADDR is.
        # The problem here is that we stick different subscriber IDs in this field so we need to be able
        # to specify a unique FROM ADDR with every newsletter. 
        # Try a reset. If it works, great.  If not we'll have to disconnect.
        
        $smtp->reset;
        
        $code = $smtp->code;
        unless($code == 250){
            # We need to reconnect and -1 will cause this one will get retried.
            return -1;
        }

        # Try to give it the FROM ADDR again now.
        $smtp->mail($from_addr);
        
        $code = $smtp->code;
    }
    
    unless($code == 250){               
        
        if($code == 503){
            # We need to reconnect and -1 will cause this one will get retried.
            return -1;
        }else{      
            $msg = $smtp->message;
            chop $msg;
            $phase = 'FROM';
            return 0;
        }
    }
    
####################
# RCPT TO: command.#
####################    
    $smtp->to($to_addr);
    
# Check server reply.
    
    $code = $smtp->code;        
    if($code == 503){
        # SMTP server needs the MAIL FROM: again
        
        $smtp->mail($from_addr);
        
        $code = $smtp->code;
        
        unless($code == 250){
            $msg = $smtp->message;
            chop $msg;
            $phase = 'RCPT';
            return 0;
        }
        
        # Try the RCPT TO: again.
        
        $smtp->to($to_addr);
        $code = $smtp->code;
        
    }
    
    unless(($code == 250) || ($code == 251)){
        $msg = $smtp->message;
        chop $msg;
        $phase = 'RCPT';
        return 0;
    }
    

################
# DATA command.#
################
    $smtp->data();
    
# Check server reply.
    
    $code = $smtp->code;
        unless($code == 354){
            $msg = $smtp->message;              
            chop $msg;
            $phase = 'DATA';
            return 0;
        }
    
###################
# Send the data   #
###################
    
    $smtp->datasend($entire_message);
    
######################################
# Finish transaction. (sends \n.\n). #
######################################
    $smtp->dataend();
    
# Check server reply.
    
    if(($code = $smtp->code) >= 400){
        $msg = $smtp->message;
        chop $msg;
        $phase = 'DEND';                
        return 0;
    }
    
    return 1;
}


1;
