
#
# Ad system CGI interface common utility functions
#
# Modified:   Kurt Gray -- Thu Nov 30 14:27:57 EST 2000     First rev.
#             Kurt Gray -- Sun Dec  3 16:04:37 EST 2000     Added pageheader and pagefooter


use FormParse;
use POSIX qw(strftime);  # used to expire adsys (adsystem password) cookie


# User access levels to restrict who can access each feature of the ad system CGI interface:

    $anonymous_access   =  10;    # Reserved for anoymous guest access (not able to access anything at this time) 
    $advertiser_access  =  20;    # Adverising clients access to view ad reports for their own capaigns
    $advertisers_access =  $advertiser_access;
    $partner_access     =  23;    # Site partners can get their own web site traffic stats but not other sites
    $auditor_access     =  25;    # Auditors have access to web traffic stats for all sites, but not ad stats
    $auditors_access    =  $auditor_access;
    $editorial_staff    =  30;    # Webmasters of the various web sites able view ad allocations and stats for their own sites
    $editorial_access   =  $editorial_staff;
    $accountants_access =  40;    # Accountants able view ad allocations and stats for all sites
    $accountant_access  =  $accountants_access;
    $director_access    =  50;    # Directors able to view stats, allocations, and ads for all sites
    $sales_dept         =  60;    # In-house ad sales people able to view ad allocations, stats, campaigns, and make notes
    $sales_access       =  $sales_dept;
    $sales_manager      =  70;    # In-house ad sales dept heads able to view ad allocations, stats, campaigns, and make notes
    $admin_access       =  80;    # Ad system controllers able to edit ad campaigns, ads, and allocations
    $controller_access  =  90;    # Ad system technical assistants able to edit ad campaigns, ads, and allocations
    $super_admin_access =  100;   # Ad system programmers able to debug and edit ad campaigns, ads, and allocations


# The following (global) array associates privilige levels in ad_system_users.dat file into an access level number:

    %access_types = ( 'PARTNERSHIP',     $partner_access,
		      'EDITOR',          $editorial_staff,
		      'WEBDEV',          $editorial_staff,
		      'MARKETING',       $editorial_staff,
		      'SITE_DIRECTOR',   $editorial_staff,
		      'TRAFFIC_AUDITOR', $auditor_access,
		      'TRAFFIC_AUDIT',   $auditor_access,
		      'ACCOUNTANT',      $accountants_access,
		      'ACCOUNTING',      $accountants_access,
		      'FINANCE',         $accountants_access,
		      'DIRECTOR',        $director_access,
		      'BIZDEV',          $director_access,
		      'SALES',           $sales_dept,
		      'SALES_MANAGER',   $sales_manager,
		      'ADMIN',           $admin_access,
		      'SALES_ADMIN',     $admin_access,
		      'TECH_ADMIN',      $controller_access,
		      'SUPER_ADMIN',     $super_admin_access );

# Setup user navbar links according to minimum access level to see that link:

    $nav_links = {};
    $nav_links->{  $editorial_staff     }->{ 'Ad Allocations'    }  =  $ad_allocedit_url;
    $nav_links->{  $editorial_staff     }->{ 'Ad Banners'        }  =  $ad_infoedit_url;
    $nav_links->{  $accountants_access  }->{ 'Ad Campaigns'      }  =  $ad_compedit_url;
    $nav_links->{  $advertiser_access   }->{ 'Ad Reports'        }  =  $ad_report_url;
    $nav_links->{  $partner_access      }->{ 'Web Traffic Stats' }  =  $traffic_report_url;
    $nav_links->{  $anonymous_access    }->{ 'Logout'            }  =  "$ad_mainmenu_url?op=logout";
    $nav_links->{  $editorial_staff     }->{ 'System Notes'      }  =  qq|$ad_notes_url?context=AdSystem|;
    $nav_links->{  $editorial_staff     }->{ 'FAQ Forum'         }  =  qq|$ad_notes_url?context=FAQ|;
    $nav_links->{  $sales_dept          }->{ 'RFP Repository'    }  =  qq|$ad_notes_url?context=RFP|;
    $nav_links->{  $sales_dept          }->{ 'Make A Note'       }  =  qq|$ad_notes_url?op=edit|;
    $nav_links->{  $admin_access        }->{ 'Users'             }  =  $ad_users_url;
    $nav_links->{  $admin_access        }->{ 'Invoices'          }  =  $ad_invoices_url;
    $nav_links->{  $super_admin_access  }->{ 'Contact Lists'     }  =  $ad_contactlists_url;
    $nav_links->{  $super_admin_access  }->{ 'Verify Ad Stats'   }  =  "/cgi-bin/ad_report_verify.pl";
    $nav_links->{  $super_admin_access  }->{ 'System Logs'       }  =  $ad_logs_url;


    %nav_colors = ();

    $nav_colors{ $advertiser_access }  = 'FFFFBB';    # Light Yellow
    $nav_colors{ $partner_access }     = 'FFFFAA';    # Mid Yellow
    $nav_colors{ $auditor_access }     = 'AAFFAA';    # Mid Green
    $nav_colors{ $editorial_staff }    = 'CCFFCC';    # Light Green
    $nav_colors{ $sales_dept }         = 'AAAAFF';    # Mid Blue
    $nav_colors{ $accountants_access } = 'AAAAFF';    # Mid Blue
    $nav_colors{ $admin_access }       = 'FFBBBB';    # Light Red
    $nav_colors{ $super_admin_access } = 'FF9999';    # Deep Red

    
# Setup start page url depending on access_level:
    
    $start_page = {};
    $start_page{ $anonymous_access }   = '';
    $start_page{ $advertiser_access }  = $ad_report_url;
    $start_page{ $editorial_staff   }  = $traffic_report_url;
    $start_page{ $accountants_access } = $traffic_report_url;
    $start_page{ $director_access }    = $traffic_report_url;
    $start_page{ $sales_dept }         = $ad_compedit_url;
    $start_page{ $sales_manager }      = $ad_compedit_url;
    $start_page{ $admin_access }       = $ad_mainmenu_url;
    $start_page{ $controller_access }  = $ad_infoedit_url;
    $start_page{ $super_admin_access } = $traffic_report_url;


# Set page header, footer, and nav links for all adsystem CGI interface web pages
    
    $page_header = qq|
        <HTML>
        <HEAD>
        <TITLE>Your Network Ad System</TITLE>
        <LINK REL="STYLESHEET" TYPE="text/css" HREF="/adsystem_styles.css">
        </HEAD>
        <BODY BGCOLOR="$backcolor">
	<H1>Your Ad System v$mod_date</H1>
            |;


    $page_footer = qq|
	</BODY>
	</HTML>
	    |;



# getUserInfo: Read ad_system_users.dat table into a hash

sub getUserInfo {

    my ( $tmp_username, 
	 $tmp_password,
	 $tmp_access_type,
	 $tmp_name,
	 $tmp_email,
	 $tmp_status,
	 $tmp_access_level ) = ();

    if ( open( INFILE, "< $adusersfile" ) ) { 
	while (<INFILE>) {
	    chomp;

	    ( $tmp_username, 
	      $tmp_password,
	      $tmp_access_type,
	      $tmp_name,
	      $tmp_email,
	      $tmp_siteids,
	      $tmp_status ) = split /\t/, $_;
	
	    $ad_system_users{ $tmp_username }->{ 'password'     } = $tmp_password;
	    $ad_system_users{ $tmp_username }->{ 'access_type'  } = $tmp_access_type;
	    $tmp_access_level = $access_types{ $tmp_access_type };
	    $ad_system_users{ $tmp_username }->{ 'access_level' } = $tmp_access_level;
	    $ad_system_users{ $tmp_username }->{ 'name'         } = $tmp_name;
	    $ad_system_users{ $tmp_username }->{ 'email'        } = $tmp_email;
	    $ad_system_users{ $tmp_username }->{ 'siteids'      } = $tmp_siteids;
	    $ad_system_users{ $tmp_username }->{ 'status'       } = $tmp_status;
	    
	}
	close INFILE;
    } else {
	$error_messages .= "ERROR: getUserInfo: Could not read ad_system_users file $adusersfile : $! <BR>\n";
    }

} # sub getUserInfo



# loginUser: Check username and password inputs and print start of output web page
#
# Inputs:   $minimum_access_level (local)   = lowest user_access_level 
#                                             allowed to access this interface,
#           HTTP_COOKIE           "adsys"   = the user's browser cookie,
#           %form                 (global)  = array of all CGI form inputs,
#           %ad_system_users      (global)  = array of users allowed to access 
#                                             the ad system CGI interfaces, 
#                                             listed in ad_config_lib.pl
# 
# Outputs:  print STDOUT normal pageheader and/or ad system login page,
#           HTTP_COOKIE           "adsys"   = user's browser cookie updated,
#           %form                 (global)  = array of all CGI form inputs,
#           $user_access_level    (global)  = the user's current ad system access level
#

sub loginUser {

    my $minimum_access_level  =  shift;
    my $user_cookie           =  '';
    my $op                    =  '';
    my $email_passwd_link     =  '';
    my $tmp_access_level      =  '';
    my $tmp_access_type       =  '';
    my $cookie_expire_time    =  '';
       $username              =  '';
       $password              =  '';


    # Read all CGI inputs

    $form = new FormParse;
    $form->Parse;		

    $username = $1 if $form->{'in'}->{'UserName'} =~ /(\w{2,10})/;
    $password = $1 if $form->{'in'}->{'Password'} =~ /([\w\-\$\#]{2,10})/;
    $op       = $1 if $form->{'in'}->{'op'} =~ /(\w+)/;
    
    # Read username and password from browser cookie

    if ( $ENV{'HTTP_COOKIE'} =~ /adsys=(\w{2,10})\:([\w\-\$\#]{2,12})/ ) {
	$username ||= $1;
	$password ||= $2;
    }


    # Handle user logout option (reset browser cookie)

    if ( $op eq 'logout' ) {
        $error_messages     .=  "NOTICE: $username logged out. Ad system password cookie reset.";
	$username            =  '';
	$password            =  '';
	$cookie_expire_time  = time - (60 * 60 * 24 * 30);
	$user_cookie_expires = strftime("%a, %d-%b-%Y %H:%M:%S GMT", localtime($cookie_expire_time));
	$user_cookie         = qq|Set-Cookie: adsys=nobody\:loggedout\; path=/\; expires=$user_cookie_expires\n|;
	# $user_cookie         =  "Set-Cookie: adsys=nobody:loggedout\n";
	$user_access_level   =  0;

    }


    # Check user access accroding to their password and ad system user group access

    if ( $username =~ /\w+/ ) {

	$user_access_level = 0;

       # If this script allows advertiser_access then add ad campaigns to ad_system_users list:

	if ( $minimum_access_level <= $advertiser_access ) {

	    my ( $tmp_username, $tmp_campaign_name, $tmp_password ) = ();
	    if ( open( INFILE, "< $adcompfile" ) ) { 
		while (<INFILE>) {
		    chomp;
		    ( $tmp_username, $tmp_campaign_name, $tmp_password ) = split /\t/, $_;
		    next if $tmp_username ne $username;
		    $ad_system_users{ $tmp_username }->{ 'name' }         = $tmp_campaign_name;
		    $ad_system_users{ $tmp_username }->{ 'password' }     = $tmp_password;
		    $ad_system_users{ $tmp_username }->{ 'access_level' } = $advertiser_access;
		    $ad_system_users{ $tmp_username }->{ 'access_type'  } = 'ADVERTISER';
		    $ad_system_users{ $tmp_username }->{ 'status' }       = 'ACTIVE';
		}
		close INFILE;
	    } else {
		$error_messages .= "ERROR: userLogin: Could not read ad_campaign_info file $adcompfile : $! <BR>\n";
	    }
	   
	} # if advertiser_access


	&getUserInfo;   # Read all users from ad_system_users.dat
	
	$tmp_access_level = $ad_system_users{ $username }->{ 'access_level' };
	$tmp_access_type  = $ad_system_users{ $username }->{ 'access_type'  };

	$email_passwd_link = qq|[<A HREF="$ad_users_url?op=mailpasswd&UserName=$username">Email my passwd</A>]|;
	$email_passwd_link = '' if $tmp_access_level < $auditor_access;

	if ( $op eq 'mailpasswd' ) {
	    $email_passwd_link = '';
	    $user_status = $ad_system_users{ $username }->{ 'status' };
	    if ( $user_status eq 'ACTIVE' ) {
		$user_password = $ad_system_users{ $username }->{ 'password' };
		if ( $user_password ) {
		    $user_id = $username;
		    $user_name = $ad_system_users{ $username }->{ 'name' };
		    $user_email = $ad_system_users{ $username }->{ 'email' };
		    if ( $user_email ) {
			&sendAccountActivatedEmail;
		    } else {
			$error_messages .= "ERROR: Can not email password for $username: No email address set for $username <BR>\n";
		    }
		} else {
		    $error_messages .= "ERROR: Can not email password for $username: No password set for $username <BR>\n";
		}
	    } else {
		$error_messages .= "ERROR: Can not email password for $username: Account not activated. <BR>\n";
	    }
	} # if mailpasswd

	# Check entered password against user's password:

	if ( $ad_system_users{ $username }->{ 'password' } =~ /\w+/ ) {
	    if (  $ad_system_users{ $username }->{ 'password' } eq $password ) {
		$user_account_status = $ad_system_users{ $username }->{ 'status' };
		if ( $user_account_status ne 'ACTIVE' ) {
		    print "Content-type: text/html\n\n";
		    print qq| $page_header ACCESS DENIED: User $username login not activated. $page_footer |;
		    exit;		    
		}
		$user_access_level =  $ad_system_users{ $username }->{ 'access_level' };
		if ( $user_access_level < $minimum_access_level ) {
		    print "Content-type: text/html\n\n";
		    print qq|
			$page_header
			    ACCESS DENIED: This part of ad system is for administrative purposes. Nothing to see here 
			    but you can click <A HREF="$ad_mainmenu_url">here</A> to go back to the main menu page.
			$page_footer
			    |;
		    exit;
		} else {
		    # Login OK: update the user's password cookie to expire 30 days from now
		    $cookie_expire_time  ||= time + (60 * 60 * 24 * 30);
		    $user_cookie_expires   = strftime("%a, %d-%b-%Y %H:%M:%S GMT", localtime($cookie_expire_time));
		    $user_cookie           = qq|Set-Cookie: adsys=$username\:$password\; path=/\; expires=$user_cookie_expires\n|;

		    if (( $users_logfile ) && ( $dontlogthispage < 1 )) {
			my $date = `/bin/date`; chomp $date;
			my $dow = $1 if $date =~ /^(\w{3}) /;
			my $users_logfile = "$users_logfile.$dow";
			if ( ( -M "$users_logfile" ) > 3 ) {
			    `/bin/cp /dev/null $users_logfile`;  # Zero out log on date roll over
			}
			if ( open LOGFILE, ">> $users_logfile" ) {
			    print LOGFILE qq|$date\t[$username]\t$ENV{'REMOTE_ADDR'}\t$ENV{'SCRIPT_NAME'}\t$form->{'in'}->{'op'}\n|;
			    close LOGFILE;
			}
		    } # logfile

		}
	    } else {
		$error_messages .= "ERROR: Password for username $username was not entered correctly. $email_passwd_link ";
	    }

	} else {
	    $error_messages .= "ERROR: Username $username is not a valid ad system user.";
	}

    } # if username


   # Start printing output page and new cookies

    print $user_cookie;
    print "Content-type: text/html \n\n";
    print $page_header;


    # Print login screen if user not logged in

    if (( $force_login == 1 ) && ( $user_access_level < $minimum_access_level )) {

	$tmp_startpage_url = $start_page{ $user_access_level } || $ad_mainmenu_url;

	print qq|
	       $error_messages
	       <FORM METHOD="POST" ACTION="$tmp_startpage_url">
	       Username: <INPUT NAME="UserName" SIZE="8">
	       <BR>
	       Password: <INPUT TYPE="PASSWORD" NAME="Password" SIZE="8">
	       <BR>
	       <INPUT TYPE="SUBMIT" NAME="ACTION" VALUE="Login">
	       </FORM>
	       $page_footer
		|;
	exit;
    }


} # sub loginUser







# userNavlinks: Setup user's navbar links:
#
#   Inputs:   (%{$nav_links->{$user_access_level}})   (global)   Hash of links and urls
#
#   Outputs:  $user_navbar  (local)  HTML Formatted list of links for the given user_access_level

sub userNavbar {

    $nav_links->{ $sales_dept }->{'My Notes'} = qq|$ad_notes_url?author=$username|;

    my $user_info_link = $user_access_level > $advertiser_access ? 
	qq|[<A HREF="$ad_users_url?user_id=$username">User Info</A>]| : '[Logged In]';

    my $user_navbar = qq|<TD BGCOLOR=\#FFFFFF> $ad_system_users{$username}->{'name'} $user_info_link</TD>|;;
    my $tmp_column  = 1;
    my $bgcolor     = '';
    my $numcolumns  = 4;

    foreach $tmp_access_level ( sort keys ( %{ %{$nav_links} } ) ) {
	next if $tmp_access_level > $user_access_level;
	$bgcolor = $nav_colors{ $tmp_access_level } || 'FFFFFF';
	foreach $tmp_link ( keys (%{$nav_links->{$tmp_access_level}}) ) {
	    $user_navbar .= qq|<TD BGCOLOR=\#$bgcolor><A HREF="$nav_links->{$tmp_access_level}->{$tmp_link}">$tmp_link</A></TD>|;
	    $user_navbar .= '</TR><TR>' if ( $tmp_column++ % $numcolumns ) == ( $numcolumns - 1 );
	}
    }

    return "<TABLE CELLSPACING=2 CELLPADDING=3><TR>$user_navbar</TR></TABLE>";

} # sub userNavbar




# Print the output web page

sub printWebpage {

    my $output_format = shift;
    my $navbar        = '';

    if ( $output_format eq 'PRINT' ) {
	$messages       = '';
	$error_messages = '';
    } else {
	$navbar = &userNavbar;
    }

    print qq|
    $messages 
    <P> 
    $error_messages 
    <P> 
    $navbar 
    <BR>
    $page_contents
    <BR>
    $page_footer
    |;


} # sub printWebPage




# Turn off buffering on STDOUT so each print statement displays immediately:

sub unbufferSTDOUT {

    $old_fh = select STDOUT;
    $| = 1;
    select $old_fh;

} # sub unbufferSTDOUT





# BackupAndSaveFile: Copy the file $filepath to $backupfile then write $file_data to $filepath
#
# Inputs:   $file_data       (global) new contents of file to save
#           $bakupfile       (local)  full path to backup file
#           $backupfilepath  (global) path of the backup directory
#           $filepath        (local)  full path of file to save
#
# Output:   $error_messages  (global) appended to if an error occured
#           $messages        (global) normal status messages

sub BackupAndSaveFile {

    my $filepath         = shift;
    my $filename         = $1 if $filepath =~ /\/?([\w\-\.]+)$/;
    my $todayfullsysdate = $1 if `/bin/date +%Y%m%d` =~ /(\d+)/;
    my $backupfile       = "$backupfilepath/$filename\.$todayfullsysdate";
    my $backup_data      = '';

    if ( open ( INFILE, "< $filepath" ) ) {
	while (<INFILE>) {
	    $backup_data .= $_;
	}
	close INFILE;
		
	if ( open DATAFILE, "> $backupfile" ) {
	    print DATAFILE $backup_data;
	    close DATAFILE;
	    $messages .= "DEBUG: Backup file $backupfile saved. <BR>\n" if $debugging == 1;
	} else {
	    $error_messages .= "ERROR: Could not create a backup file $backupfile: $! <BR> \n";
	    return 0;
	}

    } else {
	$error_messages .= "WARNING: Can not read previous version of $filepath for backup. <BR> \n";
	# Don't return here, it's OK if file didn't exist, we'll create it.
    }


    # Save new data to file

    if ( open DATAFILE, "> $filepath" ) {
	print DATAFILE $file_data;
	close DATAFILE;
	$messages .= "DEBUG: New dat file $filepath saved. <BR>\n" if $debugging == 1;
	$messages .= "NOTICE: Changes saved. <BR>\n";
    } else {
	$error_messages .= "ERROR: Can not save dat file $filepath : $! <BR> \n";
	return 0;
    }

    return 1;

} # sub BackupAndSaveFile





sub NiceDate2ISODate {
    
    my $nicedate = shift;
    return $nicedate if $nicedate !~ /\d/;
    return $nicedate if $nicedate =~ /\d{2}\/\w{3}\/\d{4}/;
    my ( $mon, $day, $year ) = split(/[\/\-]/, $nicedate);
    $day =~ s/^0//;
    $day = '0' . $day if $day < 10;
    $mon =~ s/^0//;
    my %monthlist = qw/1 Jan 2 Feb 3 Mar 4 Apr 5 May 6 Jun 7 Jul 8 Aug 9 Sep 10 Oct 11 Nov 12 Dec/;
    $year =~ s/\W//g;
    $year = '19' . $year if $year > 70 && $year < 1900;
    $year = '20' . $year if $year >= 0 && $year <= 70;

    return $day . '/' . $monthlist{$mon} . '/' . $year;

}  # sub NiceDate2ISODate: Converts 2/3/98 to 03/Feb/1998




sub ISODate2NiceDate {

    my $isodate = shift;
    return $isodate if $isodate !~ /(\d\d)\/(\w{3})\/\d{2}(\d{2})/;
    my ( $day, $month_name, $year ) = ( $1, $2, $3 );
    $day =~ s/^0//;
    my %monthlist = qw/Jan 01 Feb 02 Mar 03 Apr 04 May 05 Jun 06 Jul 07 Aug 08 Sep 09 Oct 10 Nov 11 Dec 12/;
    my $mon = $monthlist{ $month_name };
    $mon =~ s/^0//;
    return "$mon/$day/$year";

} # sub ISODate2NiceDate: Converts 09/Mar/1998 to 3/9/98



sub ISODate2SysDate {

    my $ISODate = shift;
    if ($ISODate =~ /(\d{2})\/([A-Za-z]{3})\/(\d{4})/) {
	my %monthlist = qw/Jan 01 Feb 02 Mar 03 Apr 04 May 05 Jun 06 Jul 07 Aug 08 Sep 09 Oct 10 Nov 11 Dec 12/;
	return $3 . $monthlist{$2} . $1;
    } else {
	return $ISODate;
    } 

}  # sub ISODate2SysDate: Converts 02/Jan/1998 to 19980102 to do date comparisons


sub ISODate2logdate {

    my $ISODate = shift;
    if ($ISODate =~ /(\d{2})\/([A-Za-z]{3})\/\d\d(\d{2})/) {
	my %monthlist = qw/Jan 01 Feb 02 Mar 03 Apr 04 May 05 Jun 06 Jul 07 Aug 08 Sep 09 Oct 10 Nov 11 Dec 12/;
	return $3 . $monthlist{$2} . $1;
    } else {
	return $ISODate;
    } 

}  # sub ISODate2logdate: Converts 02/Jan/1998 to 980102 to do date comparisons



sub SysDate2ISODate {

    my $SysDate = shift;
    if ( $SysDate =~ /^(\d{4})(\d{2})(\d{2})$/ ) {
	my ( $YYYY, $mm, $dd ) = ( $1, $2, $3 );
	my %monthlist = qw/01 Jan 02 Feb 03 Mar 04 Apr 05 May 06 Jun 07 Jul 08 Aug 09 Sep 10 Oct 11 Nov 12 Dec/;
	return "$dd/" . $monthlist{$mm} . "/$YYYY";
    } else {
	return $SysDate;
    }

}  # sub SysDate2ISODate: Converts 19980123 to 23/Jan/1998


sub SysDate2ISODateDow {

    my $SysDate = shift;
    if ( $SysDate =~ /^(\d{4})(\d{2})(\d{2})$/ ) {
	my ( $YYYY, $mm, $dd ) = ( $1, $2, $3 );
	my $tmp_date = "$YYYY$mm$dd";
	my $dow = Date::Manip::ParseDate( "$tmp_date" );
	$dow = Date::Manip::UnixDate( "$dow", '%a' );
	my %monthlist = qw/01 Jan 02 Feb 03 Mar 04 Apr 05 May 06 Jun 07 Jul 08 Aug 09 Sep 10 Oct 11 Nov 12 Dec/;
	return "$dd/" . $monthlist{$mm} . "/$YYYY $dow";
    } else {
	return $SysDate;
    }

}  # sub SysDate2ISODateDow: Converts 19980123 to 23/Jan/1998 Wed


sub logdate2SysDate {

    my $logdate = shift;
    if ( $logdate =~ /^(\d{2})(\d{4})$/ ) {
	my ( $yy, $mm_dd ) = ( $1, $2 );
	my $YY = ( $yy >= 70 ) ? '19' : '20';
	return "$YY$yy$mm_dd";
    } else {
	return $logdate;
    }

}  # logdate2SysDate: Converts 991123 to 19991123, and 000224 to 20000224



sub logdate2SysDateSep {

    my $logdate = shift;
    my $sep     = shift;
       $sep   ||= '/';
    if ( $logdate =~ /^(\d{2})(\d{2})(\d{2})$/ ) {
	my ( $yy, $mm, $dd ) = ( $1, $2, $3 );
	my $YY = ( $yy >= 70 ) ? '19' : '20';
	return "$YY$yy$sep$mm$sep$dd";
    } else {
	return $logdate;
    }

}  # logdate2SysDateSep: Converts 991123 to 1999/11/23, and 000224 to 2000/02/24



sub logdate2SysDateDow {

    my $logdate = shift;
    my $sep     = shift;
       $sep   ||= '/';
    if ( $logdate =~ /^(\d{2})(\d{2})(\d{2})$/ ) {
	my ( $yy, $mm, $dd ) = ( $1, $2, $3 );
	my $YY = ( $yy >= 70 ) ? '19' : '20';
	my $tmp_date = "$YY$yy$mm$dd";
	my $dow = Date::Manip::ParseDate( "$tmp_date" );
	$dow = Date::Manip::UnixDate( "$dow", '%a' );
	return "$YY$yy$sep$mm$sep$dd $dow";
    } else {
	return $logdate;
    }

}  # logdate2SysDate: Converts 991123 to '1999/11/23 Fri', and 000224 to '2000/02/24 Tue'




sub logdate2ISODate {

    my $logdate = shift;
    if ( $logdate =~ /^(\d{2})(\d{2})(\d{2})$/ ) {
	my ( $yy, $mm, $dd ) = ( $1, $2, $3 );
	my %monthlist = qw/01 Jan 02 Feb 03 Mar 04 Apr 05 May 06 Jun 07 Jul 08 Aug 09 Sep 10 Oct 11 Nov 12 Dec/;
	my $YY = ( $yy >= 70 ) ? '19' : '20';
	return "$dd/" . $monthlist{$mm} . "/$YY$yy";
    } else {
	return $logdate;
    }

}  # logdate2SysDate: Converts 991123 to 19991123, and 000224 to 20000224




# parseDateInputs: Given StartDate_in and StopDate_in (global) set @logdates list
#                  and convert dates to various common formats

sub parseDateInputs {


    # Convert date inputs to each common date format:

    if ( $StartDate_in =~ /(\d{6})/ ) {           # Date inputs can be logdate format : yymmdd

	$StartDate_yymmdd   = $StartDate_in;
	$StopDate_yymmdd    = $StopDate_in;

	$StartDate_YYYYmmdd = &logdate2SysDate( $StartDate_in );
	$StopDate_YYYYmmdd  = &logdate2SysDate( $StopDate_in );

	$StartDate_ISO      = &SysDate2ISODate( $StartDate_YYYYmmdd );
	$StopDate_ISO       = &SysDate2ISODate( $StopDate_YYYYmmdd );

	$StartDate_nice     = &ISODate2NiceDate( $StartDate_ISO );
	$StopDate_nice      = &ISODate2NiceDate( $StopDate_ISO );

    } elsif ( $StartDate_in =~ /\d\d(\d{6})$/ ) {  # Date inputs can also be SysDate format : YYYYmmdd

	$StartDate_yymmdd = $1;
	$StopDate_yymmdd = $1 if $StopDate_in =~ /\d\d(\d{6})/;

	$StartDate_YYYYmmdd = $StartDate_in;
	$StopDate_YYYYmmdd = $StopDate_in;

	$StartDate_ISO      = &SysDate2ISODate( $StartDate_YYYYmmdd );
	$StopDate_ISO       = &SysDate2ISODate( $StopDate_YYYYmmdd );

	$StartDate_nice     = &ISODate2NiceDate( $StartDate_ISO );
	$StopDate_nice      = &ISODate2NiceDate( $StopDate_ISO );

    } elsif ( $StartDate_in =~ /\d\/\w/ ) {       # Date inputs can also be either mm/dd/yy or dd/mmm/YYYY

	$StartDate_ISO      = &NiceDate2ISODate( $StartDate_in );
	$StopDate_ISO       = &NiceDate2ISODate( $StopDate_in  );

	$StartDate_YYYYmmdd = &ISODate2SysDate( $StartDate_ISO );
	$StopDate_YYYYmmdd  = &ISODate2SysDate( $StopDate_ISO );

	$StartDate_yymmdd   = $1 if $StartDate_YYYYmmdd =~ /\d\d(\d{6})/;
	$StopDate_yymmdd    = $1 if $StopDate_YYYYmmdd =~ /\d\d(\d{6})/;

	$StartDate_nice     = &ISODate2NiceDate( $StartDate_ISO );
	$StopDate_nice      = &ISODate2NiceDate( $StopDate_ISO );

    } # if StartDate


    if ( $StartDate_YYYYmmdd > $StopDate_YYYYmmdd ) {
	print "INPUT ERROR: StartDate ( $StartDate_ISO ) and StopDate ( $StopDate_ISO ) are out of order.";
	exit;
    } 


    # NOTE: This @logdates code block was copied/pasted from ad_sendfiles.pl and is similar 
    # to code in fill_ads_table.pl and web_traffic_stats.pl
	     
    $messages .= "DEBUG: Parsing logdate range $StartDate_ISO - $StopDate_ISO ... <BR>\n" if $debugging == 1;

    push @logdates, $StartDate_yymmdd;
    $tmp_date = $StartDate_YYYYmmdd;

    while ( $tmp_date ne $StopDate_YYYYmmdd ) {

	$tmp_date = &Date::Manip::ParseDateString( $tmp_date );
	$tmp_date = &Date::Manip::DateCalc( $tmp_date, "+ 1 days" );
	$tmp_date = &Date::Manip::UnixDate("$tmp_date", '%Y%m%d');
	if ( $tmp_date =~ /^\d{2}(\d{6})/ ) {
	    push @logdates, $1;
	} else {
	    $error_messages .= "ERROR: Date::Manip returned blank or odd date : $tmp_date <BR>\n";
	}

	# We can set limit on maximum number of days limit per ad report here in case we ever need to:
	if ( ( $number_of_days_limit > 1 ) && ( $numberofdays++ > $number_of_days_limit )) {
	    $messages .= "NOTICE: Number of logdates $numberofdays exceeds ";
	    $messages .= "number_of_file_transfers_limit $number_of_days_limit : ";
	    $messages .= "last date read will be $logdate <BR>\n";
	    last;
	}

    } # while logdate

    $numberofdays = @logdates;

    if ( $debugging == 1 ) {
	$messages .= "\nDEBUG: \@logdates list set to " . (join ', ', @logdates) . " = $numberofdays logdates <BR>\n";
	$messages .= qq|
     StartDate_ISO       = $StartDate_ISO
     StopDate_ISO        = $StopDate_ISO
     StartDate_yymmdd    = $StartDate_yymmdd
     StopDate_yymmdd     = $StopDate_yymmdd
     StartDate_YYYYmmdd  = $StartDate_YYYYmmdd
     StopDate_YYYYmmdd   = $StopDate_YYYYmmdd \n|;	
    }

} # sub parseDateInputs: given StartDate_in and StopDate_in setup list of @logdates




# sortAlphaNumeric: Used as an argument to "sort" to sort a list alphanumerically

sub sortAlphaNumeric {
    my $n1 = shift;
    my $n2 = shift;
    if ( $n1 =~ /^\d+/ ) {
	return $n2 <=> $n1;
    } else {
	return $n1 cmp $n2;
    }
} # sub sortAlphaNumeric: How dumb is it that cmp can't sort numbers?






# getActiveSitenames: Return hash of ACTIVE sites to which ads can be allocated and are not subsites
#
#        Inputs:  None.  Reads (global) $logname2siteid hash from ad_config_lib.pl
#
#        Outputs:  %sitenames  (global) hash of sitenames => siteid

sub getActiveSitenames {

    # Make list of ACTIVE siteids that are not subsites:

    %sitenames      = ();
    my $tmp_siteid  = '';
    my $logname     = '';
    foreach $logname ( keys %logname2siteid ) {
	$tmp_siteid = $logname2siteid{$logname};
	next if $sitesconfig{ $tmp_siteid . 'ad_status' }           ne 'ACTIVE';
	next if $sitesconfig{ $tmp_siteid . 'ad_allocation_group' } =~ /^[a-z][a-z]/;   # at000en uses 'XX' to bypass ad_rotateset.pl
	$sitenames{ $sitesconfig{ $tmp_siteid . 'sitename' } } = $tmp_siteid;
    }


} # getActiveSitenames




sub getSalesPeople {

    %ad_sales_people = ();
    foreach $tmp_key ( keys %ad_system_users ) {
	$tmp_access_level = $ad_system_users{ $tmp_key }->{ 'access_level' };
	next if (( $tmp_access_level < $sales_dept ) || ( $tmp_access_level > $admin_access ));
	$ad_sales_people{ $tmp_key } = $ad_system_users{ $tmp_key } -> { 'name' };
	$numsalespeople++;
    }

} # sub getSalesPeople




sub getInvoicesInfo {

    my $ad_campaign_in    =  shift;
    my $tmp_campaign      = '';
    my $tmp_invoice       = '';
    my $tmp_invoice_link  = '';
    $ad_invoices_info     =  {};

    if ( open INFILE, "< $adrunsfile" ) {
	while (<INFILE>) {
	    next if $_ !~ /^(\w{4})\t([\w\-]+)/;
	    $tmp_campaign = $1;
	    $tmp_invoice  = $2;
	    next if $ad_campaign_in && ( $tmp_campaign ne $ad_campaign_in );
	    $tmp_invoice_link = qq| <A HREF="$ad_invoices_url?adinvoice=$tmp_invoice">$tmp_invoice</A> |;
	    $ad_invoices_info{ $tmp_campaign }->{ 'invoice_links' }    .= $tmp_invoice_link;
	    $ad_invoices_info{ $tmp_campaign }->{ 'invoice_id' } ||= $tmp_invoice;
	}
	close INFILE;
    } else {
	$error_messages .= "ERROR: getInvoicessInfo: Can not read ad_runs file $adrunsfile : $! <BR>\n";
    }

} # sub getInvoicesInfo




# getCampaignsInfo: Make global hash of campaign info for all campaigns or a given campaign
#
#      Inputs:      $ad_campaign_in     (local)    
#
#      Outputs:     %ad_campaign_info  (global)  Hash of ad campaign id => 'name', 
#                                                         ad campaign id => 'numberofads'
#
#                   ...and if an ad campaign id was specified...
#
#                   %ad_banner_info     (global)  Hash of ad banner id => 'name'
#                                                         ad banner id => 'HTML' 
#

sub getCampaignsInfo {

    my $ad_campaign_in     =  shift;
    my $tmp_campaign       =  '';
    my $tmp_banner_id      =  '';
    my $line               =  '';
    my @tmp_campaign_info  =  ();
    $ad_campaign_info      =  {};
    $ad_banner_info        =  {}; 

    $ad_campaign_in = '' if $ad_campaign_in eq 'All';

    # Read campaign name from ad_campaign_info.dat:

    if ( open INFILE, "< $adcompfile" ) {
	while (<INFILE>) {
	    next if $_ !~ /^(\w{4})\t/;
	    $tmp_campaign = $1;
	    next if $ad_campaign_in && ( $tmp_campaign ne $ad_campaign_in );
	    @tmp_campaign_info = split /\t/;
	    $ad_campaign_info{ $tmp_campaign }->{ 'name' } = $tmp_campaign_info[1];  # campid -> campaign name
	}
	close INFILE;
    } else {
	$error_messages .= "ERROR: getCampaignsInfo: Can not read ad_campaign file $adcompfile : $! <BR>\n";
    }
    
    # Read ad banner info from ad_info.dat:

    if ( open INFILE, "< $adinfofile" ) {
	while ($line = <INFILE>) {

	    next if $line !~ /^(\w{4})(\w{6})\t/;
	    $tmp_campaign  = $1;
	    $tmp_banner_id = $2;
	    $ad_campaign_info{ $tmp_campaign }->{ 'numberofads' }++;     # campid -> number of ads


	    # If an ad_campaign was specified then get information about each banner in that campaign:

	    if ( $ad_campaign_in ) {

		next if $tmp_campaign ne $ad_campaign_in;

		$tmp_banner_id = $tmp_campaign . $tmp_banner_id;

		chomp $line;

		( $tmp, 
		  $tmp_height,
		  $tmp_width,
		  $tmp_targeturl,
		  $tmp_alttext,
		  $tmp_name ) = split /\t/, $line;

		$tmp_adtype = substr($tmp_banner_id, 4, 1);

		$ad_banner_info{ $tmp_banner_id }->{ 'height' }     = $tmp_height;
		$ad_banner_info{ $tmp_banner_id }->{ 'width' }      = $tmp_width;
		$ad_banner_info{ $tmp_banner_id }->{ 'targeturl' }  = $tmp_targeturl;
		$ad_banner_info{ $tmp_banner_id }->{ 'alttext' }    = $tmp_alttext;
		$ad_banner_info{ $tmp_banner_id }->{ 'name' }       = $tmp_name || $tmp_banner_id;
		$ad_banner_info{ $tmp_banner_id }->{ 'adtype' }     = $tmp_adtype;

		# Setup the HTML code to display the ad banner:

		if ( $tmp_adtype > 4 ) {

		    if ( open ADFILE, "< $ad_htmlad_path/ad$tmp_banner_id.dat" ) {
			while (<ADFILE>) {
			    $ad_banner_info{ $tmp_banner_id }->{ 'HTML' } .= $_;
			    $ad_banner_info{ $tmp_banner_id }->{ 'html_layer' } .= $_;
			}
			close ADFILE;
		    } else {
			$ad_banner_info{ $tmp_banner_id }->{ 'HTML' } = "NOTICE: getCampaignsInfo: Can not display HTML ad banner for $tmp_banner_id: $! <BR>\n";
			# $error_messages .= "NOTICE: getCampaignsInfo: Can not display HTML ad banner for $tmp_banner_id: $! <BR>\n";
		    }

		    if ( $tmp_adtype eq '6' ) {

			if ( open ADFILE, "< $ad_htmlad_path/ad$tmp_banner_id\-b.dat" ) {
			    while (<ADFILE>) {
				$ad_banner_info{ $tmp_banner_id }->{ 'HTML' } .= $_;
				$ad_banner_info{ $tmp_banner_id }->{ 'mega_layer' } .= $_;
			    }
			    close ADFILE;
			} else {
			    $ad_banner_info{ $tmp_banner_id }->{ 'HTML' } .= "NOTICE: getCampaignsInfo: Can not display MEGABANNER-B for $tmp_banner_id: $! <BR>\n";
			    # $error_messages .= "NOTICE: getCampaignsInfo: Can not display HTML ad banner for $tmp_banner_id: $! <BR>\n";
			}

		    }


		} else {

		    $ad_banner_info{ $tmp_banner_id }->{ 'HTML' } = qq|<A HREF="$tmp_targeturl" TARGET="_TomThumb">
			<IMG SRC="/banner/$tmp_banner_id.gif" WIDTH="$tmp_width" HEIGHT="$tmp_height" ALT="$tmp_alttext"></A>\n|;

		} # if tmp_banner_id

		# Make a formatted HTML table with the banner's info:

		$table_title = qq|Ad Banner Creative "$tmp_name"|;

		$table_data = join "\n", ( qq|Banner ID:\t$tmp_banner_id|,
					   qq|Campaign:\t$ad_campaign_info{ $tmp_campaign }->{ 'name' }|,
					   "Creative ID:\t$tmp_name",
					   "Dimensions:\t$tmp_width x $tmp_height",
					   qq|Target URL:\t<A HREF="$tmp_targeturl" TARGET="_TomThumb">$tmp_targeturl</A>|,
					   qq|Alt Text:\t"$tmp_alttext"| );
		
		&makeTable( $output_format );

		$ad_banner_info{ $tmp_banner_id }->{ 'infotable' } = $table_contents;
		$table_data     = '';
		$table_contents = '';

	    } # if ad_campaign_in

	} # while INFILE
	close INFILE;
    } else {
	$error_messages .= "ERROR: getCampaignsInfo: Can not read ad_info file $adinfofile : $! <BR>\n";
    }
    
} # sub getCampignsList




sub setCampaignNavlinks {

    $tmp_campaign      ||= $ad_campaign;
    $tmp_campaign_name ||= $ad_campaign_name;

    $nav_links->{ $accountants_access }->{"$tmp_campaign_name Ads"} = qq|$ad_infoedit_url?adcamp=$tmp_campaign|;
    $nav_links->{ $accountants_access }->{"$tmp_campaign_name Invoices "} = qq|$ad_invoices_url?adcamp=$tmp_campaign|;
    $nav_links->{ $accountants_access }->{"$tmp_campaign_name Ad Report"} = qq|$ad_report_url?adcamp=$tmp_campaign|;
    $tmp_campaign_linktext = "$tmp_campaign_name: ";
    $tmp_campaign_linktext =~ s/\s/+/g;
    $tmp_campaign_linktext =~ s/[^\w\:\+\/\.]/-/g;
    $nav_links->{ $sales_access }->{"Make A Note about $tmp_campaign_name"}   = qq|$ad_notes_url?op=edit&context=$tmp_campaign&subject=$tmp_campaign_linktext|;

} # sub setCampaignNavlinks





# ReadFile: Generic readfile function to grock files
#
#     Inputs:   $filepath  (local)  full path of the file to read
#
#     Output:   $filecontents  (local)  text contents of the file

sub ReadFile {

    my $filepath = shift @_;
    my $filecontents = '';
    
    if (open(READFILE, "< $filepath")) {	
        while (<READFILE>) {
            $filecontents .= $_;
        }
        close READFILE;
    } else {
        $filecontents = "ERROR: ReadFile: Can not read file $filepath : $!";
    }
    
    return $filecontents;

} # sub ReadFile




sub addCommas {

    local($_) = @_;
    1 while s/(.*\d)(\d\d\d)/$1,$2/;
    $_;

} # converts 1234567 to 1,234,567




# makeTable:     Makes HTML, PRE, or CSV table from a given $table_data
#                list rows separated by "\n" and columns separated by "\t", 
#                first row of $table_data is the column headers.
#
#     Inputs:    $table_data     (global)  Text data to format as a table
#
#     Output:    $table_contents (global)  Text data formatted as a table

sub makeTable {

    my $table_format    =  shift;
    my $fontspec        =  '';
    my $fontclose       =  '';
       $table_contents  =  '';

    # Include font tags in printable output format:

    if ( $table_format eq 'PRINT' ) {
	$fontspec  =  '<FONT FACE="ARIAL,HELVETICA" SIZE=2>';
	$fontclose =  '</FONT>';
	$table_format = 'HTML';
    }

    # Comma separated values:

    if ( $table_format eq 'CSV' ) {

	$table_contents .= "\n\n$table_title:\n" if $table_title;	
	$table_contents .= ( join ',', @table_headers ) . "\n";
	$table_contents .= $table_data;
	$table_contents  =~ s/\t/,/g;
	return 1;

    # Tab separated values:

    } elsif ( $table_format eq 'TSV' ) {
	
	$table_contents .= "\n\n$table_title:\n" if $table_title;
	$table_contents .= ( join "\t", @table_headers ) . "\n";
	$table_contents .= $table_data;
	return 1;

    # Fixed-width font formatted table:

    } elsif ( $table_format eq 'PRE' ) {

	# Scan table data to determine the widest line in each column:

	$max_page_width    = 110;  # Set to number of chars maximum page width
	$ellipsis          = '';   # Set to '...' if you want truncated columns to show ellipsis

	# These are local, don't set these:

	$max_column_width  = 0;
	@column_widths     = ();
	$column_number     = 0;
	$page_width        = 0;
	$number_of_columns = 0;
	@tmp_table_headers = @table_headers;

	foreach $column ( @tmp_table_headers ) {

	    # Strip HTML tags and leading/trailing spaces before measuring length of column:
	    
	    $column =~ s/<[^>]+>//g;
	    $column =~ s/^\s+//g;
	    $column =~ s/\s+$//g;
	    $column_length = length ( $column );
	    
	    $column_widths[$column_number] = $column_length if $column_length > $column_widths[$column_number];
	    
	    $number_of_columns = $column_number if $column_number > $number_of_columns;
	    
	    $column_number++;

	} # foreach column

	foreach $row ( split /\n/, $table_data ) {
	    $column_number  = 0;
	    foreach $column ( split /\t/, $row ) {

		# Strip HTML tags and leading/trailing spaces before measuring length of column:

		$column =~ s/<[^>]+>//g;
		$column =~ s/^\s+//g;
		$column =~ s/\s+$//g;
		$column_length = length ( $column );
		
		$column_widths[$column_number] = $column_length if $column_length > $column_widths[$column_number];
		
		$number_of_columns = $column_number if $column_number > $number_of_columns;

		$column_number++;

	    } # foreach column

	} # foreach row


	# If sum of column widths are wider than the page width then resize each column width:

	foreach $column_width ( @column_widths ) {
	    $page_width += $column_width;
	}

	if ( $page_width > $max_page_width ) {
	    $max_column_width = int ( $max_page_width / $number_of_columns ); 
	    foreach $column_number ( 0..$number_of_columns ) {
		$column_widths[$column_number] = $max_column_width if $column_widths[$column_number] > $max_column_width;
	    }
	} # if page_width


	# Align table headers:

	$table_contents = '';
	$table_contents .= "<H2><U>$table_title:</U></H2>\n" if $table_title;
	$table_contents .= "<PRE>";

	if ( @table_headers ) {
	    $table_contents .= '<B>';
	    $column_number  = 0;
	    foreach $column ( @table_headers ) {

		# Set HTML tags aside for a moment:

		( $column_htmltag_front, $column_text, $column_htmltag_back ) = ();
		( $column_htmltag_front, $column_text, $column_htmltag_back ) = ( $1, $2, $3 ) if $column =~ /(<[^>]+>)([^<]+)(<[^>]+>)/;
		$column = $column_text if $column_text;
		$column =~ s/\&nbsp\;/ /;

		$column_length = length ( $column );

		# Truncate and add ellipsis to column text that is too long

		if ( $max_column_width > 5 ) {
		    if ( $column_length >= $max_column_width ) {
			$column = substr( $column, 0, ( $max_column_width - length($ellipsis) ) );
			$column .= $ellipsis;
			$column_length = $max_column_width;
		    } 
		} # if max_column_width

		# Pad column text to uniform column width

		for ( $column_length..$column_widths[$column_number] ) {
		    $column_htmltag_back .= ' ';
		}

		$table_contents .= join '', ( ' ', $column_htmltag_front, "<U>$column</U>", $column_htmltag_back, ' ' );

		$column_number++;
	    }
	    $table_contents .= "</B>\n\n";

	} # foreach table_header

	# Align each column in each row of table:

	foreach $row ( split /\n/, $table_data ) {
	    $column_number  = 0;
	    foreach $column ( split /\t/, $row ) {

		# Set HTML tags aside for a moment:

		( $column_htmltag_front, $column_text, $column_htmltag_back ) = ();
		( $column_htmltag_front, $column_text, $column_htmltag_back ) = ( $1, $2, $3 ) if $column =~ /(<[^>]+>)([^<]+)(<[^>]+>)/;
		$column = $column_text if $column_text;
		$column =~ s/\&nbsp\;/ /;

		$column_length = length ( $column );

		# Truncate and add ellipsis to column text that is too long

		if ( $max_column_width > 5 ) {
		    if ( $column_length >= $max_column_width ) {
			$column = substr( $column, 0, ( $max_column_width - length($ellipsis) ) );
			$column .= $ellipsis;
			$column_length = $max_column_width;
		    } 
		} # if max_column_width

		# Pad column text to uniform column width

		for ( $column_length..$column_widths[$column_number] ) {
		    $column_htmltag_back .= ' ';
		}

		$table_contents .= join '', ( ' ', $column_htmltag_front, $column, $column_htmltag_back, '|' );

		$column_number++;

	    } # foreach column

	    $table_contents .= "\n";

	} # foreach row

	$table_contents .= "</PRE>\n";

	return 1;



    # Format table as HTML:

    } else {

	$row_number = 0;

	$table_contents  = '<TABLE BGCOLOR=WHITE WIDTH="90%" BORDER=0 CELLPADDING=3 CELLSPACING=1>';

	if ( $table_title ) {
	    $table_contents .= qq|<TH COLSPAN=10 ALIGN=LEFT> &nbsp; $table_title: </TH>\n|;
	}

	if ( @table_headers ) {
	    $table_contents .= "<TR BGCOLOR=#FFDD99><TD><B>$fontspec";
	    $table_contents .= join ": $fontclose</B></TD><TD ALIGN=MIDDLE><B>$fontspec", @table_headers;
	    $table_contents .= "</B></TD></TR>\n";
	}
	    

	foreach $row ( split /\n/, $table_data ) {
	    
	    $row_number++;
	    $bgcolor = '';
	    $bgcolor = 'BGCOLOR=#DDFFCC' if ( $row_number % 2 ) == 1;  # highlight every other row

	    $table_contents .= "<TR $bgcolor>";

	    $column_number = 0;
	    foreach $column ( split /\t/, $row ) {

		$column ||= '&nbsp;';

		# Align numeric data columns to the RIGHT and alpha columns LEFT

		if ( $column =~ /^[\d\.\,\%\sNA\/]+$/ ) {
		    $data_alignment = 'RIGHT';
		    $column = &addCommas( $column ); 
		} else {
		     $data_alignment = 'LEFT';
		}

		$alignment = ( $column_number++ < 1 ) ? 'LEFT' : $data_alignment;
		$table_contents .= qq|<TD $bgcolor ALIGN="$alignment" NOWRAP>$fontspec $column $fontclose</TD>|;

	    } # foreach column

	    $table_contents .= "</TR>\n";

	} # foreach row

	$table_contents .= "</TABLE>\n<BR><BR>\n";

	return 1;

    } # if table_format HTML

    return 0;

} # sub makeTable








# Given a (global) drop_list of items and a (local) selected item, make an HTML form drop list

sub makeDropList {

    my $drop_list_id  = shift;
    my $selected_item = shift;
    my $tmp_key       = '';
    my $selected      = '';

    $drop_list{ $selected_item } ||= $selected_item;

    $page_contents .= "<SELECT NAME=$drop_list_id><OPTION> </OPTION> ";
    foreach $tmp_key ( sort keys %drop_list ) {
	$selected = '';
	$selected = 'SELECTED' if $tmp_key eq $selected_item;
	$page_contents .= qq|<OPTION VALUE="$tmp_key" $selected> |; 
	$page_contents .= $drop_list{ $tmp_key };
	$page_contents .= " </OPTION>\n";
    }
    $page_contents .= "</SELECT> ";

} # sub makeDropList



# Given a global drop_list of items and a (local) selected item, make an HTML form list of radio buttons

sub makeRadioList {

    my $radio_list_id  = shift;
    my $selected_item  = shift;
    my $sep            = shift;
    my $tmp_key        = '';
    my $selected       = '';
       $sep          ||= '<BR>';

    $drop_list{ $selected_item } ||= $selected_item;

    foreach $tmp_key ( sort keys %drop_list ) {
	$selected = '';
	$selected = 'CHECKED' if $tmp_key eq $selected_item;
	$page_contents .= qq|<INPUT TYPE=RADIO NAME="$radio_list_id" VALUE="$tmp_key" $selected> |; 
	$page_contents .= $drop_list{ $tmp_key };
	$page_contents .= " $sep\n";
    }

} # sub makeRadioList




# Given a global drop_list of items and a (local) selected item, make an HTML form group of checkboxes
#
#     Inputs:   $checkbox_id    (local)   Basename for the checkbox inputs
#               $selected_items (local)   Comma or tab delimited list of selected (checked) values
#
#     Outputs:  $page_contents  (global)  Checkboxes HTML appended to (global) page_contents

sub makeCheckboxes {

    my $checkbox_id       = shift;
    my $selected_items    = shift;
    my $number_of_columns = 4;
    my $checkbox_number   = 0;
    my $tmp_key           = '';
    my $selected          = '';

    # Make sure selected items are in list of choices:

    foreach $tmp_key ( split /[,\t]/, $selected_items ) {
	$drop_list{ $tmp_key } ||= $tmp_key;
    }

    $page_contents .= "<TABLE><TR>\n";
    foreach $tmp_key ( sort keys %drop_list ) {
	$selected = '';
	$selected = 'CHECKED' if (( $selected_items ) && ( $selected_items =~ /\b$tmp_key\b/ ));
	$checkbox_number++;
	$page_contents .= qq|<TD><INPUT TYPE=CHECKBOX NAME="$checkbox_id\_$checkbox_number" VALUE="$tmp_key" $selected>|; 
	$page_contents .= " $drop_list{$tmp_key} </TD>\n";
	$page_contents .= "</TR>\n<TR>" if ($checkbox_number % $number_of_columns) == 0;
    }
    $page_contents .= "</TR></TABLE>\n";

} # sub makeCheckboxes


# sub readCheckboxes: Reads a checkbox group of form inputs into a comma-delimited list 
#

sub readCheckboxes {

    my $checkbox_id     = shift;
    my $numcheckboxes   = shift;
    my $checkbox_number = 0;
    my @checkedlist     = ();
    
    for $checkbox_number ( 1..$numcheckboxes ) {
	if ( $form->{'in'}->{ "$checkbox_id\_$checkbox_number" } =~ /(\w+)/ ) {
	    push @checkedlist, $1;
	    # print "DEBUG: readCheckboxes: $checkbox_id [$checkbox_number] = $1 <BR>\n";
	}
    }
    return ( join ',', @checkedlist );

} # sub readCheckBoxes





# sub makeRandomPassword: generates a random 8 character password

sub makeRandomPassword {

    my @goodchars = ();
    push @goodchars, (0..9);
    push @goodchars, (a..z);
    push @goodchars, ('-');
    push @goodchars, (A..Z);

    my ( $char, $plaintext, $charnum, $i ) = ();

    while ( $i++ < 8 ) {
	$charnum = int(rand($#goodchars + 1));
	$salt += $charnum ^ $i * $i * $charnum ^ $i;
	$char = $goodchars[$charnum];
	$plaintext .= $char;
    }

    return $plaintext;

} # sub makeRandomPassword


# End of lib

1;



